@@ -2,20 +2,23 @@ package cluster_test
 
 import (
 	"fmt"
+	"reflect"
 	"sync"
 	"sync/atomic"
 	"testing"
 	"time"
 
-	"github.com/influxdb/influxdb"
-	"github.com/influxdb/influxdb/cluster"
-	"github.com/influxdb/influxdb/models"
-	"github.com/influxdb/influxdb/services/meta"
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/cluster"
+	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/services/meta"
 )
 
+// TODO(benbjohnson): Rewrite tests to use cluster_test.MetaClient.
+
 // Ensures the points writer maps a single point to a single shard.
 func TestPointsWriter_MapShards_One(t *testing.T) {
-	ms := MetaClient{}
+	ms := PointsWriterMetaClient{}
 	rp := NewRetentionPolicy("myp", time.Hour, 3)
 
 	ms.NodeIDFn = func() uint64 { return 1 }
@@ -29,9 +32,8 @@ func TestPointsWriter_MapShards_One(t *testing.T) {
 
 	c := cluster.PointsWriter{MetaClient: ms}
 	pr := &cluster.WritePointsRequest{
-		Database:         "mydb",
-		RetentionPolicy:  "myrp",
-		ConsistencyLevel: cluster.ConsistencyLevelOne,
+		Database:        "mydb",
+		RetentionPolicy: "myrp",
 	}
 	pr.AddPoint("cpu", 1.0, time.Now(), nil)
 
@@ -50,7 +52,7 @@ func TestPointsWriter_MapShards_One(t *testing.T) {
 
 // Ensures the points writer maps a multiple points across shard group boundaries.
 func TestPointsWriter_MapShards_Multiple(t *testing.T) {
-	ms := MetaClient{}
+	ms := PointsWriterMetaClient{}
 	rp := NewRetentionPolicy("myp", time.Hour, 3)
 	AttachShardGroupInfo(rp, []meta.ShardOwner{
 		{NodeID: 1},
@@ -79,9 +81,8 @@ func TestPointsWriter_MapShards_Multiple(t *testing.T) {
 
 	c := cluster.PointsWriter{MetaClient: ms}
 	pr := &cluster.WritePointsRequest{
-		Database:         "mydb",
-		RetentionPolicy:  "myrp",
-		ConsistencyLevel: cluster.ConsistencyLevelOne,
+		Database:        "mydb",
+		RetentionPolicy: "myrp",
 	}
 
 	// Three points that range over the shardGroup duration (1h) and should map to two
@@ -124,117 +125,24 @@ func TestPointsWriter_WritePoints(t *testing.T) {
 		name            string
 		database        string
 		retentionPolicy string
-		consistency     cluster.ConsistencyLevel
 
 		// the responses returned by each shard write call.  node ID 1 = pos 0
 		err    []error
 		expErr error
 	}{
-		// Consistency one
 		{
 			name:            "write one success",
 			database:        "mydb",
 			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelOne,
 			err:             []error{nil, nil, nil},
 			expErr:          nil,
 		},
-		{
-			name:            "write one error",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelOne,
-			err:             []error{fmt.Errorf("a failure"), fmt.Errorf("a failure"), fmt.Errorf("a failure")},
-			expErr:          fmt.Errorf("write failed: a failure"),
-		},
-
-		// Consistency any
-		{
-			name:            "write any success",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelAny,
-			err:             []error{fmt.Errorf("a failure"), nil, fmt.Errorf("a failure")},
-			expErr:          nil,
-		},
-		// Consistency all
-		{
-			name:            "write all success",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelAll,
-			err:             []error{nil, nil, nil},
-			expErr:          nil,
-		},
-		{
-			name:            "write all, 2/3, partial write",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelAll,
-			err:             []error{nil, fmt.Errorf("a failure"), nil},
-			expErr:          cluster.ErrPartialWrite,
-		},
-		{
-			name:            "write all, 1/3 (failure)",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelAll,
-			err:             []error{nil, fmt.Errorf("a failure"), fmt.Errorf("a failure")},
-			expErr:          cluster.ErrPartialWrite,
-		},
-
-		// Consistency quorum
-		{
-			name:            "write quorum, 1/3 failure",
-			consistency:     cluster.ConsistencyLevelQuorum,
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			err:             []error{fmt.Errorf("a failure"), fmt.Errorf("a failure"), nil},
-			expErr:          cluster.ErrPartialWrite,
-		},
-		{
-			name:            "write quorum, 2/3 success",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelQuorum,
-			err:             []error{nil, nil, fmt.Errorf("a failure")},
-			expErr:          nil,
-		},
-		{
-			name:            "write quorum, 3/3 success",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelQuorum,
-			err:             []error{nil, nil, nil},
-			expErr:          nil,
-		},
-
-		// Error write error
-		{
-			name:            "no writes succeed",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelOne,
-			err:             []error{fmt.Errorf("a failure"), fmt.Errorf("a failure"), fmt.Errorf("a failure")},
-			expErr:          fmt.Errorf("write failed: a failure"),
-		},
-
-		// Hinted handoff w/ ANY
-		{
-			name:            "hinted handoff write succeed",
-			database:        "mydb",
-			retentionPolicy: "myrp",
-			consistency:     cluster.ConsistencyLevelAny,
-			err:             []error{fmt.Errorf("a failure"), fmt.Errorf("a failure"), fmt.Errorf("a failure")},
-			expErr:          nil,
-		},
 
 		// Write to non-existent database
 		{
 			name:            "write to non-existent database",
 			database:        "doesnt_exist",
 			retentionPolicy: "",
-			consistency:     cluster.ConsistencyLevelAny,
 			err:             []error{nil, nil, nil},
 			expErr:          fmt.Errorf("database not found: doesnt_exist"),
 		},
@@ -243,9 +151,8 @@ func TestPointsWriter_WritePoints(t *testing.T) {
 	for _, test := range tests {
 
 		pr := &cluster.WritePointsRequest{
-			Database:         test.database,
-			RetentionPolicy:  test.retentionPolicy,
-			ConsistencyLevel: test.consistency,
+			Database:        test.database,
+			RetentionPolicy: test.retentionPolicy,
 		}
 
 		// Three points that range over the shardGroup duration (1h) and should map to two
@@ -304,9 +211,9 @@ func TestPointsWriter_WritePoints(t *testing.T) {
 			},
 		}
 
-		ms := NewMetaClient()
-		ms.DatabaseFn = func(database string) (*meta.DatabaseInfo, error) {
-			return nil, nil
+		ms := NewPointsWriterMetaClient()
+		ms.DatabaseFn = func(database string) *meta.DatabaseInfo {
+			return nil
 		}
 		ms.NodeIDFn = func() uint64 { return 1 }
 
@@ -327,7 +234,7 @@ func TestPointsWriter_WritePoints(t *testing.T) {
 		c.Open()
 		defer c.Close()
 
-		err := c.WritePoints(pr)
+		err := c.WritePoints(pr.Database, pr.RetentionPolicy, models.ConsistencyLevelOne, pr.Points)
 		if err == nil && test.expErr != nil {
 			t.Errorf("PointsWriter.WritePoints(): '%s' error: got %v, exp %v", test.name, err, test.expErr)
 		}
@@ -341,7 +248,7 @@ func TestPointsWriter_WritePoints(t *testing.T) {
 		if test.expErr == nil {
 			select {
 			case p := <-subPoints:
-				if p != pr {
+				if !reflect.DeepEqual(p, pr) {
 					t.Errorf("PointsWriter.WritePoints(): '%s' error: unexpected WritePointsRequest got %v, exp %v", test.name, p, pr)
 				}
 			default:
@@ -374,8 +281,8 @@ func (f *fakeStore) CreateShard(database, retentionPolicy string, shardID uint64
 	return f.CreateShardfn(database, retentionPolicy, shardID)
 }
 
-func NewMetaClient() *MetaClient {
-	ms := &MetaClient{}
+func NewPointsWriterMetaClient() *PointsWriterMetaClient {
+	ms := &PointsWriterMetaClient{}
 	rp := NewRetentionPolicy("myp", time.Hour, 3)
 	AttachShardGroupInfo(rp, []meta.ShardOwner{
 		{NodeID: 1},
@@ -403,29 +310,29 @@ func NewMetaClient() *MetaClient {
 	return ms
 }
 
-type MetaClient struct {
+type PointsWriterMetaClient struct {
 	NodeIDFn                      func() uint64
 	RetentionPolicyFn             func(database, name string) (*meta.RetentionPolicyInfo, error)
 	CreateShardGroupIfNotExistsFn func(database, policy string, timestamp time.Time) (*meta.ShardGroupInfo, error)
-	DatabaseFn                    func(database string) (*meta.DatabaseInfo, error)
+	DatabaseFn                    func(database string) *meta.DatabaseInfo
 	ShardOwnerFn                  func(shardID uint64) (string, string, *meta.ShardGroupInfo)
 }
 
-func (m MetaClient) NodeID() uint64 { return m.NodeIDFn() }
+func (m PointsWriterMetaClient) NodeID() uint64 { return m.NodeIDFn() }
 
-func (m MetaClient) RetentionPolicy(database, name string) (*meta.RetentionPolicyInfo, error) {
+func (m PointsWriterMetaClient) RetentionPolicy(database, name string) (*meta.RetentionPolicyInfo, error) {
 	return m.RetentionPolicyFn(database, name)
 }
 
-func (m MetaClient) CreateShardGroup(database, policy string, timestamp time.Time) (*meta.ShardGroupInfo, error) {
+func (m PointsWriterMetaClient) CreateShardGroup(database, policy string, timestamp time.Time) (*meta.ShardGroupInfo, error) {
 	return m.CreateShardGroupIfNotExistsFn(database, policy, timestamp)
 }
 
-func (m MetaClient) Database(database string) (*meta.DatabaseInfo, error) {
+func (m PointsWriterMetaClient) Database(database string) *meta.DatabaseInfo {
 	return m.DatabaseFn(database)
 }
 
-func (m MetaClient) ShardOwner(shardID uint64) (string, string, *meta.ShardGroupInfo) {
+func (m PointsWriterMetaClient) ShardOwner(shardID uint64) (string, string, *meta.ShardGroupInfo) {
 	return m.ShardOwnerFn(shardID)
 }
 
