@@ -51,17 +51,29 @@ func MustParseStatement(s string) Statement {
 // ParseExpr parses an expression string and returns its AST representation.
 func ParseExpr(s string) (Expr, error) { return NewParser(strings.NewReader(s)).ParseExpr() }
 
+// MustParseExpr parses an expression string and returns its AST. Panic on error.
+func MustParseExpr(s string) Expr {
+	expr, err := ParseExpr(s)
+	if err != nil {
+		panic(err.Error())
+	}
+	return expr
+}
+
 // ParseQuery parses an InfluxQL string and returns a Query AST object.
 func (p *Parser) ParseQuery() (*Query, error) {
 	var statements Statements
-	var semi bool
+	semi := true
 
 	for {
-		if tok, _, _ := p.scanIgnoreWhitespace(); tok == EOF {
+		if tok, pos, lit := p.scanIgnoreWhitespace(); tok == EOF {
 			return &Query{Statements: statements}, nil
-		} else if !semi && tok == SEMICOLON {
+		} else if tok == SEMICOLON {
 			semi = true
 		} else {
+			if !semi {
+				return nil, newParseError(tokstr(tok, lit), []string{";"}, pos)
+			}
 			p.unscan()
 			s, err := p.ParseStatement()
 			if err != nil {
@@ -96,8 +108,10 @@ func (p *Parser) ParseStatement() (Statement, error) {
 		return p.parseAlterStatement()
 	case SET:
 		return p.parseSetPasswordUserStatement()
+	case KILL:
+		return p.parseKillQueryStatement()
 	default:
-		return nil, newParseError(tokstr(tok, lit), []string{"SELECT", "DELETE", "SHOW", "CREATE", "DROP", "GRANT", "REVOKE", "ALTER", "SET"}, pos)
+		return nil, newParseError(tokstr(tok, lit), []string{"SELECT", "DELETE", "SHOW", "CREATE", "DROP", "GRANT", "REVOKE", "ALTER", "SET", "KILL"}, pos)
 	}
 }
 
@@ -112,8 +126,6 @@ func (p *Parser) parseShowStatement() (Statement, error) {
 		return p.parseGrantsForUserStatement()
 	case DATABASES:
 		return p.parseShowDatabasesStatement()
-	case SERVERS:
-		return p.parseShowServersStatement()
 	case FIELD:
 		tok, pos, lit := p.scanIgnoreWhitespace()
 		if tok == KEYS {
@@ -122,6 +134,8 @@ func (p *Parser) parseShowStatement() (Statement, error) {
 		return nil, newParseError(tokstr(tok, lit), []string{"KEYS"}, pos)
 	case MEASUREMENTS:
 		return p.parseShowMeasurementsStatement()
+	case QUERIES:
+		return p.parseShowQueriesStatement()
 	case RETENTION:
 		tok, pos, lit := p.scanIgnoreWhitespace()
 		if tok == POLICIES {
@@ -162,9 +176,9 @@ func (p *Parser) parseShowStatement() (Statement, error) {
 		"FIELD",
 		"GRANTS",
 		"MEASUREMENTS",
+		"QUERIES",
 		"RETENTION",
 		"SERIES",
-		"SERVERS",
 		"TAG",
 		"USERS",
 		"STATS",
@@ -205,28 +219,29 @@ func (p *Parser) parseCreateStatement() (Statement, error) {
 // This function assumes the DROP token has already been consumed.
 func (p *Parser) parseDropStatement() (Statement, error) {
 	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok == SERIES {
-		return p.parseDropSeriesStatement()
-	} else if tok == MEASUREMENT {
-		return p.parseDropMeasurementStatement()
-	} else if tok == CONTINUOUS {
+	switch tok {
+	case CONTINUOUS:
 		return p.parseDropContinuousQueryStatement()
-	} else if tok == DATABASE {
+	case DATABASE:
 		return p.parseDropDatabaseStatement()
-	} else if tok == RETENTION {
+	case MEASUREMENT:
+		return p.parseDropMeasurementStatement()
+	case RETENTION:
 		if tok, pos, lit := p.scanIgnoreWhitespace(); tok != POLICY {
 			return nil, newParseError(tokstr(tok, lit), []string{"POLICY"}, pos)
 		}
 		return p.parseDropRetentionPolicyStatement()
-	} else if tok == USER {
-		return p.parseDropUserStatement()
-	} else if tok == META || tok == DATA {
-		return p.parseDropServerStatement(tok)
-	} else if tok == SUBSCRIPTION {
+	case SERIES:
+		return p.parseDropSeriesStatement()
+	case SHARD:
+		return p.parseDropShardStatement()
+	case SUBSCRIPTION:
 		return p.parseDropSubscriptionStatement()
+	case USER:
+		return p.parseDropUserStatement()
+	default:
+		return nil, newParseError(tokstr(tok, lit), []string{"CONTINUOUS", "MEASUREMENT", "RETENTION", "SERIES", "SHARD", "SUBSCRIPTION", "USER"}, pos)
 	}
-
-	return nil, newParseError(tokstr(tok, lit), []string{"SERIES", "CONTINUOUS", "MEASUREMENT", "SERVER", "SUBSCRIPTION"}, pos)
 }
 
 // parseAlterStatement parses a string and returns an alter statement.
@@ -275,6 +290,20 @@ func (p *Parser) parseSetPasswordUserStatement() (*SetPasswordUserStatement, err
 	return stmt, nil
 }
 
+// parseKillQueryStatement parses a string and returns a kill statement.
+// This function assumes the KILL token has already been consumed.
+func (p *Parser) parseKillQueryStatement() (*KillQueryStatement, error) {
+	if err := p.parseTokens([]Token{QUERY}); err != nil {
+		return nil, err
+	}
+
+	qid, err := p.parseUInt64()
+	if err != nil {
+		return nil, err
+	}
+	return &KillQueryStatement{QueryID: qid}, nil
+}
+
 // parseCreateSubscriptionStatement parses a string and returns a CreatesubScriptionStatement.
 // This function assumes the "CREATE SUBSCRIPTION" tokens have already been consumed.
 func (p *Parser) parseCreateSubscriptionStatement() (*CreateSubscriptionStatement, error) {
@@ -355,8 +384,7 @@ func (p *Parser) parseCreateRetentionPolicyStatement() (*CreateRetentionPolicySt
 	stmt.Database = ident
 
 	// Parse required DURATION token.
-	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok != DURATION {
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != DURATION {
 		return nil, newParseError(tokstr(tok, lit), []string{"DURATION"}, pos)
 	}
 
@@ -368,7 +396,7 @@ func (p *Parser) parseCreateRetentionPolicyStatement() (*CreateRetentionPolicySt
 	stmt.Duration = d
 
 	// Parse required REPLICATION token.
-	if tok, pos, lit = p.scanIgnoreWhitespace(); tok != REPLICATION {
+	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != REPLICATION {
 		return nil, newParseError(tokstr(tok, lit), []string{"REPLICATION"}, pos)
 	}
 
@@ -379,11 +407,25 @@ func (p *Parser) parseCreateRetentionPolicyStatement() (*CreateRetentionPolicySt
 	}
 	stmt.Replication = n
 
+	// Parse optional SHARD token.
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == SHARD {
+		if tok, pos, lit := p.scanIgnoreWhitespace(); tok != DURATION {
+			return nil, newParseError(tokstr(tok, lit), []string{"DURATION"}, pos)
+		}
+		d, err := p.parseDuration()
+		if err != nil {
+			return nil, err
+		}
+		stmt.ShardGroupDuration = d
+	} else {
+		p.unscan()
+	}
+
 	// Parse optional DEFAULT token.
-	if tok, pos, lit = p.scanIgnoreWhitespace(); tok == DEFAULT {
+	if tok, _, _ := p.scanIgnoreWhitespace(); tok == DEFAULT {
 		stmt.Default = true
-	} else if tok != EOF && tok != SEMICOLON {
-		return nil, newParseError(tokstr(tok, lit), []string{"DEFAULT"}, pos)
+	} else {
+		p.unscan()
 	}
 
 	return stmt, nil
@@ -434,11 +476,22 @@ Loop:
 				return nil, err
 			}
 			stmt.Replication = &n
+		case SHARD:
+			tok, pos, lit := p.scanIgnoreWhitespace()
+			if tok == DURATION {
+				d, err := p.parseDuration()
+				if err != nil {
+					return nil, err
+				}
+				stmt.ShardGroupDuration = &d
+			} else {
+				return nil, newParseError(tokstr(tok, lit), []string{"DURATION"}, pos)
+			}
 		case DEFAULT:
 			stmt.Default = true
 		default:
 			if i < 1 {
-				return nil, newParseError(tokstr(tok, lit), []string{"DURATION", "RETENTION", "DEFAULT"}, pos)
+				return nil, newParseError(tokstr(tok, lit), []string{"DURATION", "RETENTION", "SHARD", "DEFAULT"}, pos)
 			}
 			p.unscan()
 			break Loop
@@ -451,13 +504,8 @@ Loop:
 // parseInt parses a string and returns an integer literal.
 func (p *Parser) parseInt(min, max int) (int, error) {
 	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok != NUMBER {
-		return 0, newParseError(tokstr(tok, lit), []string{"number"}, pos)
-	}
-
-	// Return an error if the number has a fractional part.
-	if strings.Contains(lit, ".") {
-		return 0, &ParseError{Message: "number must be an integer", Pos: pos}
+	if tok != INTEGER {
+		return 0, newParseError(tokstr(tok, lit), []string{"integer"}, pos)
 	}
 
 	// Convert string to int.
@@ -477,8 +525,8 @@ func (p *Parser) parseInt(min, max int) (int, error) {
 // parseUInt32 parses a string and returns a 32-bit unsigned integer literal.
 func (p *Parser) parseUInt32() (uint32, error) {
 	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok != NUMBER {
-		return 0, newParseError(tokstr(tok, lit), []string{"number"}, pos)
+	if tok != INTEGER {
+		return 0, newParseError(tokstr(tok, lit), []string{"integer"}, pos)
 	}
 
 	// Convert string to unsigned 32-bit integer
@@ -493,8 +541,8 @@ func (p *Parser) parseUInt32() (uint32, error) {
 // parseUInt64 parses a string and returns a 64-bit unsigned integer literal.
 func (p *Parser) parseUInt64() (uint64, error) {
 	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok != NUMBER {
-		return 0, newParseError(tokstr(tok, lit), []string{"number"}, pos)
+	if tok != INTEGER {
+		return 0, newParseError(tokstr(tok, lit), []string{"integer"}, pos)
 	}
 
 	// Convert string to unsigned 64-bit integer
@@ -510,7 +558,7 @@ func (p *Parser) parseUInt64() (uint64, error) {
 // This function assumes the DURATION token has already been consumed.
 func (p *Parser) parseDuration() (time.Duration, error) {
 	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok != DURATION_VAL && tok != INF {
+	if tok != DURATIONVAL && tok != INF {
 		return 0, newParseError(tokstr(tok, lit), []string{"duration"}, pos)
 	}
 
@@ -945,33 +993,34 @@ func (p *Parser) parseTarget(tr targetRequirement) (*Target, error) {
 	return t, nil
 }
 
-// parseDeleteStatement parses a delete string and returns a DeleteStatement.
+// parseDeleteStatement parses a string and returns a delete statement.
 // This function assumes the DELETE token has already been consumed.
-func (p *Parser) parseDeleteStatement() (*DeleteStatement, error) {
-	// TODO remove and do not skip test once we wire up DELETE FROM.
-	// See issues https://github.com/influxdb/influxdb/issues/1647
-	// and https://github.com/influxdb/influxdb/issues/4404
-	return nil, errors.New("DELETE FROM is currently not supported. Use DROP SERIES or DROP MEASUREMENT instead")
-	//stmt := &DeleteStatement{}
+func (p *Parser) parseDeleteStatement() (Statement, error) {
+	stmt := &DeleteSeriesStatement{}
+	var err error
 
-	//// Parse source
-	//if tok, pos, lit := p.scanIgnoreWhitespace(); tok != FROM {
-	//	return nil, newParseError(tokstr(tok, lit), []string{"FROM"}, pos)
-	//}
-	//source, err := p.parseSource()
-	//if err != nil {
-	//	return nil, err
-	//}
-	//stmt.Source = source
+	tok, pos, lit := p.scanIgnoreWhitespace()
 
-	//// Parse condition: "WHERE EXPR".
-	//condition, err := p.parseCondition()
-	//if err != nil {
-	//	return nil, err
-	//}
-	//stmt.Condition = condition
+	if tok == FROM {
+		// Parse source.
+		if stmt.Sources, err = p.parseSources(); err != nil {
+			return nil, err
+		}
+	} else {
+		p.unscan()
+	}
 
-	//return stmt, nil
+	// Parse condition: "WHERE EXPR".
+	if stmt.Condition, err = p.parseCondition(); err != nil {
+		return nil, err
+	}
+
+	// If they didn't provide a FROM or a WHERE, this query is invalid
+	if stmt.Condition == nil && stmt.Sources == nil {
+		return nil, newParseError(tokstr(tok, lit), []string{"FROM", "WHERE"}, pos)
+	}
+
+	return stmt, nil
 }
 
 // parseShowSeriesStatement parses a string and returns a ShowSeriesStatement.
@@ -1064,6 +1113,12 @@ func (p *Parser) parseShowMeasurementsStatement() (*ShowMeasurementsStatement, e
 	return stmt, nil
 }
 
+// parseShowQueriesStatement parses a string and returns a ShowQueriesStatement.
+// This function assumes the "SHOW QUERIES" tokens have been consumed.
+func (p *Parser) parseShowQueriesStatement() (*ShowQueriesStatement, error) {
+	return &ShowQueriesStatement{}, nil
+}
+
 // parseShowRetentionPoliciesStatement parses a string and returns a ShowRetentionPoliciesStatement.
 // This function assumes the "SHOW RETENTION POLICIES" tokens have been consumed.
 func (p *Parser) parseShowRetentionPoliciesStatement() (*ShowRetentionPoliciesStatement, error) {
@@ -1307,27 +1362,18 @@ func (p *Parser) parseDropSeriesStatement() (*DropSeriesStatement, error) {
 	return stmt, nil
 }
 
-// parseDropServerStatement parses a string and returns a DropServerStatement.
-// This function assumes the "DROP <META|DATA>" tokens have already been consumed.
-func (p *Parser) parseDropServerStatement(tok Token) (*DropServerStatement, error) {
-	// Parse the SERVER token
-	if tok, pos, lit := p.scanIgnoreWhitespace(); tok != SERVER {
-		return nil, newParseError(tokstr(tok, lit), []string{"SERVER"}, pos)
-	}
-
-	s := &DropServerStatement{}
+// parseDropShardStatement parses a string and returns a
+// DropShardStatement. This function assumes the "DROP SHARD" tokens
+// have already been consumed.
+func (p *Parser) parseDropShardStatement() (*DropShardStatement, error) {
 	var err error
+	stmt := &DropShardStatement{}
 
-	if tok == META {
-		s.Meta = true
-	}
-
-	// Parse the server's ID.
-	if s.NodeID, err = p.parseUInt64(); err != nil {
+	// Parse the ID of the shard to be dropped.
+	if stmt.ID, err = p.parseUInt64(); err != nil {
 		return nil, err
 	}
-
-	return s, nil
+	return stmt, nil
 }
 
 // parseShowContinuousQueriesStatement parses a string and returns a ShowContinuousQueriesStatement.
@@ -1343,13 +1389,6 @@ func (p *Parser) parseShowContinuousQueriesStatement() (*ShowContinuousQueriesSt
 	return stmt, nil
 }
 
-// parseShowServersStatement parses a string and returns a ShowServersStatement.
-// This function assumes the "SHOW SERVERS" tokens have already been consumed.
-func (p *Parser) parseShowServersStatement() (*ShowServersStatement, error) {
-	stmt := &ShowServersStatement{}
-	return stmt, nil
-}
-
 // parseGrantsForUserStatement parses a string and returns a ShowGrantsForUserStatement.
 // This function assumes the "SHOW GRANTS" tokens have already been consumed.
 func (p *Parser) parseGrantsForUserStatement() (*ShowGrantsForUserStatement, error) {
@@ -1476,10 +1515,10 @@ func (p *Parser) parseCreateDatabaseStatement() (*CreateDatabaseStatement, error
 
 	// Look for "WITH"
 	if tok, _, _ := p.scanIgnoreWhitespace(); tok == WITH {
-		// validate that at least one of DURATION, REPLICATION or NAME is provided
+		// validate that at least one of DURATION, NAME, REPLICATION or SHARD is provided
 		tok, pos, lit := p.scanIgnoreWhitespace()
-		if tok != DURATION && tok != REPLICATION && tok != NAME {
-			return nil, newParseError(tokstr(tok, lit), []string{"DURATION", "REPLICATION", "NAME"}, pos)
+		if tok != DURATION && tok != NAME && tok != REPLICATION && tok != SHARD {
+			return nil, newParseError(tokstr(tok, lit), []string{"DURATION", "NAME", "REPLICATION", "SHARD"}, pos)
 		}
 		// rewind
 		p.unscan()
@@ -1500,7 +1539,7 @@ func (p *Parser) parseCreateDatabaseStatement() (*CreateDatabaseStatement, error
 		stmt.RetentionPolicyDuration = rpDuration
 
 		// Look for "REPLICATION"
-		var rpReplication int = 1 // default is 1
+		var rpReplication = 1 // default is 1
 		if err := p.parseTokens([]Token{REPLICATION}); err != nil {
 			p.unscan()
 		} else {
@@ -1511,6 +1550,23 @@ func (p *Parser) parseCreateDatabaseStatement() (*CreateDatabaseStatement, error
 		}
 		stmt.RetentionPolicyReplication = rpReplication
 
+		// Look for "SHARD"
+		var rpShardGroupDuration time.Duration
+		if err := p.parseTokens([]Token{SHARD}); err != nil {
+			p.unscan()
+		} else {
+			// Look for "DURATION"
+			tok, pos, lit := p.scanIgnoreWhitespace()
+			if tok != DURATION {
+				return nil, newParseError(tokstr(tok, lit), []string{"DURATION"}, pos)
+			}
+			rpShardGroupDuration, err = p.parseDuration()
+			if err != nil {
+				return nil, err
+			}
+			stmt.RetentionPolicyShardGroupDuration = rpShardGroupDuration
+		}
+
 		// Look for "NAME"
 		var rpName string = "default" // default is default
 		if err := p.parseTokens([]Token{NAME}); err != nil {
@@ -1525,7 +1581,6 @@ func (p *Parser) parseCreateDatabaseStatement() (*CreateDatabaseStatement, error
 	} else {
 		p.unscan()
 	}
-
 	return stmt, nil
 }
 
@@ -2039,11 +2094,14 @@ func (p *Parser) parseFill() (FillOption, interface{}, error) {
 	case "previous":
 		return PreviousFill, nil, nil
 	default:
-		num, ok := lit.Args[0].(*NumberLiteral)
-		if !ok {
+		switch num := lit.Args[0].(type) {
+		case *IntegerLiteral:
+			return NumberFill, num.Val, nil
+		case *NumberLiteral:
+			return NumberFill, num.Val, nil
+		default:
 			return NullFill, nil, fmt.Errorf("expected number argument in fill()")
 		}
-		return NumberFill, num.Val, nil
 	}
 }
 
@@ -2058,19 +2116,12 @@ func (p *Parser) parseOptionalTokenAndInt(t Token) (int, error) {
 
 	// Scan the number.
 	tok, pos, lit := p.scanIgnoreWhitespace()
-	if tok != NUMBER {
-		return 0, newParseError(tokstr(tok, lit), []string{"number"}, pos)
-	}
-
-	// Return an error if the number has a fractional part.
-	if strings.Contains(lit, ".") {
-		msg := fmt.Sprintf("fractional parts not allowed in %s", t.String())
-		return 0, &ParseError{Message: msg, Pos: pos}
+	if tok != INTEGER {
+		return 0, newParseError(tokstr(tok, lit), []string{"integer"}, pos)
 	}
 
 	// Parse number.
 	n, _ := strconv.ParseInt(lit, 10, 64)
-
 	if n < 0 {
 		msg := fmt.Sprintf("%s must be >= 0", t.String())
 		return 0, &ParseError{Message: msg, Pos: pos}
@@ -2320,9 +2371,15 @@ func (p *Parser) parseUnaryExpr() (Expr, error) {
 			return nil, &ParseError{Message: "unable to parse number", Pos: pos}
 		}
 		return &NumberLiteral{Val: v}, nil
+	case INTEGER:
+		v, err := strconv.ParseInt(lit, 10, 64)
+		if err != nil {
+			return nil, &ParseError{Message: "unable to parse integer", Pos: pos}
+		}
+		return &IntegerLiteral{Val: v}, nil
 	case TRUE, FALSE:
 		return &BooleanLiteral{Val: (tok == TRUE)}, nil
-	case DURATION_VAL:
+	case DURATIONVAL:
 		v, _ := ParseDuration(lit)
 		return &DurationLiteral{Val: v}, nil
 	case MUL:
@@ -2413,7 +2470,7 @@ func (p *Parser) parseResample() (time.Duration, time.Duration, error) {
 	var interval time.Duration
 	if p.parseTokenMaybe(EVERY) {
 		tok, pos, lit := p.scanIgnoreWhitespace()
-		if tok != DURATION_VAL {
+		if tok != DURATIONVAL {
 			return 0, 0, newParseError(tokstr(tok, lit), []string{"duration"}, pos)
 		}
 
@@ -2427,7 +2484,7 @@ func (p *Parser) parseResample() (time.Duration, time.Duration, error) {
 	var maxDuration time.Duration
 	if p.parseTokenMaybe(FOR) {
 		tok, pos, lit := p.scanIgnoreWhitespace()
-		if tok != DURATION_VAL {
+		if tok != DURATIONVAL {
 			return 0, 0, newParseError(tokstr(tok, lit), []string{"duration"}, pos)
 		}
 
