@@ -1,8 +1,9 @@
-package monitor
+package monitor // import "github.com/influxdata/influxdb/monitor"
 
 import (
 	"expvar"
 	"fmt"
+	"io"
 	"log"
 	"os"
 	"runtime"
@@ -11,10 +12,10 @@ import (
 	"sync"
 	"time"
 
-	"github.com/influxdb/influxdb"
-	"github.com/influxdb/influxdb/cluster"
-	"github.com/influxdb/influxdb/models"
-	"github.com/influxdb/influxdb/services/meta"
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/monitor/diagnostics"
+	"github.com/influxdata/influxdb/services/meta"
 )
 
 const leaderWaitTimeout = 30 * time.Second
@@ -25,46 +26,6 @@ const (
 	MonitorRetentionPolicyDuration = 7 * 24 * time.Hour
 )
 
-// DiagsClient is the interface modules implement if they register diags with monitor.
-type DiagsClient interface {
-	Diagnostics() (*Diagnostic, error)
-}
-
-// The DiagsClientFunc type is an adapter to allow the use of
-// ordinary functions as Diagnostis clients.
-type DiagsClientFunc func() (*Diagnostic, error)
-
-// Diagnostics calls f().
-func (f DiagsClientFunc) Diagnostics() (*Diagnostic, error) {
-	return f()
-}
-
-// Diagnostic represents a table of diagnostic information. The first value
-// is the name of the columns, the second is a slice of interface slices containing
-// the values for each column, by row. This information is never written to an InfluxDB
-// system and is display-only. An example showing, say, connections follows:
-//
-//     source_ip    source_port       dest_ip     dest_port
-//     182.1.0.2    2890              127.0.0.1   38901
-//     174.33.1.2   2924              127.0.0.1   38902
-type Diagnostic struct {
-	Columns []string
-	Rows    [][]interface{}
-}
-
-// NewDiagnostic initialises a new Diagnostic with the specified columns.
-func NewDiagnostic(columns []string) *Diagnostic {
-	return &Diagnostic{
-		Columns: columns,
-		Rows:    make([][]interface{}, 0),
-	}
-}
-
-// AddRow appends the provided row to the Diagnostic's rows.
-func (d *Diagnostic) AddRow(r []interface{}) {
-	d.Rows = append(d.Rows, r)
-}
-
 // Monitor represents an instance of the monitor system.
 type Monitor struct {
 	// Build information for diagnostics.
@@ -77,7 +38,7 @@ type Monitor struct {
 	done chan struct{}
 	mu   sync.Mutex
 
-	diagRegistrations map[string]DiagsClient
+	diagRegistrations map[string]diagnostics.Client
 
 	storeCreated           bool
 	storeEnabled           bool
@@ -98,8 +59,11 @@ type Monitor struct {
 
 	NodeID uint64
 
+	// Writer for pushing stats back into the database.
+	// This causes a circular dependency if it depends on cluster directly so it
+	// is wrapped in a simpler interface.
 	PointsWriter interface {
-		WritePoints(p *cluster.WritePointsRequest) error
+		WritePoints(database, retentionPolicy string, points models.Points) error
 	}
 
 	Logger *log.Logger
@@ -108,12 +72,13 @@ type Monitor struct {
 // New returns a new instance of the monitor system.
 func New(c Config) *Monitor {
 	return &Monitor{
-		done:              make(chan struct{}),
-		diagRegistrations: make(map[string]DiagsClient),
-		storeEnabled:      c.StoreEnabled,
-		storeDatabase:     c.StoreDatabase,
-		storeInterval:     time.Duration(c.StoreInterval),
-		Logger:            log.New(os.Stderr, "[monitor] ", log.LstdFlags),
+		done:                 make(chan struct{}),
+		diagRegistrations:    make(map[string]diagnostics.Client),
+		storeEnabled:         c.StoreEnabled,
+		storeDatabase:        c.StoreDatabase,
+		storeInterval:        time.Duration(c.StoreInterval),
+		storeRetentionPolicy: MonitorRetentionPolicy,
+		Logger:               log.New(os.Stderr, "[monitor] ", log.LstdFlags),
 	}
 }
 
@@ -121,6 +86,7 @@ func New(c Config) *Monitor {
 // for identification purpose.
 func (m *Monitor) Open() error {
 	m.Logger.Printf("Starting monitor system")
+	m.done = make(chan struct{})
 
 	// Self-register various stats and diagnostics.
 	m.RegisterDiagnosticsClient("build", &build{
@@ -135,7 +101,6 @@ func (m *Monitor) Open() error {
 
 	// If enabled, record stats in a InfluxDB system.
 	if m.storeEnabled {
-
 		// Start periodic writes to system.
 		m.wg.Add(1)
 		go m.storeStatistics()
@@ -148,17 +113,23 @@ func (m *Monitor) Open() error {
 func (m *Monitor) Close() {
 	m.Logger.Println("shutting down monitor system")
 	close(m.done)
+
 	m.wg.Wait()
 	m.done = nil
+	m.DeregisterDiagnosticsClient("build")
+	m.DeregisterDiagnosticsClient("runtime")
+	m.DeregisterDiagnosticsClient("network")
+	m.DeregisterDiagnosticsClient("system")
 }
 
-// SetLogger sets the internal logger to the logger passed in.
-func (m *Monitor) SetLogger(l *log.Logger) {
-	m.Logger = l
+// SetLogOutput sets the writer to which all logs are written. It must not be
+// called after Open is called.
+func (m *Monitor) SetLogOutput(w io.Writer) {
+	m.Logger = log.New(w, "[monitor] ", log.LstdFlags)
 }
 
 // RegisterDiagnosticsClient registers a diagnostics client with the given name and tags.
-func (m *Monitor) RegisterDiagnosticsClient(name string, client DiagsClient) {
+func (m *Monitor) RegisterDiagnosticsClient(name string, client diagnostics.Client) {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 	m.diagRegistrations[name] = client
@@ -287,11 +258,11 @@ func (m *Monitor) Statistics(tags map[string]string) ([]*Statistic, error) {
 // Diagnostics fetches diagnostic information for each registered
 // diagnostic client. It skips any clients that return an error when
 // retrieving their diagnostics.
-func (m *Monitor) Diagnostics() (map[string]*Diagnostic, error) {
+func (m *Monitor) Diagnostics() (map[string]*diagnostics.Diagnostics, error) {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
-	diags := make(map[string]*Diagnostic, len(m.diagRegistrations))
+	diags := make(map[string]*diagnostics.Diagnostics, len(m.diagRegistrations))
 	for k, v := range m.diagRegistrations {
 		d, err := v.Diagnostics()
 		if err != nil {
@@ -377,13 +348,7 @@ func (m *Monitor) storeStatistics() {
 				points = append(points, pt)
 			}
 
-			err = m.PointsWriter.WritePoints(&cluster.WritePointsRequest{
-				Database:         m.storeDatabase,
-				RetentionPolicy:  m.storeRetentionPolicy,
-				ConsistencyLevel: cluster.ConsistencyLevelOne,
-				Points:           points,
-			})
-			if err != nil {
+			if err := m.PointsWriter.WritePoints(m.storeDatabase, m.storeRetentionPolicy, points); err != nil {
 				m.Logger.Printf("failed to store statistics: %s", err)
 			}
 		case <-m.done:
@@ -411,7 +376,7 @@ func newStatistic(name string, tags map[string]string, values map[string]interfa
 }
 
 // valueNames returns a sorted list of the value names, if any.
-func (s *Statistic) valueNames() []string {
+func (s *Statistic) ValueNames() []string {
 	a := make([]string, 0, len(s.Values))
 	for k := range s.Values {
 		a = append(a, k)
@@ -420,8 +385,8 @@ func (s *Statistic) valueNames() []string {
 	return a
 }
 
-// DiagnosticFromMap returns a Diagnostic from a map.
-func DiagnosticFromMap(m map[string]interface{}) *Diagnostic {
+// DiagnosticsFromMap returns a Diagnostics from a map.
+func DiagnosticsFromMap(m map[string]interface{}) *diagnostics.Diagnostics {
 	// Display columns in deterministic order.
 	sortedKeys := make([]string, 0, len(m))
 	for k := range m {
@@ -429,7 +394,7 @@ func DiagnosticFromMap(m map[string]interface{}) *Diagnostic {
 	}
 	sort.Strings(sortedKeys)
 
-	d := NewDiagnostic(sortedKeys)
+	d := diagnostics.NewDiagnostics(sortedKeys)
 	row := make([]interface{}, len(sortedKeys))
 	for i, k := range sortedKeys {
 		row[i] = m[k]
