@@ -1,19 +1,19 @@
-package continuous_querier
+package continuous_querier // import "github.com/influxdata/influxdb/services/continuous_querier"
 
 import (
 	"errors"
 	"expvar"
 	"fmt"
+	"io"
 	"log"
 	"os"
 	"strings"
 	"sync"
 	"time"
 
-	"github.com/influxdb/influxdb"
-	"github.com/influxdb/influxdb/influxql"
-	"github.com/influxdb/influxdb/services/meta"
-	"github.com/influxdb/influxdb/tsdb"
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/influxql"
+	"github.com/influxdata/influxdb/services/meta"
 )
 
 const (
@@ -36,17 +36,11 @@ type ContinuousQuerier interface {
 	Run(database, name string, t time.Time) error
 }
 
-// queryExecutor is an internal interface to make testing easier.
-type queryExecutor interface {
-	ExecuteQuery(query *influxql.Query, database string, chunkSize int, closing chan struct{}) (<-chan *influxql.Result, error)
-}
-
 // metaClient is an internal interface to make testing easier.
 type metaClient interface {
 	AcquireLease(name string) (l *meta.Lease, err error)
-	Databases() ([]meta.DatabaseInfo, error)
-	Database(name string) (*meta.DatabaseInfo, error)
-	NodeID() uint64
+	Databases() []meta.DatabaseInfo
+	Database(name string) *meta.DatabaseInfo
 }
 
 // RunRequest is a request to run one or more CQs.
@@ -74,7 +68,7 @@ func (rr *RunRequest) matches(cq *meta.ContinuousQueryInfo) bool {
 // Service manages continuous query execution.
 type Service struct {
 	MetaClient    metaClient
-	QueryExecutor queryExecutor
+	QueryExecutor *influxql.QueryExecutor
 	Config        *Config
 	RunInterval   time.Duration
 	// RunCh can be used by clients to signal service to run CQs.
@@ -134,9 +128,10 @@ func (s *Service) Close() error {
 	return nil
 }
 
-// SetLogger sets the internal logger to the logger passed in.
-func (s *Service) SetLogger(l *log.Logger) {
-	s.Logger = l
+// SetLogOutput sets the writer to which all logs are written. It must not be
+// called after Open is called.
+func (s *Service) SetLogOutput(w io.Writer) {
+	s.Logger = log.New(w, "[continuous_querier] ", log.LstdFlags)
 }
 
 // Run runs the specified continuous query, or all CQs if none is specified.
@@ -145,20 +140,14 @@ func (s *Service) Run(database, name string, t time.Time) error {
 
 	if database != "" {
 		// Find the requested database.
-		db, err := s.MetaClient.Database(database)
-		if err != nil {
-			return err
-		} else if db == nil {
-			return tsdb.ErrDatabaseNotFound(database)
+		db := s.MetaClient.Database(database)
+		if db == nil {
+			return influxql.ErrDatabaseNotFound(database)
 		}
 		dbs = append(dbs, *db)
 	} else {
 		// Get all databases.
-		var err error
-		dbs, err = s.MetaClient.Databases()
-		if err != nil {
-			return err
-		}
+		dbs = s.MetaClient.Databases()
 	}
 
 	// Loop through databases.
@@ -169,8 +158,9 @@ func (s *Service) Run(database, name string, t time.Time) error {
 		for _, cq := range db.ContinuousQueries {
 			if name == "" || cq.Name == name {
 				// Remove the last run time for the CQ
-				if _, ok := s.lastRuns[cq.Name]; ok {
-					delete(s.lastRuns, cq.Name)
+				id := fmt.Sprintf("%s:%s", db.Name, cq.Name)
+				if _, ok := s.lastRuns[id]; ok {
+					delete(s.lastRuns, id)
 				}
 			}
 		}
@@ -213,11 +203,7 @@ func (s *Service) backgroundLoop() {
 // hasContinuousQueries returns true if any CQs exist.
 func (s *Service) hasContinuousQueries() bool {
 	// Get list of all databases.
-	dbs, err := s.MetaClient.Databases()
-	if err != nil {
-		s.Logger.Println("error getting databases")
-		return false
-	}
+	dbs := s.MetaClient.Databases()
 	// Loop through all databases executing CQs.
 	for _, db := range dbs {
 		if len(db.ContinuousQueries) > 0 {
@@ -230,11 +216,7 @@ func (s *Service) hasContinuousQueries() bool {
 // runContinuousQueries gets CQs from the meta store and runs them.
 func (s *Service) runContinuousQueries(req *RunRequest) {
 	// Get list of all databases.
-	dbs, err := s.MetaClient.Databases()
-	if err != nil {
-		s.Logger.Println("error getting databases")
-		return
-	}
+	dbs := s.MetaClient.Databases()
 	// Loop through all databases executing CQs.
 	for _, db := range dbs {
 		// TODO: distribute across nodes
@@ -266,7 +248,8 @@ func (s *Service) ExecuteContinuousQuery(dbi *meta.DatabaseInfo, cqi *meta.Conti
 	// Get the last time this CQ was run from the service's cache.
 	s.mu.Lock()
 	defer s.mu.Unlock()
-	cq.LastRun, cq.HasRun = s.lastRuns[cqi.Name]
+	id := fmt.Sprintf("%s:%s", dbi.Name, cqi.Name)
+	cq.LastRun, cq.HasRun = s.lastRuns[id]
 
 	// Set the retention policy to default if it wasn't specified in the query.
 	if cq.intoRP() == "" {
@@ -297,7 +280,7 @@ func (s *Service) ExecuteContinuousQuery(dbi *meta.DatabaseInfo, cqi *meta.Conti
 	// We're about to run the query so store the current time closest to the nearest interval.
 	// If all is going well, this time should be the same as nextRun.
 	cq.LastRun = now.Truncate(resampleEvery)
-	s.lastRuns[cqi.Name] = cq.LastRun
+	s.lastRuns[id] = cq.LastRun
 
 	// Retrieve the oldest interval we should calculate based on the next time
 	// interval. We do this instead of using the current time just in case any
@@ -351,10 +334,8 @@ func (s *Service) runContinuousQueryAndWriteResult(cq *ContinuousQuery) error {
 	defer close(closing)
 
 	// Execute the SELECT.
-	ch, err := s.QueryExecutor.ExecuteQuery(q, cq.Database, NoChunkingSize, closing)
-	if err != nil {
-		return err
-	}
+	ch := s.QueryExecutor.ExecuteQuery(q, cq.Database, NoChunkingSize, false, closing)
+
 	// There is only one statement, so we will only ever receive one result
 	res, ok := <-ch
 	if !ok {
