@@ -5,16 +5,15 @@ import (
 	"encoding/binary"
 	"io"
 	"io/ioutil"
-	"log"
 	"net"
 	"os"
 	"path/filepath"
 	"testing"
 
-	"github.com/influxdb/influxdb/services/copier"
-	"github.com/influxdb/influxdb/tcp"
-	"github.com/influxdb/influxdb/tsdb"
-	_ "github.com/influxdb/influxdb/tsdb/engine"
+	"github.com/influxdata/influxdb/services/copier"
+	"github.com/influxdata/influxdb/tcp"
+	"github.com/influxdata/influxdb/tsdb"
+	_ "github.com/influxdata/influxdb/tsdb/engine"
 )
 
 // Ensure the service can return shard data.
@@ -102,7 +101,7 @@ func NewService() *Service {
 	s.Service.TSDBStore = &s.TSDBStore
 
 	if !testing.Verbose() {
-		s.SetLogger(log.New(ioutil.Discard, "", 0))
+		s.SetLogOutput(ioutil.Discard)
 	}
 	return s
 }
@@ -163,7 +162,7 @@ func MustOpenShard(id uint64) *Shard {
 
 	sh := &Shard{
 		Shard: tsdb.NewShard(id,
-			tsdb.NewDatabaseIndex(),
+			tsdb.NewDatabaseIndex("db"),
 			filepath.Join(path, "data"),
 			filepath.Join(path, "wal"),
 			tsdb.NewEngineOptions(),
