@@ -1,9 +1,10 @@
-package graphite
+package graphite // import "github.com/influxdata/influxdb/services/graphite"
 
 import (
 	"bufio"
 	"expvar"
 	"fmt"
+	"io"
 	"log"
 	"math"
 	"net"
@@ -12,11 +13,11 @@ import (
 	"sync"
 	"time"
 
-	"github.com/influxdb/influxdb"
-	"github.com/influxdb/influxdb/cluster"
-	"github.com/influxdb/influxdb/monitor"
-	"github.com/influxdb/influxdb/services/meta"
-	"github.com/influxdb/influxdb/tsdb"
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/monitor/diagnostics"
+	"github.com/influxdata/influxdb/services/meta"
+	"github.com/influxdata/influxdb/tsdb"
 )
 
 const (
@@ -51,14 +52,13 @@ func (c *tcpConnection) Close() {
 type Service struct {
 	mu sync.Mutex
 
-	bindAddress      string
-	database         string
-	protocol         string
-	batchSize        int
-	batchPending     int
-	batchTimeout     time.Duration
-	consistencyLevel cluster.ConsistencyLevel
-	udpReadBuffer    int
+	bindAddress   string
+	database      string
+	protocol      string
+	batchSize     int
+	batchPending  int
+	batchTimeout  time.Duration
+	udpReadBuffer int
 
 	batcher *tsdb.PointBatcher
 	parser  *Parser
@@ -67,6 +67,7 @@ type Service struct {
 	statMap          *expvar.Map
 	tcpConnectionsMu sync.Mutex
 	tcpConnections   map[string]*tcpConnection
+	diagsKey         string
 
 	ln      net.Listener
 	addr    net.Addr
@@ -76,11 +77,11 @@ type Service struct {
 	done chan struct{}
 
 	Monitor interface {
-		RegisterDiagnosticsClient(name string, client monitor.DiagsClient)
+		RegisterDiagnosticsClient(name string, client diagnostics.Client)
 		DeregisterDiagnosticsClient(name string)
 	}
 	PointsWriter interface {
-		WritePoints(p *cluster.WritePointsRequest) error
+		WritePoints(database, retentionPolicy string, consistencyLevel models.ConsistencyLevel, points []models.Point) error
 	}
 	MetaClient interface {
 		CreateDatabase(name string) (*meta.DatabaseInfo, error)
@@ -103,14 +104,9 @@ func NewService(c Config) (*Service, error) {
 		logger:         log.New(os.Stderr, "[graphite] ", log.LstdFlags),
 		tcpConnections: make(map[string]*tcpConnection),
 		done:           make(chan struct{}),
+		diagsKey:       strings.Join([]string{"graphite", d.Protocol, d.BindAddress}, ":"),
 	}
 
-	consistencyLevel, err := cluster.ParseConsistencyLevel(d.ConsistencyLevel)
-	if err != nil {
-		return nil, err
-	}
-	s.consistencyLevel = consistencyLevel
-
 	parser, err := NewParserWithOptions(Options{
 		Templates:   d.Templates,
 		DefaultTags: d.DefaultTags(),
@@ -133,13 +129,12 @@ func (s *Service) Open() error {
 
 	// Configure expvar monitoring. It's OK to do this even if the service fails to open and
 	// should be done before any data could arrive for the service.
-	key := strings.Join([]string{"graphite", s.protocol, s.bindAddress}, ":")
 	tags := map[string]string{"proto": s.protocol, "bind": s.bindAddress}
-	s.statMap = influxdb.NewStatistics(key, "graphite", tags)
+	s.statMap = influxdb.NewStatistics(s.diagsKey, "graphite", tags)
 
 	// Register diagnostics if a Monitor service is available.
 	if s.Monitor != nil {
-		s.Monitor.RegisterDiagnosticsClient(key, s)
+		s.Monitor.RegisterDiagnosticsClient(s.diagsKey, s)
 	}
 
 	if _, err := s.MetaClient.CreateDatabase(s.database); err != nil {
@@ -194,6 +189,11 @@ func (s *Service) Close() error {
 	if s.batcher != nil {
 		s.batcher.Stop()
 	}
+
+	if s.Monitor != nil {
+		s.Monitor.DeregisterDiagnosticsClient(s.diagsKey)
+	}
+
 	close(s.done)
 	s.wg.Wait()
 	s.done = nil
@@ -201,9 +201,10 @@ func (s *Service) Close() error {
 	return nil
 }
 
-// SetLogger sets the internal logger to the logger passed in.
-func (s *Service) SetLogger(l *log.Logger) {
-	s.logger = l
+// SetLogOutput sets the writer to which all logs are written. It must not be
+// called after Open is called.
+func (s *Service) SetLogOutput(w io.Writer) {
+	s.logger = log.New(w, "[graphite] ", log.LstdFlags)
 }
 
 // Addr returns the address the Service binds to.
@@ -333,7 +334,7 @@ func (s *Service) handleLine(line string) {
 	point, err := s.parser.Parse(line)
 	if err != nil {
 		switch err := err.(type) {
-		case *UnsupposedValueError:
+		case *UnsupportedValueError:
 			// Graphite ignores NaN values with no error.
 			if math.IsNaN(err.Value) {
 				s.statMap.Add(statPointsNaNFail, 1)
@@ -354,12 +355,7 @@ func (s *Service) processBatches(batcher *tsdb.PointBatcher) {
 	for {
 		select {
 		case batch := <-batcher.Out():
-			if err := s.PointsWriter.WritePoints(&cluster.WritePointsRequest{
-				Database:         s.database,
-				RetentionPolicy:  "",
-				ConsistencyLevel: s.consistencyLevel,
-				Points:           batch,
-			}); err == nil {
+			if err := s.PointsWriter.WritePoints(s.database, "", models.ConsistencyLevelAny, batch); err == nil {
 				s.statMap.Add(statBatchesTransmitted, 1)
 				s.statMap.Add(statPointsTransmitted, int64(len(batch)))
 			} else {
@@ -374,16 +370,15 @@ func (s *Service) processBatches(batcher *tsdb.PointBatcher) {
 }
 
 // Diagnostics returns diagnostics of the graphite service.
-func (s *Service) Diagnostics() (*monitor.Diagnostic, error) {
+func (s *Service) Diagnostics() (*diagnostics.Diagnostics, error) {
 	s.tcpConnectionsMu.Lock()
 	defer s.tcpConnectionsMu.Unlock()
 
-	d := &monitor.Diagnostic{
+	d := &diagnostics.Diagnostics{
 		Columns: []string{"local", "remote", "connect time"},
 		Rows:    make([][]interface{}, 0, len(s.tcpConnections)),
 	}
 	for _, v := range s.tcpConnections {
-		_ = v
 		d.Rows = append(d.Rows, []interface{}{v.conn.LocalAddr().String(), v.conn.RemoteAddr().String(), v.connectTime})
 	}
 	return d, nil
