@@ -1,5 +1,8 @@
 package httpd
 
+// DefaultBindAddress is the default address to bind to.
+const DefaultBindAddress = ":8086"
+
 // Config represents a configuration for a HTTP service.
 type Config struct {
 	Enabled          bool   `toml:"enabled"`
@@ -10,6 +13,7 @@ type Config struct {
 	PprofEnabled     bool   `toml:"pprof-enabled"`
 	HTTPSEnabled     bool   `toml:"https-enabled"`
 	HTTPSCertificate string `toml:"https-certificate"`
+	MaxRowLimit      int    `toml:"max-row-limit"`
 }
 
 // NewConfig returns a new Config with default settings.
@@ -20,5 +24,6 @@ func NewConfig() Config {
 		LogEnabled:       true,
 		HTTPSEnabled:     false,
 		HTTPSCertificate: "/etc/ssl/influxdb.pem",
+		MaxRowLimit:      DefaultChunkSize,
 	}
 }
