@@ -1,9 +1,10 @@
-package httpd
+package httpd // import "github.com/influxdata/influxdb/services/httpd"
 
 import (
 	"crypto/tls"
 	"expvar"
 	"fmt"
+	"io"
 	"log"
 	"net"
 	"net/http"
@@ -11,22 +12,26 @@ import (
 	"strings"
 	"time"
 
-	"github.com/influxdb/influxdb"
+	"github.com/influxdata/influxdb"
 )
 
 // statistics gathered by the httpd package.
 const (
-	statRequest                      = "req"               // Number of HTTP requests served
-	statCQRequest                    = "cqReq"             // Number of CQ-execute requests served
-	statQueryRequest                 = "queryReq"          // Number of query requests served
-	statWriteRequest                 = "writeReq"          // Number of write requests serverd
-	statPingRequest                  = "pingReq"           // Number of ping requests served
-	statStatusRequest                = "statusReq"         // Number of status requests served
-	statWriteRequestBytesReceived    = "writeReqBytes"     // Sum of all bytes in write requests
-	statQueryRequestBytesTransmitted = "queryRespBytes"    // Sum of all bytes returned in query reponses
-	statPointsWrittenOK              = "pointsWrittenOK"   // Number of points written OK
-	statPointsWrittenFail            = "pointsWrittenFail" // Number of points that failed to be written
-	statAuthFail                     = "authFail"          // Number of authentication failures
+	statRequest                      = "req"                // Number of HTTP requests served
+	statCQRequest                    = "cqReq"              // Number of CQ-execute requests served
+	statQueryRequest                 = "queryReq"           // Number of query requests served
+	statWriteRequest                 = "writeReq"           // Number of write requests serverd
+	statPingRequest                  = "pingReq"            // Number of ping requests served
+	statStatusRequest                = "statusReq"          // Number of status requests served
+	statWriteRequestBytesReceived    = "writeReqBytes"      // Sum of all bytes in write requests
+	statQueryRequestBytesTransmitted = "queryRespBytes"     // Sum of all bytes returned in query reponses
+	statPointsWrittenOK              = "pointsWrittenOK"    // Number of points written OK
+	statPointsWrittenFail            = "pointsWrittenFail"  // Number of points that failed to be written
+	statAuthFail                     = "authFail"           // Number of authentication failures
+	statRequestDuration              = "reqDurationNs"      // Number of (wall-time) nanoseconds spent inside requests
+	statQueryRequestDuration         = "queryReqDurationNs" // Number of (wall-time) nanoseconds spent inside query requests
+	statWriteRequestDuration         = "writeReqDurationNs" // Number of (wall-time) nanoseconds spent inside write requests
+	statRequestsActive               = "reqActive"          // Number of currently active requests
 )
 
 // Service manages the listener and handler for an HTTP endpoint.
@@ -60,6 +65,7 @@ func NewService(c Config) *Service {
 			c.AuthEnabled,
 			c.LogEnabled,
 			c.WriteTracing,
+			c.MaxRowLimit,
 			statMap,
 		),
 		Logger: log.New(os.Stderr, "[httpd] ", log.LstdFlags),
@@ -125,9 +131,12 @@ func (s *Service) Close() error {
 	return nil
 }
 
-// SetLogger sets the internal logger to the logger passed in.
-func (s *Service) SetLogger(l *log.Logger) {
+// SetLogOutput sets the writer to which all logs are written. It must not be
+// called after Open is called.
+func (s *Service) SetLogOutput(w io.Writer) {
+	l := log.New(w, "[httpd] ", log.LstdFlags)
 	s.Logger = l
+	s.Handler.Logger = l
 }
 
 // Err returns a channel for fatal errors that occur on the listener.
