@@ -1,14 +1,17 @@
 package meta
 
 import (
+	"errors"
 	"fmt"
 	"sort"
+	"strings"
+	"sync"
 	"time"
 
 	"github.com/gogo/protobuf/proto"
-	"github.com/influxdb/influxdb"
-	"github.com/influxdb/influxdb/influxql"
-	"github.com/influxdb/influxdb/services/meta/internal"
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/influxql"
+	internal "github.com/influxdata/influxdb/services/meta/internal"
 )
 
 //go:generate protoc --gogo_out=. internal/meta.proto
@@ -29,178 +32,37 @@ type Data struct {
 	Term      uint64 // associated raft term
 	Index     uint64 // associated raft index
 	ClusterID uint64
-	MetaNodes []NodeInfo
-	DataNodes []NodeInfo
 	Databases []DatabaseInfo
 	Users     []UserInfo
 
-	MaxNodeID       uint64
 	MaxShardGroupID uint64
 	MaxShardID      uint64
 }
 
-// DataNode returns a node by id.
-func (data *Data) DataNode(id uint64) *NodeInfo {
-	for i := range data.DataNodes {
-		if data.DataNodes[i].ID == id {
-			return &data.DataNodes[i]
-		}
-	}
-	return nil
-}
+// NewShardOwner sets the owner of the provided shard to the data node
+// that currently owns the fewest number of shards. If multiple nodes
+// own the same (fewest) number of shards, then one of those nodes
+// becomes the new shard owner.
+func NewShardOwner(s ShardInfo, ownerFreqs map[int]int) (uint64, error) {
+	var (
+		minId   = -1
+		minFreq int
+	)
 
-// CreateDataNode adds a node to the metadata.
-func (data *Data) CreateDataNode(host, tcpHost string) error {
-	// Ensure a node with the same host doesn't already exist.
-	for _, n := range data.DataNodes {
-		if n.Host == host {
-			return ErrNodeExists
+	for id, freq := range ownerFreqs {
+		if minId == -1 || freq < minFreq {
+			minId, minFreq = int(id), freq
 		}
 	}
 
-	// Append new node.
-	data.MaxNodeID++
-	data.DataNodes = append(data.DataNodes, NodeInfo{
-		ID:      data.MaxNodeID,
-		Host:    host,
-		TCPHost: tcpHost,
-	})
-
-	return nil
-}
-
-// setDataNode adds a data node with a pre-specified nodeID.
-// this should only be used when the cluster is upgrading from 0.9 to 0.10
-func (data *Data) setDataNode(nodeID uint64, host, tcpHost string) error {
-	// Ensure a node with the same host doesn't already exist.
-	for _, n := range data.DataNodes {
-		if n.Host == host {
-			return ErrNodeExists
-		}
+	if minId < 0 {
+		return 0, fmt.Errorf("cannot reassign shard %d due to lack of data nodes", s.ID)
 	}
 
-	// Append new node.
-	data.DataNodes = append(data.DataNodes, NodeInfo{
-		ID:      nodeID,
-		Host:    host,
-		TCPHost: tcpHost,
-	})
-
-	return nil
-}
-
-// DeleteNode removes a node from the metadata.
-func (data *Data) DeleteDataNode(id uint64) error {
-	// Node has to be larger than 0 to be real
-	if id == 0 {
-		return ErrNodeIDRequired
-	}
-
-	// Remove node id from all shard infos
-	for di, d := range data.Databases {
-		for ri, rp := range d.RetentionPolicies {
-			for sgi, sg := range rp.ShardGroups {
-				for si, s := range sg.Shards {
-					if s.OwnedBy(id) {
-						var owners []ShardOwner
-						for _, o := range s.Owners {
-							if o.NodeID != id {
-								owners = append(owners, o)
-							}
-						}
-						data.Databases[di].RetentionPolicies[ri].ShardGroups[sgi].Shards[si].Owners = owners
-					}
-				}
-			}
-		}
-	}
-
-	// Remove this node from the in memory nodes
-	var nodes []NodeInfo
-	for _, n := range data.DataNodes {
-		if n.ID == id {
-			continue
-		}
-		nodes = append(nodes, n)
-	}
-
-	if len(nodes) == len(data.DataNodes) {
-		return ErrNodeNotFound
-	}
-
-	data.DataNodes = nodes
-	return nil
-}
-
-// MetaNode returns a node by id.
-func (data *Data) MetaNode(id uint64) *NodeInfo {
-	for i := range data.MetaNodes {
-		if data.MetaNodes[i].ID == id {
-			return &data.MetaNodes[i]
-		}
-	}
-	return nil
-}
-
-// CreateMetaNode will add a new meta node to the metastore
-func (data *Data) CreateMetaNode(httpAddr, tcpAddr string) error {
-	// Ensure a node with the same host doesn't already exist.
-	for _, n := range data.MetaNodes {
-		if n.Host == httpAddr {
-			return ErrNodeExists
-		}
-	}
-
-	// Append new node.
-	data.MaxNodeID++
-	data.MetaNodes = append(data.MetaNodes, NodeInfo{
-		ID:      data.MaxNodeID,
-		Host:    httpAddr,
-		TCPHost: tcpAddr,
-	})
-
-	return nil
-}
-
-// SetMetaNode will update the information for the single meta
-// node or create a new metanode. If there are more than 1 meta
-// nodes already, an error will be returned
-func (data *Data) SetMetaNode(httpAddr, tcpAddr string) error {
-	if len(data.MetaNodes) > 1 {
-		return fmt.Errorf("can't set meta node when there are more than 1 in the metastore")
-	}
-
-	if len(data.MetaNodes) == 0 {
-		return data.CreateMetaNode(httpAddr, tcpAddr)
-	}
-
-	data.MetaNodes[0].Host = httpAddr
-	data.MetaNodes[0].TCPHost = tcpAddr
-
-	return nil
-}
-
-// DeleteMetaNode will remove the meta node from the store
-func (data *Data) DeleteMetaNode(id uint64) error {
-	// Node has to be larger than 0 to be real
-	if id == 0 {
-		return ErrNodeIDRequired
-	}
-
-	var nodes []NodeInfo
-	for _, n := range data.MetaNodes {
-		if n.ID == id {
-			continue
-		}
-		nodes = append(nodes, n)
-	}
-
-	if len(nodes) == len(data.MetaNodes) {
-		return ErrNodeNotFound
-	}
-
-	data.MetaNodes = nodes
-	return nil
+	// Update the shard owner frequencies and set the new owner on the
+	// shard.
+	ownerFreqs[minId]++
+	return uint64(minId), nil
 }
 
 // Database returns a database by name.
@@ -240,15 +102,16 @@ func (data *Data) CreateDatabase(name string) error {
 	return nil
 }
 
-// DropDatabase removes a database by name.
+// DropDatabase removes a database by name. It does not return an error
+// if the database cannot be found.
 func (data *Data) DropDatabase(name string) error {
 	for i := range data.Databases {
 		if data.Databases[i].Name == name {
 			data.Databases = append(data.Databases[:i], data.Databases[i+1:]...)
-			return nil
+			break
 		}
 	}
-	return influxdb.ErrDatabaseNotFound(name)
+	return nil
 }
 
 // RetentionPolicy returns a retention policy for a database by name.
@@ -276,26 +139,30 @@ func (data *Data) CreateRetentionPolicy(database string, rpi *RetentionPolicyInf
 		return ErrReplicationFactorTooLow
 	}
 
+	// Normalise ShardDuration before comparing to any existing
+	// retention policies
+	rpi.ShardGroupDuration = normalisedShardDuration(rpi.ShardGroupDuration, rpi.Duration)
+
 	// Find database.
 	di := data.Database(database)
 	if di == nil {
 		return influxdb.ErrDatabaseNotFound(database)
 	} else if rp := di.RetentionPolicy(rpi.Name); rp != nil {
 		// RP with that name already exists.  Make sure they're the same.
-		if rp.ReplicaN != rpi.ReplicaN || rp.Duration != rpi.Duration {
+		if rp.ReplicaN != rpi.ReplicaN || rp.Duration != rpi.Duration || rp.ShardGroupDuration != rpi.ShardGroupDuration {
 			return ErrRetentionPolicyExists
 		}
 		return nil
 	}
 
-	// Append new policy.
-	di.RetentionPolicies = append(di.RetentionPolicies, RetentionPolicyInfo{
+	// Append copy of new policy.
+	rp := RetentionPolicyInfo{
 		Name:               rpi.Name,
 		Duration:           rpi.Duration,
-		ShardGroupDuration: shardGroupDuration(rpi.Duration),
 		ReplicaN:           rpi.ReplicaN,
-	})
-
+		ShardGroupDuration: rpi.ShardGroupDuration,
+	}
+	di.RetentionPolicies = append(di.RetentionPolicies, rp)
 	return nil
 }
 
@@ -304,7 +171,8 @@ func (data *Data) DropRetentionPolicy(database, name string) error {
 	// Find database.
 	di := data.Database(database)
 	if di == nil {
-		return influxdb.ErrDatabaseNotFound(database)
+		// no database? no problem
+		return nil
 	}
 
 	// Remove from list.
@@ -320,9 +188,10 @@ func (data *Data) DropRetentionPolicy(database, name string) error {
 
 // RetentionPolicyUpdate represents retention policy fields to be updated.
 type RetentionPolicyUpdate struct {
-	Name     *string
-	Duration *time.Duration
-	ReplicaN *int
+	Name               *string
+	Duration           *time.Duration
+	ReplicaN           *int
+	ShardGroupDuration *time.Duration
 }
 
 // SetName sets the RetentionPolicyUpdate.Name
@@ -334,6 +203,9 @@ func (rpu *RetentionPolicyUpdate) SetDuration(v time.Duration) { rpu.Duration =
 // SetReplicaN sets the RetentionPolicyUpdate.ReplicaN
 func (rpu *RetentionPolicyUpdate) SetReplicaN(v int) { rpu.ReplicaN = &v }
 
+// SetShardGroupDuration sets the RetentionPolicyUpdate.ShardGroupDuration
+func (rpu *RetentionPolicyUpdate) SetShardGroupDuration(v time.Duration) { rpu.ShardGroupDuration = &v }
+
 // UpdateRetentionPolicy updates an existing retention policy.
 func (data *Data) UpdateRetentionPolicy(database, name string, rpu *RetentionPolicyUpdate) error {
 	// Find database.
@@ -364,12 +236,17 @@ func (data *Data) UpdateRetentionPolicy(database, name string, rpu *RetentionPol
 	}
 	if rpu.Duration != nil {
 		rpi.Duration = *rpu.Duration
-		rpi.ShardGroupDuration = shardGroupDuration(rpi.Duration)
 	}
 	if rpu.ReplicaN != nil {
 		rpi.ReplicaN = *rpu.ReplicaN
 	}
 
+	if rpu.ShardGroupDuration != nil {
+		rpi.ShardGroupDuration = *rpu.ShardGroupDuration
+	} else {
+		rpi.ShardGroupDuration = shardGroupDuration(rpi.Duration)
+	}
+
 	return nil
 }
 
@@ -389,6 +266,38 @@ func (data *Data) SetDefaultRetentionPolicy(database, name string) error {
 	return nil
 }
 
+// DropShard removes a shard by ID.
+//
+// DropShard won't return an error if the shard can't be found, which
+// allows the command to be re-run in the case that the meta store
+// succeeds but a data node fails.
+func (data *Data) DropShard(id uint64) {
+	found := -1
+	for dbidx, dbi := range data.Databases {
+		for rpidx, rpi := range dbi.RetentionPolicies {
+			for sgidx, sg := range rpi.ShardGroups {
+				for sidx, s := range sg.Shards {
+					if s.ID == id {
+						found = sidx
+						break
+					}
+				}
+
+				if found > -1 {
+					shards := sg.Shards
+					data.Databases[dbidx].RetentionPolicies[rpidx].ShardGroups[sgidx].Shards = append(shards[:found], shards[found+1:]...)
+
+					if len(shards) == 1 {
+						// We just deleted the last shard in the shard group.
+						data.Databases[dbidx].RetentionPolicies[rpidx].ShardGroups[sgidx].DeletedAt = time.Now()
+					}
+					return
+				}
+			}
+		}
+	}
+}
+
 // ShardGroups returns a list of all shard groups on a database and policy.
 func (data *Data) ShardGroups(database, policy string) ([]ShardGroupInfo, error) {
 	// Find retention policy.
@@ -443,11 +352,6 @@ func (data *Data) ShardGroupByTimestamp(database, policy string, timestamp time.
 
 // CreateShardGroup creates a shard group on a database and policy for a given timestamp.
 func (data *Data) CreateShardGroup(database, policy string, timestamp time.Time) error {
-	// Ensure there are nodes in the metadata.
-	if len(data.DataNodes) == 0 {
-		return nil
-	}
-
 	// Find retention policy.
 	rpi, err := data.RetentionPolicy(database, policy)
 	if err != nil {
@@ -461,19 +365,6 @@ func (data *Data) CreateShardGroup(database, policy string, timestamp time.Time)
 		return nil
 	}
 
-	// Require at least one replica but no more replicas than nodes.
-	replicaN := rpi.ReplicaN
-	if replicaN == 0 {
-		replicaN = 1
-	} else if replicaN > len(data.DataNodes) {
-		replicaN = len(data.DataNodes)
-	}
-
-	// Determine shard count by node count divided by replication factor.
-	// This will ensure nodes will get distributed across nodes evenly and
-	// replicated the correct number of times.
-	shardN := len(data.DataNodes) / replicaN
-
 	// Create the shard group.
 	data.MaxShardGroupID++
 	sgi := ShardGroupInfo{}
@@ -481,23 +372,9 @@ func (data *Data) CreateShardGroup(database, policy string, timestamp time.Time)
 	sgi.StartTime = timestamp.Truncate(rpi.ShardGroupDuration).UTC()
 	sgi.EndTime = sgi.StartTime.Add(rpi.ShardGroupDuration).UTC()
 
-	// Create shards on the group.
-	sgi.Shards = make([]ShardInfo, shardN)
-	for i := range sgi.Shards {
-		data.MaxShardID++
-		sgi.Shards[i] = ShardInfo{ID: data.MaxShardID}
-	}
-
-	// Assign data nodes to shards via round robin.
-	// Start from a repeatably "random" place in the node list.
-	nodeIndex := int(data.Index % uint64(len(data.DataNodes)))
-	for i := range sgi.Shards {
-		si := &sgi.Shards[i]
-		for j := 0; j < replicaN; j++ {
-			nodeID := data.DataNodes[nodeIndex%len(data.DataNodes)].ID
-			si.Owners = append(si.Owners, ShardOwner{NodeID: nodeID})
-			nodeIndex++
-		}
+	data.MaxShardID++
+	sgi.Shards = []ShardInfo{
+		{ID: data.MaxShardID},
 	}
 
 	// Retention policy has a new shard group, so update the policy. Shard
@@ -538,8 +415,14 @@ func (data *Data) CreateContinuousQuery(database, name, query string) error {
 	}
 
 	// Ensure the name doesn't already exist.
-	for i := range di.ContinuousQueries {
-		if di.ContinuousQueries[i].Name == name {
+	for _, cq := range di.ContinuousQueries {
+		if cq.Name == name {
+			// If the query string is the same, we'll silently return,
+			// otherwise we'll assume the user might be trying to
+			// overwrite an existing CQ with a different query.
+			if strings.ToLower(cq.Query) == strings.ToLower(query) {
+				return nil
+			}
 			return ErrContinuousQueryExists
 		}
 	}
@@ -574,8 +457,7 @@ func (data *Data) CreateSubscription(database, rp, name, mode string, destinatio
 	rpi, err := data.RetentionPolicy(database, rp)
 	if err != nil {
 		return err
-	}
-	if rpi == nil {
+	} else if rpi == nil {
 		return influxdb.ErrRetentionPolicyNotFound(rp)
 	}
 
@@ -601,6 +483,8 @@ func (data *Data) DropSubscription(database, rp, name string) error {
 	rpi, err := data.RetentionPolicy(database, rp)
 	if err != nil {
 		return err
+	} else if rpi == nil {
+		return influxdb.ErrRetentionPolicyNotFound(rp)
 	}
 
 	for i := range rpi.Subscriptions {
@@ -663,6 +547,19 @@ func (data *Data) UpdateUser(name, hash string) error {
 	return ErrUserNotFound
 }
 
+// CloneUsers returns a copy of the user infos
+func (data *Data) CloneUsers() []UserInfo {
+	if len(data.Users) == 0 {
+		return []UserInfo{}
+	}
+	users := make([]UserInfo, len(data.Users))
+	for i := range data.Users {
+		users[i] = data.Users[i].clone()
+	}
+
+	return users
+}
+
 // SetPrivilege sets a privilege for a user on a database.
 func (data *Data) SetPrivilege(name, database string, p influxql.Privilege) error {
 	ui := data.User(name)
@@ -720,36 +617,8 @@ func (data *Data) UserPrivilege(name, database string) (*influxql.Privilege, err
 func (data *Data) Clone() *Data {
 	other := *data
 
-	// Copy nodes.
-	if data.DataNodes != nil {
-		other.DataNodes = make([]NodeInfo, len(data.DataNodes))
-		for i := range data.DataNodes {
-			other.DataNodes[i] = data.DataNodes[i].clone()
-		}
-	}
-
-	if data.MetaNodes != nil {
-		other.MetaNodes = make([]NodeInfo, len(data.MetaNodes))
-		for i := range data.MetaNodes {
-			other.MetaNodes[i] = data.MetaNodes[i].clone()
-		}
-	}
-
-	// Deep copy databases.
-	if data.Databases != nil {
-		other.Databases = make([]DatabaseInfo, len(data.Databases))
-		for i := range data.Databases {
-			other.Databases[i] = data.Databases[i].clone()
-		}
-	}
-
-	// Copy users.
-	if data.Users != nil {
-		other.Users = make([]UserInfo, len(data.Users))
-		for i := range data.Users {
-			other.Users[i] = data.Users[i].clone()
-		}
-	}
+	other.Databases = data.CloneDatabases()
+	other.Users = data.CloneUsers()
 
 	return &other
 }
@@ -761,19 +630,11 @@ func (data *Data) marshal() *internal.Data {
 		Index:     proto.Uint64(data.Index),
 		ClusterID: proto.Uint64(data.ClusterID),
 
-		MaxNodeID:       proto.Uint64(data.MaxNodeID),
 		MaxShardGroupID: proto.Uint64(data.MaxShardGroupID),
 		MaxShardID:      proto.Uint64(data.MaxShardID),
-	}
 
-	pb.DataNodes = make([]*internal.NodeInfo, len(data.DataNodes))
-	for i := range data.DataNodes {
-		pb.DataNodes[i] = data.DataNodes[i].marshal()
-	}
-
-	pb.MetaNodes = make([]*internal.NodeInfo, len(data.MetaNodes))
-	for i := range data.MetaNodes {
-		pb.MetaNodes[i] = data.MetaNodes[i].marshal()
+		// Need this for reverse compatibility
+		MaxNodeID: proto.Uint64(0),
 	}
 
 	pb.Databases = make([]*internal.DatabaseInfo, len(data.Databases))
@@ -795,28 +656,9 @@ func (data *Data) unmarshal(pb *internal.Data) {
 	data.Index = pb.GetIndex()
 	data.ClusterID = pb.GetClusterID()
 
-	data.MaxNodeID = pb.GetMaxNodeID()
 	data.MaxShardGroupID = pb.GetMaxShardGroupID()
 	data.MaxShardID = pb.GetMaxShardID()
 
-	// TODO: Nodes is deprecated. This is being left here to make migration from 0.9.x to 0.10.0 possible
-	if len(pb.GetNodes()) > 0 {
-		data.DataNodes = make([]NodeInfo, len(pb.GetNodes()))
-		for i, x := range pb.GetNodes() {
-			data.DataNodes[i].unmarshal(x)
-		}
-	} else {
-		data.DataNodes = make([]NodeInfo, len(pb.GetDataNodes()))
-		for i, x := range pb.GetDataNodes() {
-			data.DataNodes[i].unmarshal(x)
-		}
-	}
-
-	data.MetaNodes = make([]NodeInfo, len(pb.GetMetaNodes()))
-	for i, x := range pb.GetMetaNodes() {
-		data.MetaNodes[i].unmarshal(x)
-	}
-
 	data.Databases = make([]DatabaseInfo, len(pb.GetDatabases()))
 	for i, x := range pb.GetDatabases() {
 		data.Databases[i].unmarshal(x)
@@ -1090,6 +932,21 @@ func (rpi RetentionPolicyInfo) clone() RetentionPolicyInfo {
 	return other
 }
 
+// MarshalBinary encodes rpi to a binary format.
+func (rpi *RetentionPolicyInfo) MarshalBinary() ([]byte, error) {
+	return proto.Marshal(rpi.marshal())
+}
+
+// UnmarshalBinary decodes rpi from a binary format.
+func (rpi *RetentionPolicyInfo) UnmarshalBinary(data []byte) error {
+	var pb internal.RetentionPolicyInfo
+	if err := proto.Unmarshal(data, &pb); err != nil {
+		return err
+	}
+	rpi.unmarshal(&pb)
+	return nil
+}
+
 // shardGroupDuration returns the duration for a shard group based on a policy duration.
 func shardGroupDuration(d time.Duration) time.Duration {
 	if d >= 180*24*time.Hour || d == 0 { // 6 months or 0
@@ -1100,6 +957,14 @@ func shardGroupDuration(d time.Duration) time.Duration {
 	return 1 * time.Hour
 }
 
+// normalisedShardDuration returns normalised shard duration based on a policy duration.
+func normalisedShardDuration(sgd, d time.Duration) time.Duration {
+	if sgd == 0 {
+		return shardGroupDuration(d)
+	}
+	return sgd
+}
+
 // ShardGroupInfo represents metadata about a shard group. The DeletedAt field is important
 // because it makes it clear that a ShardGroup has been marked as deleted, and allow the system
 // to be sure that a ShardGroup is not simply missing. If the DeletedAt is set, the system can
@@ -1112,7 +977,8 @@ type ShardGroupInfo struct {
 	Shards    []ShardInfo
 }
 
-// ShardGroupInfos is a collection of ShardGroupInfo
+// ShardGroupInfos implements sort.Interface on []ShardGroupInfo, based
+// on the StartTime field.
 type ShardGroupInfos []ShardGroupInfo
 
 func (a ShardGroupInfos) Len() int           { return len(a) }
@@ -1191,7 +1057,7 @@ type ShardInfo struct {
 	Owners []ShardOwner
 }
 
-// OwnedBy returns whether the shard's owner IDs includes nodeID.
+// OwnedBy determines whether the shard's owner IDs includes nodeID.
 func (si ShardInfo) OwnedBy(nodeID uint64) bool {
 	for _, so := range si.Owners {
 		if so.NodeID == nodeID {
@@ -1399,6 +1265,50 @@ func (ui *UserInfo) unmarshal(pb *internal.UserInfo) {
 	}
 }
 
+type Lease struct {
+	Name       string    `json:"name"`
+	Expiration time.Time `json:"expiration"`
+	Owner      uint64    `json:"owner"`
+}
+
+type Leases struct {
+	mu sync.Mutex
+	m  map[string]*Lease
+	d  time.Duration
+}
+
+func NewLeases(d time.Duration) *Leases {
+	return &Leases{
+		m: make(map[string]*Lease),
+		d: d,
+	}
+}
+
+func (leases *Leases) Acquire(name string, nodeID uint64) (*Lease, error) {
+	leases.mu.Lock()
+	defer leases.mu.Unlock()
+
+	l, ok := leases.m[name]
+	if ok {
+		if time.Now().After(l.Expiration) || l.Owner == nodeID {
+			l.Expiration = time.Now().Add(leases.d)
+			l.Owner = nodeID
+			return l, nil
+		}
+		return l, errors.New("another node has the lease")
+	}
+
+	l = &Lease{
+		Name:       name,
+		Expiration: time.Now().Add(leases.d),
+		Owner:      nodeID,
+	}
+
+	leases.m[name] = l
+
+	return l, nil
+}
+
 // MarshalTime converts t to nanoseconds since epoch. A zero time returns 0.
 func MarshalTime(t time.Time) int64 {
 	if t.IsZero() {
