@@ -1,4 +1,4 @@
-package opentsdb
+package opentsdb // import "github.com/influxdata/influxdb/services/opentsdb"
 
 import (
 	"bufio"
@@ -16,11 +16,10 @@ import (
 	"sync"
 	"time"
 
-	"github.com/influxdb/influxdb"
-	"github.com/influxdb/influxdb/cluster"
-	"github.com/influxdb/influxdb/models"
-	"github.com/influxdb/influxdb/services/meta"
-	"github.com/influxdb/influxdb/tsdb"
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/services/meta"
+	"github.com/influxdata/influxdb/tsdb"
 )
 
 const leaderWaitTimeout = 30 * time.Second
@@ -57,13 +56,12 @@ type Service struct {
 	tls  bool
 	cert string
 
-	BindAddress      string
-	Database         string
-	RetentionPolicy  string
-	ConsistencyLevel cluster.ConsistencyLevel
+	BindAddress     string
+	Database        string
+	RetentionPolicy string
 
 	PointsWriter interface {
-		WritePoints(p *cluster.WritePointsRequest) error
+		WritePoints(database, retentionPolicy string, consistencyLevel models.ConsistencyLevel, points []models.Point) error
 	}
 	MetaClient interface {
 		CreateDatabase(name string) (*meta.DatabaseInfo, error)
@@ -82,25 +80,22 @@ type Service struct {
 
 // NewService returns a new instance of Service.
 func NewService(c Config) (*Service, error) {
-	consistencyLevel, err := cluster.ParseConsistencyLevel(c.ConsistencyLevel)
-	if err != nil {
-		return nil, err
-	}
+	// Use defaults where necessary.
+	d := c.WithDefaults()
 
 	s := &Service{
-		done:             make(chan struct{}),
-		tls:              c.TLSEnabled,
-		cert:             c.Certificate,
-		err:              make(chan error),
-		BindAddress:      c.BindAddress,
-		Database:         c.Database,
-		RetentionPolicy:  c.RetentionPolicy,
-		ConsistencyLevel: consistencyLevel,
-		batchSize:        c.BatchSize,
-		batchPending:     c.BatchPending,
-		batchTimeout:     time.Duration(c.BatchTimeout),
-		Logger:           log.New(os.Stderr, "[opentsdb] ", log.LstdFlags),
-		LogPointErrors:   c.LogPointErrors,
+		done:            make(chan struct{}),
+		tls:             d.TLSEnabled,
+		cert:            d.Certificate,
+		err:             make(chan error),
+		BindAddress:     d.BindAddress,
+		Database:        d.Database,
+		RetentionPolicy: d.RetentionPolicy,
+		batchSize:       d.BatchSize,
+		batchPending:    d.BatchPending,
+		batchTimeout:    time.Duration(d.BatchTimeout),
+		Logger:          log.New(os.Stderr, "[opentsdb] ", log.LstdFlags),
+		LogPointErrors:  d.LogPointErrors,
 	}
 	return s, nil
 }
@@ -182,8 +177,11 @@ func (s *Service) Close() error {
 	return nil
 }
 
-// SetLogger sets the internal logger to the logger passed in.
-func (s *Service) SetLogger(l *log.Logger) { s.Logger = l }
+// SetLogOutput sets the writer to which all logs are written. It must not be
+// called after Open is called.
+func (s *Service) SetLogOutput(w io.Writer) {
+	s.Logger = log.New(w, "[opentsdb] ", log.LstdFlags)
+}
 
 // Err returns a channel for fatal errors that occur on the listener.
 func (s *Service) Err() <-chan error { return s.err }
@@ -357,12 +355,11 @@ func (s *Service) handleTelnetConn(conn net.Conn) {
 // serveHTTP handles connections in HTTP format.
 func (s *Service) serveHTTP() {
 	srv := &http.Server{Handler: &Handler{
-		Database:         s.Database,
-		RetentionPolicy:  s.RetentionPolicy,
-		ConsistencyLevel: s.ConsistencyLevel,
-		PointsWriter:     s.PointsWriter,
-		Logger:           s.Logger,
-		statMap:          s.statMap,
+		Database:        s.Database,
+		RetentionPolicy: s.RetentionPolicy,
+		PointsWriter:    s.PointsWriter,
+		Logger:          s.Logger,
+		statMap:         s.statMap,
 	}}
 	srv.Serve(s.httpln)
 }
@@ -373,12 +370,7 @@ func (s *Service) processBatches(batcher *tsdb.PointBatcher) {
 	for {
 		select {
 		case batch := <-batcher.Out():
-			if err := s.PointsWriter.WritePoints(&cluster.WritePointsRequest{
-				Database:         s.Database,
-				RetentionPolicy:  s.RetentionPolicy,
-				ConsistencyLevel: s.ConsistencyLevel,
-				Points:           batch,
-			}); err == nil {
+			if err := s.PointsWriter.WritePoints(s.Database, s.RetentionPolicy, models.ConsistencyLevelAny, batch); err == nil {
 				s.statMap.Add(statBatchesTrasmitted, 1)
 				s.statMap.Add(statPointsTransmitted, int64(len(batch)))
 			} else {
