@@ -9,13 +9,18 @@ import (
 	"sort"
 	"time"
 
-	"github.com/boltdb/bolt"
-	"github.com/influxdb/influxdb/models"
+	"github.com/influxdata/influxdb/influxql"
+	"github.com/influxdata/influxdb/models"
 )
 
 var (
 	// ErrFormatNotFound is returned when no format can be determined from a path.
 	ErrFormatNotFound = errors.New("format not found")
+
+	// ErrUnknownEngineFormat is returned when the engine format is
+	// unknown. ErrUnknownEngineFormat is currently returned if a format
+	// other than tsm1 is encountered.
+	ErrUnknownEngineFormat = errors.New("unknown engine format")
 )
 
 // Engine represents a swappable storage engine for the shard.
@@ -24,16 +29,17 @@ type Engine interface {
 	Close() error
 
 	SetLogOutput(io.Writer)
-	LoadMetadataIndex(shard *Shard, index *DatabaseIndex, measurementFields map[string]*MeasurementFields) error
+	LoadMetadataIndex(shardID uint64, index *DatabaseIndex) error
 
-	Begin(writable bool) (Tx, error)
-	WritePoints(points []models.Point, measurementFieldsToSave map[string]*MeasurementFields, seriesToCreate []*SeriesCreate) error
+	CreateIterator(opt influxql.IteratorOptions) (influxql.Iterator, error)
+	SeriesKeys(opt influxql.IteratorOptions) (influxql.SeriesList, error)
+	WritePoints(points []models.Point) error
+	ContainsSeries(keys []string) (map[string]bool, error)
 	DeleteSeries(keys []string) error
+	DeleteSeriesRange(keys []string, min, max int64) error
 	DeleteMeasurement(name string, seriesKeys []string) error
 	SeriesCount() (n int, err error)
-
-	// PerformMaintenance will get called periodically by the store
-	PerformMaintenance()
+	MeasurementFields(measurement string) *MeasurementFields
 
 	// Format will return the format for the engine
 	Format() EngineFormat
@@ -43,12 +49,12 @@ type Engine interface {
 	Backup(w io.Writer, basePath string, since time.Time) error
 }
 
+// EngineFormat represents the format for an engine.
 type EngineFormat int
 
 const (
-	B1Format EngineFormat = iota
-	BZ1Format
-	TSM1Format
+	// TSM1Format is the format used by the tsm1 engine.
+	TSM1Format EngineFormat = 2
 )
 
 // NewEngineFunc creates a new engine.
@@ -68,7 +74,7 @@ func RegisterEngine(name string, fn NewEngineFunc) {
 // RegisteredEngines returns the slice of currently registered engines.
 func RegisteredEngines() []string {
 	a := make([]string, 0, len(newEngineFuncs))
-	for k, _ := range newEngineFuncs {
+	for k := range newEngineFuncs {
 		a = append(a, k)
 	}
 	sort.Strings(a)
@@ -83,49 +89,14 @@ func NewEngine(path string, walPath string, options EngineOptions) (Engine, erro
 		return newEngineFuncs[options.EngineVersion](path, walPath, options), nil
 	}
 
-	// Only bolt and tsm1 based storage engines are currently supported
-	var format string
-	if err := func() error {
-		// if it's a dir then it's a tsm1 engine
-		f, err := os.Open(path)
-		if err != nil {
-			return err
-		}
-		fi, err := f.Stat()
-		f.Close()
-		if err != nil {
-			return err
-		}
-		if fi.Mode().IsDir() {
-			format = "tsm1"
-			return nil
-		}
-
-		db, err := bolt.Open(path, 0666, &bolt.Options{Timeout: 1 * time.Second})
-		if err != nil {
-			return err
-		}
-		defer db.Close()
-
-		return db.View(func(tx *bolt.Tx) error {
-			// Retrieve the meta bucket.
-			b := tx.Bucket([]byte("meta"))
-
-			// If no format is specified then it must be an original b1 database.
-			if b == nil {
-				format = "b1"
-				return nil
-			}
-
-			// Save the format.
-			format = string(b.Get([]byte("format")))
-			if format == "v1" {
-				format = "b1"
-			}
-			return nil
-		})
-	}(); err != nil {
+	// If it's a dir then it's a tsm1 engine
+	format := "tsm1"
+	if fi, err := os.Stat(path); err != nil {
 		return nil, err
+	} else if !fi.Mode().IsDir() {
+		return nil, ErrUnknownEngineFormat
+	} else {
+		format = "tsm1"
 	}
 
 	// Lookup engine by format.
@@ -139,10 +110,7 @@ func NewEngine(path string, walPath string, options EngineOptions) (Engine, erro
 
 // EngineOptions represents the options used to initialize the engine.
 type EngineOptions struct {
-	EngineVersion          string
-	MaxWALSize             int
-	WALFlushInterval       time.Duration
-	WALPartitionFlushDelay time.Duration
+	EngineVersion string
 
 	Config Config
 }
@@ -150,25 +118,11 @@ type EngineOptions struct {
 // NewEngineOptions returns the default options.
 func NewEngineOptions() EngineOptions {
 	return EngineOptions{
-		EngineVersion:          DefaultEngine,
-		MaxWALSize:             DefaultMaxWALSize,
-		WALFlushInterval:       DefaultWALFlushInterval,
-		WALPartitionFlushDelay: DefaultWALPartitionFlushDelay,
-		Config:                 NewConfig(),
+		EngineVersion: DefaultEngine,
+		Config:        NewConfig(),
 	}
 }
 
-// Tx represents a transaction.
-type Tx interface {
-	io.WriterTo
-
-	Size() int64
-	Commit() error
-	Rollback() error
-
-	Cursor(series string, fields []string, dec *FieldCodec, ascending bool) Cursor
-}
-
 // DedupeEntries returns slices with unique keys (the first 8 bytes).
 func DedupeEntries(a [][]byte) [][]byte {
 	// Convert to a map where the last slice is used.
@@ -189,6 +143,7 @@ func DedupeEntries(a [][]byte) [][]byte {
 	return other
 }
 
+// ByteSlices wraps a list of byte-slices for sorting.
 type ByteSlices [][]byte
 
 func (a ByteSlices) Len() int           { return len(a) }
