@@ -2,13 +2,14 @@ package tsm1_test
 
 import (
 	"fmt"
+	"math"
 	"os"
 	"path/filepath"
 	"testing"
 	"time"
 
-	"github.com/influxdb/influxdb/models"
-	"github.com/influxdb/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
 )
 
 //  Tests compacting a Cache snapshot into a single TSM file
@@ -16,16 +17,16 @@ func TestCompactor_Snapshot(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
 
-	v1 := tsm1.NewValue(time.Unix(1, 0), float64(1))
-	v2 := tsm1.NewValue(time.Unix(1, 0), float64(1))
-	v3 := tsm1.NewValue(time.Unix(2, 0), float64(2))
+	v1 := tsm1.NewValue(1, float64(1))
+	v2 := tsm1.NewValue(1, float64(1))
+	v3 := tsm1.NewValue(2, float64(2))
 
 	points1 := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v1},
 		"cpu,host=B#!~#value": []tsm1.Value{v2, v3},
 	}
 
-	c := tsm1.NewCache(0)
+	c := tsm1.NewCache(0, "")
 	for k, v := range points1 {
 		if err := c.Write(k, v); err != nil {
 			t.Fatalf("failed to write key foo to cache: %s", err.Error())
@@ -48,8 +49,7 @@ func TestCompactor_Snapshot(t *testing.T) {
 
 	r := MustOpenTSMReader(files[0])
 
-	keys := r.Keys()
-	if got, exp := len(keys), 2; got != exp {
+	if got, exp := r.KeyCount(), 2; got != exp {
 		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
 	}
 
@@ -83,22 +83,22 @@ func TestCompactor_CompactFull(t *testing.T) {
 	defer os.RemoveAll(dir)
 
 	// write 3 TSM files with different data and one new point
-	a1 := tsm1.NewValue(time.Unix(1, 0), 1.1)
+	a1 := tsm1.NewValue(1, 1.1)
 	writes := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{a1},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
-	a2 := tsm1.NewValue(time.Unix(2, 0), 1.2)
-	b1 := tsm1.NewValue(time.Unix(1, 0), 2.1)
+	a2 := tsm1.NewValue(2, 1.2)
+	b1 := tsm1.NewValue(1, 2.1)
 	writes = map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{a2},
 		"cpu,host=B#!~#value": []tsm1.Value{b1},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
-	a3 := tsm1.NewValue(time.Unix(1, 0), 1.3)
-	c1 := tsm1.NewValue(time.Unix(1, 0), 3.1)
+	a3 := tsm1.NewValue(1, 1.3)
+	c1 := tsm1.NewValue(1, 3.1)
 	writes = map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{a3},
 		"cpu,host=C#!~#value": []tsm1.Value{c1},
@@ -140,8 +140,7 @@ func TestCompactor_CompactFull(t *testing.T) {
 
 	r := MustOpenTSMReader(files[0])
 
-	keys := r.Keys()
-	if got, exp := len(keys), 3; got != exp {
+	if got, exp := r.KeyCount(), 3; got != exp {
 		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
 	}
 
@@ -176,20 +175,20 @@ func TestCompactor_CompactFull_SkipFullBlocks(t *testing.T) {
 	defer os.RemoveAll(dir)
 
 	// write 3 TSM files with different data and one new point
-	a1 := tsm1.NewValue(time.Unix(1, 0), 1.1)
-	a2 := tsm1.NewValue(time.Unix(2, 0), 1.2)
+	a1 := tsm1.NewValue(1, 1.1)
+	a2 := tsm1.NewValue(2, 1.2)
 	writes := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
 	}
 	f1 := MustWriteTSM(dir, 1, writes)
 
-	a3 := tsm1.NewValue(time.Unix(3, 0), 1.3)
+	a3 := tsm1.NewValue(3, 1.3)
 	writes = map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{a3},
 	}
 	f2 := MustWriteTSM(dir, 2, writes)
 
-	a4 := tsm1.NewValue(time.Unix(4, 0), 1.4)
+	a4 := tsm1.NewValue(4, 1.4)
 	writes = map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{a4},
 	}
@@ -231,8 +230,7 @@ func TestCompactor_CompactFull_SkipFullBlocks(t *testing.T) {
 
 	r := MustOpenTSMReader(files[0])
 
-	keys := r.Keys()
-	if got, exp := len(keys), 1; got != exp {
+	if got, exp := r.KeyCount(), 1; got != exp {
 		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
 	}
 
@@ -263,12 +261,313 @@ func TestCompactor_CompactFull_SkipFullBlocks(t *testing.T) {
 	}
 }
 
+// Ensures that a full compaction will skip over blocks that have the full
+// range of time contained in the block tombstoned
+func TestCompactor_CompactFull_TombstonedSkipBlock(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+
+	// write 3 TSM files with different data and one new point
+	a1 := tsm1.NewValue(1, 1.1)
+	a2 := tsm1.NewValue(2, 1.2)
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
+	}
+	f1 := MustWriteTSM(dir, 1, writes)
+
+	ts := tsm1.Tombstoner{
+		Path: f1,
+	}
+	ts.AddRange([]string{"cpu,host=A#!~#value"}, math.MinInt64, math.MaxInt64)
+
+	a3 := tsm1.NewValue(3, 1.3)
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a3},
+	}
+	f2 := MustWriteTSM(dir, 2, writes)
+
+	a4 := tsm1.NewValue(4, 1.4)
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a4},
+	}
+	f3 := MustWriteTSM(dir, 3, writes)
+
+	compactor := &tsm1.Compactor{
+		Dir:       dir,
+		FileStore: &fakeFileStore{},
+		Size:      2,
+	}
+
+	files, err := compactor.CompactFull([]string{f1, f2, f3})
+	if err != nil {
+		t.Fatalf("unexpected error writing snapshot: %v", err)
+	}
+
+	if got, exp := len(files), 1; got != exp {
+		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
+	}
+
+	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	if err != nil {
+		t.Fatalf("unexpected error parsing file name: %v", err)
+	}
+	expSeq = expSeq + 1
+
+	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	if err != nil {
+		t.Fatalf("unexpected error parsing file name: %v", err)
+	}
+
+	if gotGen != expGen {
+		t.Fatalf("wrong generation for new file: got %v, exp %v", gotGen, expGen)
+	}
+
+	if gotSeq != expSeq {
+		t.Fatalf("wrong sequence for new file: got %v, exp %v", gotSeq, expSeq)
+	}
+
+	r := MustOpenTSMReader(files[0])
+
+	if got, exp := r.KeyCount(), 1; got != exp {
+		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
+	}
+
+	var data = []struct {
+		key    string
+		points []tsm1.Value
+	}{
+		{"cpu,host=A#!~#value", []tsm1.Value{a3, a4}},
+	}
+
+	for _, p := range data {
+		values, err := r.ReadAll(p.key)
+		if err != nil {
+			t.Fatalf("unexpected error reading: %v", err)
+		}
+
+		if got, exp := len(values), len(p.points); got != exp {
+			t.Fatalf("values length mismatch %s: got %v, exp %v", p.key, got, exp)
+		}
+
+		for i, point := range p.points {
+			assertValueEqual(t, values[i], point)
+		}
+	}
+
+	if got, exp := len(r.Entries("cpu,host=A#!~#value")), 1; got != exp {
+		t.Fatalf("block count mismatch: got %v, exp %v", got, exp)
+	}
+}
+
+// Ensures that a full compaction will decode and combine blocks with
+// partial tombstoned values
+func TestCompactor_CompactFull_TombstonedPartialBlock(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+
+	// write 3 TSM files with different data and one new point
+	a1 := tsm1.NewValue(1, 1.1)
+	a2 := tsm1.NewValue(2, 1.2)
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a1, a2},
+	}
+	f1 := MustWriteTSM(dir, 1, writes)
+
+	ts := tsm1.Tombstoner{
+		Path: f1,
+	}
+	// a1 should remain after compaction
+	ts.AddRange([]string{"cpu,host=A#!~#value"}, 2, math.MaxInt64)
+
+	a3 := tsm1.NewValue(3, 1.3)
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a3},
+	}
+	f2 := MustWriteTSM(dir, 2, writes)
+
+	a4 := tsm1.NewValue(4, 1.4)
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a4},
+	}
+	f3 := MustWriteTSM(dir, 3, writes)
+
+	compactor := &tsm1.Compactor{
+		Dir:       dir,
+		FileStore: &fakeFileStore{},
+		Size:      2,
+	}
+
+	files, err := compactor.CompactFull([]string{f1, f2, f3})
+	if err != nil {
+		t.Fatalf("unexpected error writing snapshot: %v", err)
+	}
+
+	if got, exp := len(files), 1; got != exp {
+		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
+	}
+
+	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	if err != nil {
+		t.Fatalf("unexpected error parsing file name: %v", err)
+	}
+	expSeq = expSeq + 1
+
+	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	if err != nil {
+		t.Fatalf("unexpected error parsing file name: %v", err)
+	}
+
+	if gotGen != expGen {
+		t.Fatalf("wrong generation for new file: got %v, exp %v", gotGen, expGen)
+	}
+
+	if gotSeq != expSeq {
+		t.Fatalf("wrong sequence for new file: got %v, exp %v", gotSeq, expSeq)
+	}
+
+	r := MustOpenTSMReader(files[0])
+
+	if got, exp := r.KeyCount(), 1; got != exp {
+		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
+	}
+
+	var data = []struct {
+		key    string
+		points []tsm1.Value
+	}{
+		{"cpu,host=A#!~#value", []tsm1.Value{a1, a3, a4}},
+	}
+
+	for _, p := range data {
+		values, err := r.ReadAll(p.key)
+		if err != nil {
+			t.Fatalf("unexpected error reading: %v", err)
+		}
+
+		if got, exp := len(values), len(p.points); got != exp {
+			t.Fatalf("values length mismatch %s: got %v, exp %v", p.key, got, exp)
+		}
+
+		for i, point := range p.points {
+			assertValueEqual(t, values[i], point)
+		}
+	}
+
+	if got, exp := len(r.Entries("cpu,host=A#!~#value")), 2; got != exp {
+		t.Fatalf("block count mismatch: got %v, exp %v", got, exp)
+	}
+}
+
+// Ensures that a full compaction will decode and combine blocks with
+// multiple tombstoned ranges within the block e.g. (t1, t2, t3, t4)
+// having t2 and t3 removed
+func TestCompactor_CompactFull_TombstonedMultipleRanges(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+
+	// write 3 TSM files with different data and one new point
+	a1 := tsm1.NewValue(1, 1.1)
+	a2 := tsm1.NewValue(2, 1.2)
+	a3 := tsm1.NewValue(3, 1.3)
+	a4 := tsm1.NewValue(4, 1.4)
+
+	writes := map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a1, a2, a3, a4},
+	}
+	f1 := MustWriteTSM(dir, 1, writes)
+
+	ts := tsm1.Tombstoner{
+		Path: f1,
+	}
+	// a1, a3 should remain after compaction
+	ts.AddRange([]string{"cpu,host=A#!~#value"}, 2, 2)
+	ts.AddRange([]string{"cpu,host=A#!~#value"}, 4, 4)
+
+	a5 := tsm1.NewValue(5, 1.5)
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a5},
+	}
+	f2 := MustWriteTSM(dir, 2, writes)
+
+	a6 := tsm1.NewValue(6, 1.6)
+	writes = map[string][]tsm1.Value{
+		"cpu,host=A#!~#value": []tsm1.Value{a6},
+	}
+	f3 := MustWriteTSM(dir, 3, writes)
+
+	compactor := &tsm1.Compactor{
+		Dir:       dir,
+		FileStore: &fakeFileStore{},
+		Size:      2,
+	}
+
+	files, err := compactor.CompactFull([]string{f1, f2, f3})
+	if err != nil {
+		t.Fatalf("unexpected error writing snapshot: %v", err)
+	}
+
+	if got, exp := len(files), 1; got != exp {
+		t.Fatalf("files length mismatch: got %v, exp %v", got, exp)
+	}
+
+	expGen, expSeq, err := tsm1.ParseTSMFileName(f3)
+	if err != nil {
+		t.Fatalf("unexpected error parsing file name: %v", err)
+	}
+	expSeq = expSeq + 1
+
+	gotGen, gotSeq, err := tsm1.ParseTSMFileName(files[0])
+	if err != nil {
+		t.Fatalf("unexpected error parsing file name: %v", err)
+	}
+
+	if gotGen != expGen {
+		t.Fatalf("wrong generation for new file: got %v, exp %v", gotGen, expGen)
+	}
+
+	if gotSeq != expSeq {
+		t.Fatalf("wrong sequence for new file: got %v, exp %v", gotSeq, expSeq)
+	}
+
+	r := MustOpenTSMReader(files[0])
+
+	if got, exp := r.KeyCount(), 1; got != exp {
+		t.Fatalf("keys length mismatch: got %v, exp %v", got, exp)
+	}
+
+	var data = []struct {
+		key    string
+		points []tsm1.Value
+	}{
+		{"cpu,host=A#!~#value", []tsm1.Value{a1, a3, a5, a6}},
+	}
+
+	for _, p := range data {
+		values, err := r.ReadAll(p.key)
+		if err != nil {
+			t.Fatalf("unexpected error reading: %v", err)
+		}
+
+		if got, exp := len(values), len(p.points); got != exp {
+			t.Fatalf("values length mismatch %s: got %v, exp %v", p.key, got, exp)
+		}
+
+		for i, point := range p.points {
+			assertValueEqual(t, values[i], point)
+		}
+	}
+
+	if got, exp := len(r.Entries("cpu,host=A#!~#value")), 2; got != exp {
+		t.Fatalf("block count mismatch: got %v, exp %v", got, exp)
+	}
+}
+
 // Tests that a single TSM file can be read and iterated over
 func TestTSMKeyIterator_Single(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
 
-	v1 := tsm1.NewValue(time.Unix(1, 0), 1.1)
+	v1 := tsm1.NewValue(1, 1.1)
 	writes := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v1},
 	}
@@ -317,8 +616,8 @@ func TestTSMKeyIterator_Chunked(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
 
-	v0 := tsm1.NewValue(time.Unix(1, 0), 1.1)
-	v1 := tsm1.NewValue(time.Unix(2, 0), 2.1)
+	v0 := tsm1.NewValue(1, 1.1)
+	v1 := tsm1.NewValue(2, 2.1)
 	writes := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v0, v1},
 	}
@@ -372,8 +671,8 @@ func TestTSMKeyIterator_Duplicate(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
 
-	v1 := tsm1.NewValue(time.Unix(1, 0), int64(1))
-	v2 := tsm1.NewValue(time.Unix(1, 0), int64(2))
+	v1 := tsm1.NewValue(1, int64(1))
+	v2 := tsm1.NewValue(1, int64(2))
 
 	writes1 := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v1},
@@ -427,16 +726,18 @@ func TestTSMKeyIterator_MultipleKeysDeleted(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
 
-	v1 := tsm1.NewValue(time.Unix(2, 0), int64(1))
+	v1 := tsm1.NewValue(2, int64(1))
 	points1 := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v1},
 	}
 
 	r1 := MustTSMReader(dir, 1, points1)
-	r1.Delete([]string{"cpu,host=A#!~#value"})
+	if e := r1.Delete([]string{"cpu,host=A#!~#value"}); nil != e {
+		t.Fatal(e)
+	}
 
-	v2 := tsm1.NewValue(time.Unix(1, 0), float64(1))
-	v3 := tsm1.NewValue(time.Unix(1, 0), float64(1))
+	v2 := tsm1.NewValue(1, float64(1))
+	v3 := tsm1.NewValue(1, float64(1))
 
 	points2 := map[string][]tsm1.Value{
 		"cpu,host=A#!~#count": []tsm1.Value{v2},
@@ -489,13 +790,13 @@ func TestTSMKeyIterator_MultipleKeysDeleted(t *testing.T) {
 }
 
 func TestCacheKeyIterator_Single(t *testing.T) {
-	v0 := tsm1.NewValue(time.Unix(1, 0).UTC(), 1.0)
+	v0 := tsm1.NewValue(1, 1.0)
 
 	writes := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v0},
 	}
 
-	c := tsm1.NewCache(0)
+	c := tsm1.NewCache(0, "")
 
 	for k, v := range writes {
 		if err := c.Write(k, v); err != nil {
@@ -536,14 +837,14 @@ func TestCacheKeyIterator_Single(t *testing.T) {
 }
 
 func TestCacheKeyIterator_Chunked(t *testing.T) {
-	v0 := tsm1.NewValue(time.Unix(1, 0).UTC(), 1.0)
-	v1 := tsm1.NewValue(time.Unix(2, 0).UTC(), 2.0)
+	v0 := tsm1.NewValue(1, 1.0)
+	v1 := tsm1.NewValue(2, 2.0)
 
 	writes := map[string][]tsm1.Value{
 		"cpu,host=A#!~#value": []tsm1.Value{v0, v1},
 	}
 
-	c := tsm1.NewCache(0)
+	c := tsm1.NewCache(0, "")
 
 	for k, v := range writes {
 		if err := c.Write(k, v); err != nil {
@@ -1064,7 +1365,7 @@ func TestDefaultPlanner_Plan_CompactsMiddleSteps(t *testing.T) {
 }
 
 func assertValueEqual(t *testing.T, a, b tsm1.Value) {
-	if got, exp := a.Time(), b.Time(); !got.Equal(exp) {
+	if got, exp := a.UnixNano(), b.UnixNano(); got != exp {
 		t.Fatalf("time mismatch: got %v, exp %v", got, exp)
 	}
 	if got, exp := a.Value(), b.Value(); got != exp {
@@ -1073,7 +1374,7 @@ func assertValueEqual(t *testing.T, a, b tsm1.Value) {
 }
 
 func assertEqual(t *testing.T, a tsm1.Value, b models.Point, field string) {
-	if got, exp := a.Time(), b.Time(); !got.Equal(exp) {
+	if got, exp := a.UnixNano(), b.UnixNano(); got != exp {
 		t.Fatalf("time mismatch: got %v, exp %v", got, exp)
 	}
 	if got, exp := a.Value(), b.Fields()[field]; got != exp {
@@ -1151,10 +1452,7 @@ func MustOpenTSMReader(name string) *tsm1.TSMReader {
 		panic(fmt.Sprintf("open file: %v", err))
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		panic(fmt.Sprintf("new reader: %v", err))
 	}
