@@ -1,22 +1,24 @@
-package tsm1
+package tsm1 // import "github.com/influxdata/influxdb/tsdb/engine/tsm1"
 
 import (
 	"archive/tar"
 	"fmt"
 	"io"
 	"log"
+	"math"
 	"os"
 	"path/filepath"
-	"sort"
 	"strings"
 	"sync"
 	"time"
 
-	"github.com/influxdb/influxdb/influxql"
-	"github.com/influxdb/influxdb/models"
-	"github.com/influxdb/influxdb/tsdb"
+	"github.com/influxdata/influxdb/influxql"
+	"github.com/influxdata/influxdb/models"
+	"github.com/influxdata/influxdb/tsdb"
 )
 
+//go:generate tmpl -data=@iterator.gen.go.tmpldata iterator.gen.go.tmpl
+
 func init() {
 	tsdb.RegisterEngine("tsm1", NewEngine)
 }
@@ -36,8 +38,13 @@ type Engine struct {
 	done chan struct{}
 	wg   sync.WaitGroup
 
-	path   string
-	logger *log.Logger
+	path      string
+	logger    *log.Logger
+	logOutput io.Writer
+
+	// TODO(benbjohnson): Index needs to be moved entirely into engine.
+	index             *tsdb.DatabaseIndex
+	measurementFields map[string]*tsdb.MeasurementFields
 
 	WAL            *WAL
 	Cache          *Cache
@@ -65,7 +72,7 @@ func NewEngine(path string, walPath string, opt tsdb.EngineOptions) tsdb.Engine
 	fs := NewFileStore(path)
 	fs.traceLogging = opt.Config.DataLoggingEnabled
 
-	cache := NewCache(uint64(opt.Config.CacheMaxMemorySize))
+	cache := NewCache(uint64(opt.Config.CacheMaxMemorySize), path)
 
 	c := &Compactor{
 		Dir:       path,
@@ -73,8 +80,8 @@ func NewEngine(path string, walPath string, opt tsdb.EngineOptions) tsdb.Engine
 	}
 
 	e := &Engine{
-		path:   path,
-		logger: log.New(os.Stderr, "[tsm1] ", log.LstdFlags),
+		path:              path,
+		measurementFields: make(map[string]*tsdb.MeasurementFields),
 
 		WAL:   w,
 		Cache: cache,
@@ -90,6 +97,7 @@ func NewEngine(path string, walPath string, opt tsdb.EngineOptions) tsdb.Engine
 		CacheFlushMemorySizeThreshold: opt.Config.CacheSnapshotMemorySize,
 		CacheFlushWriteColdDuration:   time.Duration(opt.Config.CacheSnapshotWriteColdDuration),
 	}
+	e.SetLogOutput(os.Stderr)
 
 	return e
 }
@@ -97,8 +105,31 @@ func NewEngine(path string, walPath string, opt tsdb.EngineOptions) tsdb.Engine
 // Path returns the path the engine was opened with.
 func (e *Engine) Path() string { return e.path }
 
-// PerformMaintenance is for periodic maintenance of the store. A no-op for b1
-func (e *Engine) PerformMaintenance() {
+// Index returns the database index.
+func (e *Engine) Index() *tsdb.DatabaseIndex {
+	e.mu.Lock()
+	defer e.mu.Unlock()
+	return e.index
+}
+
+// MeasurementFields returns the measurement fields for a measurement.
+func (e *Engine) MeasurementFields(measurement string) *tsdb.MeasurementFields {
+	e.mu.RLock()
+	m := e.measurementFields[measurement]
+	e.mu.RUnlock()
+
+	if m != nil {
+		return m
+	}
+
+	e.mu.Lock()
+	m = e.measurementFields[measurement]
+	if m == nil {
+		m = tsdb.NewMeasurementFields()
+		e.measurementFields[measurement] = m
+	}
+	e.mu.Unlock()
+	return m
 }
 
 // Format returns the format type of this engine
@@ -141,8 +172,15 @@ func (e *Engine) Open() error {
 	return nil
 }
 
-// Close closes the engine.
+// Close closes the engine. Subsequent calls to Close are a nop.
 func (e *Engine) Close() error {
+	e.mu.RLock()
+	if e.done == nil {
+		e.mu.RUnlock()
+		return nil
+	}
+	e.mu.RUnlock()
+
 	// Shutdown goroutines and wait.
 	close(e.done)
 	e.wg.Wait()
@@ -150,6 +188,7 @@ func (e *Engine) Close() error {
 	// Lock now and close everything else down.
 	e.mu.Lock()
 	defer e.mu.Unlock()
+	e.done = nil // Ensures that the channel will not be closed again.
 
 	if err := e.FileStore.Close(); err != nil {
 		return err
@@ -157,48 +196,47 @@ func (e *Engine) Close() error {
 	return e.WAL.Close()
 }
 
-// SetLogOutput is a no-op.
-func (e *Engine) SetLogOutput(w io.Writer) {}
+// SetLogOutput sets the logger used for all messages. It must not be called
+// after the Open method has been called.
+func (e *Engine) SetLogOutput(w io.Writer) {
+	e.logger = log.New(w, "[tsm1] ", log.LstdFlags)
+	e.WAL.SetLogOutput(w)
+	e.FileStore.SetLogOutput(w)
+	e.logOutput = w
+}
 
 // LoadMetadataIndex loads the shard metadata into memory.
-func (e *Engine) LoadMetadataIndex(_ *tsdb.Shard, index *tsdb.DatabaseIndex, measurementFields map[string]*tsdb.MeasurementFields) error {
-	keys := e.FileStore.Keys()
+func (e *Engine) LoadMetadataIndex(shardID uint64, index *tsdb.DatabaseIndex) error {
+	// Save reference to index for iterator creation.
+	e.index = index
 
-	keysLoaded := make(map[string]bool)
-
-	for _, k := range keys {
-		typ, err := e.FileStore.Type(k)
-		if err != nil {
-			return err
-		}
+	if err := e.FileStore.WalkKeys(func(key string, typ byte) error {
 		fieldType, err := tsmFieldTypeToInfluxQLDataType(typ)
 		if err != nil {
 			return err
 		}
 
-		if err := e.addToIndexFromKey(k, fieldType, index, measurementFields); err != nil {
+		if err := e.addToIndexFromKey(shardID, key, fieldType, index); err != nil {
 			return err
 		}
-
-		keysLoaded[k] = true
+		return nil
+	}); err != nil {
+		return err
 	}
 
 	// load metadata from the Cache
-	e.Cache.Lock() // shouldn't need the lock, but just to be safe
-	defer e.Cache.Unlock()
+	e.Cache.RLock() // shouldn't need the lock, but just to be safe
+	defer e.Cache.RUnlock()
 
 	for key, entry := range e.Cache.Store() {
-		if keysLoaded[key] {
-			continue
-		}
 
 		fieldType, err := entry.values.InfluxQLType()
 		if err != nil {
-			log.Printf("error getting the data type of values for key %s: %s", key, err.Error())
+			e.logger.Printf("error getting the data type of values for key %s: %s", key, err.Error())
 			continue
 		}
 
-		if err := e.addToIndexFromKey(key, fieldType, index, measurementFields); err != nil {
+		if err := e.addToIndexFromKey(shardID, key, fieldType, index); err != nil {
 			return err
 		}
 	}
@@ -224,11 +262,11 @@ func (e *Engine) Backup(w io.Writer, basePath string, since time.Time) error {
 
 	// grab all the files and tombstones that have a modified time after since
 	for _, f := range e.FileStore.files {
-		if stat := f.Stats(); stat.LastModified.After(since) {
+		if stat := f.Stats(); stat.LastModified > since.UnixNano() {
 			files = append(files, f.Stats())
 		}
 		for _, t := range f.TombstoneFiles() {
-			if t.LastModified.After(since) {
+			if t.LastModified > since.UnixNano() {
 				files = append(files, f.Stats())
 			}
 		}
@@ -251,7 +289,7 @@ func (e *Engine) Backup(w io.Writer, basePath string, since time.Time) error {
 func (e *Engine) writeFileToBackup(f FileStat, shardRelativePath string, tw *tar.Writer) error {
 	h := &tar.Header{
 		Name:    filepath.Join(shardRelativePath, filepath.Base(f.Path)),
-		ModTime: f.LastModified,
+		ModTime: time.Unix(0, f.LastModified),
 		Size:    int64(f.Size),
 	}
 	if err := tw.WriteHeader(h); err != nil {
@@ -271,45 +309,43 @@ func (e *Engine) writeFileToBackup(f FileStat, shardRelativePath string, tw *tar
 
 // addToIndexFromKey will pull the measurement name, series key, and field name from a composite key and add it to the
 // database index and measurement fields
-func (e *Engine) addToIndexFromKey(key string, fieldType influxql.DataType, index *tsdb.DatabaseIndex, measurementFields map[string]*tsdb.MeasurementFields) error {
+func (e *Engine) addToIndexFromKey(shardID uint64, key string, fieldType influxql.DataType, index *tsdb.DatabaseIndex) error {
 	seriesKey, field := seriesAndFieldFromCompositeKey(key)
 	measurement := tsdb.MeasurementFromSeriesKey(seriesKey)
 
 	m := index.CreateMeasurementIndexIfNotExists(measurement)
 	m.SetFieldName(field)
 
-	mf := measurementFields[measurement]
+	mf := e.measurementFields[measurement]
 	if mf == nil {
-		mf = &tsdb.MeasurementFields{
-			Fields: map[string]*tsdb.Field{},
-		}
-		measurementFields[measurement] = mf
+		mf = tsdb.NewMeasurementFields()
+		e.measurementFields[measurement] = mf
 	}
 
 	if err := mf.CreateFieldIfNotExists(field, fieldType, false); err != nil {
 		return err
 	}
 
-	_, tags, err := models.ParseKey(seriesKey)
-	if err == nil {
-		return err
-	}
+	// ignore error because ParseKey returns "missing fields" and we don't have
+	// fields (in line protocol format) in the series key
+	_, tags, _ := models.ParseKey(seriesKey)
 
 	s := tsdb.NewSeries(seriesKey, tags)
 	s.InitializeShards()
 	index.CreateSeriesIndexIfNotExists(measurement, s)
+	s.AssignShard(shardID)
 
 	return nil
 }
 
 // WritePoints writes metadata and point data into the engine.
 // Returns an error if new points are added to an existing key.
-func (e *Engine) WritePoints(points []models.Point, measurementFieldsToSave map[string]*tsdb.MeasurementFields, seriesToCreate []*tsdb.SeriesCreate) error {
+func (e *Engine) WritePoints(points []models.Point) error {
 	values := map[string][]Value{}
 	for _, p := range points {
 		for k, v := range p.Fields() {
 			key := string(p.Key()) + keyFieldSeparator + k
-			values[key] = append(values[key], NewValue(p.Time(), v))
+			values[key] = append(values[key], NewValue(p.Time().UnixNano(), v))
 		}
 	}
 
@@ -326,8 +362,44 @@ func (e *Engine) WritePoints(points []models.Point, measurementFieldsToSave map[
 	return err
 }
 
-// DeleteSeries deletes the series from the engine.
+// ContainsSeries returns a map of keys indicating whether the key exists and
+// has values or not.
+func (e *Engine) ContainsSeries(keys []string) (map[string]bool, error) {
+	// keyMap is used to see if a given key exists.  keys
+	// are the measurement + tagset (minus separate & field)
+	keyMap := map[string]bool{}
+	for _, k := range keys {
+		keyMap[k] = false
+	}
+
+	for _, k := range e.Cache.Keys() {
+		seriesKey, _ := seriesAndFieldFromCompositeKey(k)
+		keyMap[seriesKey] = true
+	}
+
+	if err := e.FileStore.WalkKeys(func(k string, _ byte) error {
+		seriesKey, _ := seriesAndFieldFromCompositeKey(k)
+		if _, ok := keyMap[seriesKey]; ok {
+			keyMap[seriesKey] = true
+		}
+		return nil
+	}); err != nil {
+		return nil, err
+	}
+	return keyMap, nil
+}
+
+// DeleteSeries removes all series keys from the engine.
 func (e *Engine) DeleteSeries(seriesKeys []string) error {
+	return e.DeleteSeriesRange(seriesKeys, math.MinInt64, math.MaxInt64)
+}
+
+// DeleteSeriesRange removes the values between min and max (inclusive) from all series.
+func (e *Engine) DeleteSeriesRange(seriesKeys []string, min, max int64) error {
+	if len(seriesKeys) == 0 {
+		return nil
+	}
+
 	e.mu.RLock()
 	defer e.mu.RUnlock()
 
@@ -340,36 +412,46 @@ func (e *Engine) DeleteSeries(seriesKeys []string) error {
 
 	var deleteKeys []string
 	// go through the keys in the file store
-	for _, k := range e.FileStore.Keys() {
+	if err := e.FileStore.WalkKeys(func(k string, _ byte) error {
 		seriesKey, _ := seriesAndFieldFromCompositeKey(k)
 		if _, ok := keyMap[seriesKey]; ok {
 			deleteKeys = append(deleteKeys, k)
 		}
+		return nil
+	}); err != nil {
+		return err
+	}
+
+	if err := e.FileStore.DeleteRange(deleteKeys, min, max); err != nil {
+		return err
 	}
-	e.FileStore.Delete(deleteKeys)
 
 	// find the keys in the cache and remove them
 	walKeys := make([]string, 0)
-	e.Cache.Lock()
-	defer e.Cache.Unlock()
-
+	e.Cache.RLock()
 	s := e.Cache.Store()
 	for k, _ := range s {
 		seriesKey, _ := seriesAndFieldFromCompositeKey(k)
 		if _, ok := keyMap[seriesKey]; ok {
 			walKeys = append(walKeys, k)
-			delete(s, k)
 		}
 	}
+	e.Cache.RUnlock()
+
+	e.Cache.DeleteRange(walKeys, min, max)
 
 	// delete from the WAL
-	_, err := e.WAL.Delete(walKeys)
+	_, err := e.WAL.DeleteRange(walKeys, min, max)
 
 	return err
 }
 
 // DeleteMeasurement deletes a measurement and all related series.
 func (e *Engine) DeleteMeasurement(name string, seriesKeys []string) error {
+	e.mu.Lock()
+	delete(e.measurementFields, name)
+	e.mu.Unlock()
+
 	return e.DeleteSeries(seriesKeys)
 }
 
@@ -378,21 +460,28 @@ func (e *Engine) SeriesCount() (n int, err error) {
 	return 0, nil
 }
 
-// Begin starts a new transaction on the engine.
-func (e *Engine) Begin(writable bool) (tsdb.Tx, error) {
-	return &tx{engine: e}, nil
-}
-
 func (e *Engine) WriteTo(w io.Writer) (n int64, err error) { panic("not implemented") }
 
 // WriteSnapshot will snapshot the cache and write a new TSM file with its contents, releasing the snapshot when done.
 func (e *Engine) WriteSnapshot() error {
 	// Lock and grab the cache snapshot along with all the closed WAL
 	// filenames associated with the snapshot
+
+	var started *time.Time
+
+	defer func() {
+		if started != nil {
+			e.Cache.UpdateCompactTime(time.Now().Sub(*started))
+		}
+	}()
+
 	closedFiles, snapshot, compactor, err := func() ([]string, *Cache, *Compactor, error) {
 		e.mu.Lock()
 		defer e.mu.Unlock()
 
+		now := time.Now()
+		started = &now
+
 		if err := e.WAL.CloseSegment(); err != nil {
 			return nil, nil, nil, err
 		}
@@ -402,7 +491,10 @@ func (e *Engine) WriteSnapshot() error {
 			return nil, nil, nil, err
 		}
 
-		snapshot := e.Cache.Snapshot()
+		snapshot, err := e.Cache.Snapshot()
+		if err != nil {
+			return nil, nil, nil, err
+		}
 
 		return segments, snapshot, e.Compactor.Clone(), nil
 	}()
@@ -420,7 +512,13 @@ func (e *Engine) WriteSnapshot() error {
 }
 
 // writeSnapshotAndCommit will write the passed cache to a new TSM file and remove the closed WAL segments
-func (e *Engine) writeSnapshotAndCommit(closedFiles []string, snapshot *Cache, compactor *Compactor) error {
+func (e *Engine) writeSnapshotAndCommit(closedFiles []string, snapshot *Cache, compactor *Compactor) (err error) {
+
+	defer func() {
+		if err != nil {
+			e.Cache.ClearSnapshot(false)
+		}
+	}()
 	// write the new snapshot files
 	newFiles, err := compactor.WriteSnapshot(snapshot)
 	if err != nil {
@@ -438,7 +536,7 @@ func (e *Engine) writeSnapshotAndCommit(closedFiles []string, snapshot *Cache, c
 	}
 
 	// clear the snapshot from the in-memory cache, then the old WAL files
-	e.Cache.ClearSnapshot(snapshot)
+	e.Cache.ClearSnapshot(true)
 
 	if err := e.WAL.Remove(closedFiles); err != nil {
 		e.logger.Printf("error removing closed wal segments: %v", err)
@@ -456,6 +554,7 @@ func (e *Engine) compactCache() {
 			return
 
 		default:
+			e.Cache.UpdateAge()
 			if e.ShouldCompactCache(e.WAL.LastWriteTime()) {
 				err := e.WriteSnapshot()
 				if err != nil {
@@ -603,7 +702,16 @@ func (e *Engine) reloadCache() error {
 		return err
 	}
 
+	limit := e.Cache.MaxSize()
+	defer func() {
+		e.Cache.SetMaxSize(limit)
+	}()
+
+	// Disable the max size during loading
+	e.Cache.SetMaxSize(0)
+
 	loader := NewCacheLoader(files)
+	loader.SetLogOutput(e.logOutput)
 	if err := loader.Load(e.Cache); err != nil {
 		return err
 	}
@@ -625,217 +733,301 @@ func (e *Engine) cleanup() error {
 	return nil
 }
 
-func (e *Engine) KeyCursor(key string) *KeyCursor {
+func (e *Engine) KeyCursor(key string, t int64, ascending bool) *KeyCursor {
 	e.mu.RLock()
 	defer e.mu.RUnlock()
-	return e.FileStore.KeyCursor(key)
+	return e.FileStore.KeyCursor(key, t, ascending)
 }
 
-type tx struct {
-	engine *Engine
-}
-
-// Cursor returns a cursor for all cached and TSM-based data.
-func (t *tx) Cursor(series string, fields []string, dec *tsdb.FieldCodec, ascending bool) tsdb.Cursor {
-	if len(fields) == 1 {
-		key := SeriesFieldKey(series, fields[0])
-		return &devCursor{
-			series:       series,
-			fields:       fields,
-			cache:        t.engine.Cache.Values(key),
-			tsmKeyCursor: t.engine.KeyCursor(key),
-			ascending:    ascending,
-		}
-	}
-
-	// multiple fields. use just the MultiFieldCursor, which also handles time collisions
-	// so we don't need to use the combined cursor
-	var cursors []tsdb.Cursor
-	var cursorFields []string
-	for _, field := range fields {
-		key := SeriesFieldKey(series, field)
-		wc := &devCursor{
-			series:       series,
-			fields:       []string{field},
-			cache:        t.engine.Cache.Values(key),
-			tsmKeyCursor: t.engine.KeyCursor(key),
-			ascending:    ascending,
+func (e *Engine) CreateIterator(opt influxql.IteratorOptions) (influxql.Iterator, error) {
+	if call, ok := opt.Expr.(*influxql.Call); ok {
+		refOpt := opt
+		refOpt.Expr = call.Args[0].(*influxql.VarRef)
+		inputs, err := e.createVarRefIterator(refOpt)
+		if err != nil {
+			return nil, err
 		}
 
-		// double up the fields since there's one for the wal and one for the index
-		cursorFields = append(cursorFields, field)
-		cursors = append(cursors, wc)
-	}
-	return NewMultiFieldCursor(cursorFields, cursors, ascending)
-}
-
-func (t *tx) Rollback() error                          { return nil }
-func (t *tx) Size() int64                              { panic("not implemented") }
-func (t *tx) Commit() error                            { panic("not implemented") }
-func (t *tx) WriteTo(w io.Writer) (n int64, err error) { panic("not implemented") }
-
-// devCursor is a cursor that combines both TSM and cached data.
-type devCursor struct {
-	series string
-	fields []string
-
-	cache         Values
-	cachePos      int
-	cacheKeyBuf   int64
-	cacheValueBuf interface{}
-
-	tsmValues   Values
-	tsmPos      int
-	tsmKeyBuf   int64
-	tsmValueBuf interface{}
-
-	tsmKeyCursor *KeyCursor
-	ascending    bool
-}
-
-// SeekTo positions the cursor at the timestamp specified by seek and returns the
-// timestamp and value.
-func (c *devCursor) SeekTo(seek int64) (int64, interface{}) {
-	// Seek to position in cache.
-	c.cacheKeyBuf, c.cacheValueBuf = func() (int64, interface{}) {
-		// Seek to position in cache index.
-		c.cachePos = sort.Search(len(c.cache), func(i int) bool {
-			return c.cache[i].Time().UnixNano() >= seek
-		})
-
-		if c.cachePos < len(c.cache) {
-			v := c.cache[c.cachePos]
-			if v.UnixNano() == seek || c.ascending {
-				// Exact seek found or, if ascending, next one is good.
-				return v.UnixNano(), v.Value()
+		input := influxql.NewMergeIterator(inputs, opt)
+		if input != nil {
+			if opt.InterruptCh != nil {
+				input = influxql.NewInterruptIterator(input, opt.InterruptCh)
 			}
-			// Nothing available if descending.
-			return tsdb.EOF, nil
+			return influxql.NewCallIterator(input, opt)
 		}
-
-		// Ascending cursor, no match in the cache.
-		if c.ascending {
-			return tsdb.EOF, nil
-		}
-
-		// Descending cursor, go to previous value in cache, and return if it exists.
-		c.cachePos--
-		if c.cachePos < 0 {
-			return tsdb.EOF, nil
-		}
-		return c.cache[c.cachePos].UnixNano(), c.cache[c.cachePos].Value()
-	}()
-
-	// Seek to position to tsm block.
-	if c.ascending {
-		c.tsmValues, _ = c.tsmKeyCursor.SeekTo(time.Unix(0, seek-1), c.ascending)
-	} else {
-		c.tsmValues, _ = c.tsmKeyCursor.SeekTo(time.Unix(0, seek+1), c.ascending)
+		return nil, nil
 	}
 
-	c.tsmPos = sort.Search(len(c.tsmValues), func(i int) bool {
-		return c.tsmValues[i].Time().UnixNano() >= seek
-	})
-
-	if !c.ascending {
-		c.tsmPos--
+	itrs, err := e.createVarRefIterator(opt)
+	if err != nil {
+		return nil, err
 	}
 
-	if c.tsmPos >= 0 && c.tsmPos < len(c.tsmValues) {
-		c.tsmKeyBuf = c.tsmValues[c.tsmPos].Time().UnixNano()
-		c.tsmValueBuf = c.tsmValues[c.tsmPos].Value()
-	} else {
-		c.tsmKeyBuf = tsdb.EOF
-		c.tsmKeyCursor.Close()
+	itr := influxql.NewSortedMergeIterator(itrs, opt)
+	if itr != nil && opt.InterruptCh != nil {
+		itr = influxql.NewInterruptIterator(itr, opt.InterruptCh)
 	}
-
-	return c.read()
+	return itr, nil
 }
 
-// Next returns the next value from the cursor.
-func (c *devCursor) Next() (int64, interface{}) {
-	return c.read()
+func (e *Engine) SeriesKeys(opt influxql.IteratorOptions) (influxql.SeriesList, error) {
+	seriesList := influxql.SeriesList{}
+	mms := tsdb.Measurements(e.index.MeasurementsByName(influxql.Sources(opt.Sources).Names()))
+	for _, mm := range mms {
+		// Determine tagsets for this measurement based on dimensions and filters.
+		tagSets, err := mm.TagSets(opt.Dimensions, opt.Condition)
+		if err != nil {
+			return nil, err
+		}
+
+		// Calculate tag sets and apply SLIMIT/SOFFSET.
+		tagSets = influxql.LimitTagSets(tagSets, opt.SLimit, opt.SOffset)
+		for _, t := range tagSets {
+			series := influxql.Series{
+				Name: mm.Name,
+				Tags: influxql.NewTags(t.Tags),
+				Aux:  make([]influxql.DataType, len(opt.Aux)),
+			}
+
+			// Determine the aux field types.
+			for _, seriesKey := range t.SeriesKeys {
+				tags := influxql.NewTags(e.index.TagsForSeries(seriesKey))
+				for i, field := range opt.Aux {
+					typ := func() influxql.DataType {
+						mf := e.measurementFields[mm.Name]
+						if mf == nil {
+							return influxql.Unknown
+						}
+
+						f := mf.Field(field)
+						if f == nil {
+							return influxql.Unknown
+						}
+						return f.Type
+					}()
+
+					if typ == influxql.Unknown {
+						if v := tags.Value(field); v != "" {
+							// All tags are strings.
+							typ = influxql.String
+						}
+					}
+
+					if typ != influxql.Unknown {
+						if series.Aux[i] == influxql.Unknown || typ < series.Aux[i] {
+							series.Aux[i] = typ
+						}
+					}
+				}
+			}
+			seriesList = append(seriesList, series)
+		}
+	}
+	return seriesList, nil
 }
 
-// Ascending returns whether the cursor returns data in time-ascending order.
-func (c *devCursor) Ascending() bool { return c.ascending }
+// createVarRefIterator creates an iterator for a variable reference.
+func (e *Engine) createVarRefIterator(opt influxql.IteratorOptions) ([]influxql.Iterator, error) {
+	ref, _ := opt.Expr.(*influxql.VarRef)
 
-// read returns the next value for the cursor.
-func (c *devCursor) read() (int64, interface{}) {
-	var key int64
-	var value interface{}
+	var itrs []influxql.Iterator
+	if err := func() error {
+		mms := tsdb.Measurements(e.index.MeasurementsByName(influxql.Sources(opt.Sources).Names()))
 
-	// Determine where the next datum should come from -- the cache or the TSM files.
+		// Retrieve the maximum number of fields (without time).
+		conditionFields := make([]string, len(influxql.ExprNames(opt.Condition)))
 
-	switch {
-	// No more data in cache or in TSM files.
-	case c.cacheKeyBuf == tsdb.EOF && c.tsmKeyBuf == tsdb.EOF:
-		key = tsdb.EOF
+		for _, mm := range mms {
+			// Determine tagsets for this measurement based on dimensions and filters.
+			tagSets, err := mm.TagSets(opt.Dimensions, opt.Condition)
+			if err != nil {
+				return err
+			}
 
-	// Both cache and tsm files have the same key, cache takes precedence.
-	case c.cacheKeyBuf == c.tsmKeyBuf:
-		key = c.cacheKeyBuf
-		value = c.cacheValueBuf
-		c.cacheKeyBuf, c.cacheValueBuf = c.nextCache()
-		c.tsmKeyBuf, c.tsmValueBuf = c.nextTSM()
+			// Calculate tag sets and apply SLIMIT/SOFFSET.
+			tagSets = influxql.LimitTagSets(tagSets, opt.SLimit, opt.SOffset)
 
-	// Buffered cache key precedes that in TSM file.
-	case c.ascending && (c.cacheKeyBuf != tsdb.EOF && (c.cacheKeyBuf < c.tsmKeyBuf || c.tsmKeyBuf == tsdb.EOF)),
-		!c.ascending && (c.cacheKeyBuf != tsdb.EOF && (c.cacheKeyBuf > c.tsmKeyBuf || c.tsmKeyBuf == tsdb.EOF)):
-		key = c.cacheKeyBuf
-		value = c.cacheValueBuf
-		c.cacheKeyBuf, c.cacheValueBuf = c.nextCache()
+			for _, t := range tagSets {
+				inputs := make([]influxql.Iterator, 0, len(t.SeriesKeys))
+				for i, seriesKey := range t.SeriesKeys {
+					fields := 0
+					if t.Filters[i] != nil {
+						// Retrieve non-time fields from this series filter and filter out tags.
+						for _, f := range influxql.ExprNames(t.Filters[i]) {
+							conditionFields[fields] = f
+							fields++
+						}
+					}
 
-	// Buffered TSM key precedes that in cache.
+					input, err := e.createVarRefSeriesIterator(ref, mm, seriesKey, t, t.Filters[i], conditionFields[:fields], opt)
+					if err != nil {
+						return err
+					} else if input == nil {
+						continue
+					}
+					inputs = append(inputs, input)
+				}
+
+				if len(inputs) > 0 && (opt.Limit > 0 || opt.Offset > 0) {
+					var itr influxql.Iterator
+					if opt.MergeSorted() {
+						itr = influxql.NewSortedMergeIterator(inputs, opt)
+					} else {
+						itr = influxql.NewMergeIterator(inputs, opt)
+					}
+					itrs = append(itrs, newLimitIterator(itr, opt))
+				} else {
+					itrs = append(itrs, inputs...)
+				}
+			}
+		}
+		return nil
+	}(); err != nil {
+		influxql.Iterators(itrs).Close()
+		return nil, err
+	}
+
+	return itrs, nil
+}
+
+// createVarRefSeriesIterator creates an iterator for a variable reference for a series.
+func (e *Engine) createVarRefSeriesIterator(ref *influxql.VarRef, mm *tsdb.Measurement, seriesKey string, t *influxql.TagSet, filter influxql.Expr, conditionFields []string, opt influxql.IteratorOptions) (influxql.Iterator, error) {
+	tags := influxql.NewTags(e.index.TagsForSeries(seriesKey))
+
+	// Create options specific for this series.
+	itrOpt := opt
+	itrOpt.Condition = filter
+
+	// Build auxilary cursors.
+	// Tag values should be returned if the field doesn't exist.
+	var aux []cursorAt
+	if len(opt.Aux) > 0 {
+		aux = make([]cursorAt, len(opt.Aux))
+		for i := range aux {
+			// Create cursor from field.
+			cur := e.buildCursor(mm.Name, seriesKey, opt.Aux[i], opt)
+			if cur != nil {
+				aux[i] = newBufCursor(cur, opt.Ascending)
+				continue
+			}
+
+			// If field doesn't exist, use the tag value.
+			// However, if the tag value is blank then return a null.
+			if v := tags.Value(opt.Aux[i]); v == "" {
+				aux[i] = &stringNilLiteralCursor{}
+			} else {
+				aux[i] = &stringLiteralCursor{value: v}
+			}
+		}
+	}
+
+	// Build conditional field cursors.
+	// If a conditional field doesn't exist then ignore the series.
+	var conds []cursorAt
+	if len(conditionFields) > 0 {
+		conds = make([]cursorAt, len(conditionFields))
+		for i := range conds {
+			cur := e.buildCursor(mm.Name, seriesKey, conditionFields[i], opt)
+			if cur != nil {
+				conds[i] = newBufCursor(cur, opt.Ascending)
+				continue
+			}
+
+			// If field doesn't exist, use the tag value.
+			// However, if the tag value is blank then return a null.
+			if v := tags.Value(conditionFields[i]); v == "" {
+				conds[i] = &stringNilLiteralCursor{}
+			} else {
+				conds[i] = &stringLiteralCursor{value: v}
+			}
+		}
+	}
+
+	// Limit tags to only the dimensions selected.
+	tags = tags.Subset(opt.Dimensions)
+
+	// If it's only auxiliary fields then it doesn't matter what type of iterator we use.
+	if ref == nil {
+		return newFloatIterator(mm.Name, tags, itrOpt, nil, aux, conds, conditionFields), nil
+	}
+
+	// Build main cursor.
+	cur := e.buildCursor(mm.Name, seriesKey, ref.Val, opt)
+
+	// If the field doesn't exist then don't build an iterator.
+	if cur == nil {
+		return nil, nil
+	}
+
+	switch cur := cur.(type) {
+	case floatCursor:
+		return newFloatIterator(mm.Name, tags, itrOpt, cur, aux, conds, conditionFields), nil
+	case integerCursor:
+		return newIntegerIterator(mm.Name, tags, itrOpt, cur, aux, conds, conditionFields), nil
+	case stringCursor:
+		return newStringIterator(mm.Name, tags, itrOpt, cur, aux, conds, conditionFields), nil
+	case booleanCursor:
+		return newBooleanIterator(mm.Name, tags, itrOpt, cur, aux, conds, conditionFields), nil
 	default:
-		key = c.tsmKeyBuf
-		value = c.tsmValueBuf
-		c.tsmKeyBuf, c.tsmValueBuf = c.nextTSM()
-	}
-
-	return key, value
-}
-
-// nextCache returns the next value from the cache.
-func (c *devCursor) nextCache() (int64, interface{}) {
-	if c.ascending {
-		c.cachePos++
-		if c.cachePos >= len(c.cache) {
-			return tsdb.EOF, nil
-		}
-		return c.cache[c.cachePos].UnixNano(), c.cache[c.cachePos].Value()
-	} else {
-		c.cachePos--
-		if c.cachePos < 0 {
-			return tsdb.EOF, nil
-		}
-		return c.cache[c.cachePos].UnixNano(), c.cache[c.cachePos].Value()
+		panic("unreachable")
 	}
 }
 
-// nextTSM returns the next value from the TSM files.
-func (c *devCursor) nextTSM() (int64, interface{}) {
-	if c.ascending {
-		c.tsmPos++
-		if c.tsmPos >= len(c.tsmValues) {
-			c.tsmValues, _ = c.tsmKeyCursor.Next(c.ascending)
-			if len(c.tsmValues) == 0 {
-				return tsdb.EOF, nil
-			}
-			c.tsmPos = 0
-		}
-		return c.tsmValues[c.tsmPos].UnixNano(), c.tsmValues[c.tsmPos].Value()
-	} else {
-		c.tsmPos--
-		if c.tsmPos < 0 {
-			c.tsmValues, _ = c.tsmKeyCursor.Next(c.ascending)
-			if len(c.tsmValues) == 0 {
-				return tsdb.EOF, nil
-			}
-			c.tsmPos = len(c.tsmValues) - 1
-		}
-		return c.tsmValues[c.tsmPos].UnixNano(), c.tsmValues[c.tsmPos].Value()
+// buildCursor creates an untyped cursor for a field.
+func (e *Engine) buildCursor(measurement, seriesKey, field string, opt influxql.IteratorOptions) cursor {
+	// Look up fields for measurement.
+	mf := e.measurementFields[measurement]
+	if mf == nil {
+		return nil
 	}
+
+	// Find individual field.
+	f := mf.Field(field)
+	if f == nil {
+		return nil
+	}
+
+	// Return appropriate cursor based on type.
+	switch f.Type {
+	case influxql.Float:
+		return e.buildFloatCursor(measurement, seriesKey, field, opt)
+	case influxql.Integer:
+		return e.buildIntegerCursor(measurement, seriesKey, field, opt)
+	case influxql.String:
+		return e.buildStringCursor(measurement, seriesKey, field, opt)
+	case influxql.Boolean:
+		return e.buildBooleanCursor(measurement, seriesKey, field, opt)
+	default:
+		panic("unreachable")
+	}
+}
+
+// buildFloatCursor creates a cursor for a float field.
+func (e *Engine) buildFloatCursor(measurement, seriesKey, field string, opt influxql.IteratorOptions) floatCursor {
+	cacheValues := e.Cache.Values(SeriesFieldKey(seriesKey, field))
+	keyCursor := e.KeyCursor(SeriesFieldKey(seriesKey, field), opt.SeekTime(), opt.Ascending)
+	return newFloatCursor(opt.SeekTime(), opt.Ascending, cacheValues, keyCursor)
+}
+
+// buildIntegerCursor creates a cursor for an integer field.
+func (e *Engine) buildIntegerCursor(measurement, seriesKey, field string, opt influxql.IteratorOptions) integerCursor {
+	cacheValues := e.Cache.Values(SeriesFieldKey(seriesKey, field))
+	keyCursor := e.KeyCursor(SeriesFieldKey(seriesKey, field), opt.SeekTime(), opt.Ascending)
+	return newIntegerCursor(opt.SeekTime(), opt.Ascending, cacheValues, keyCursor)
+}
+
+// buildStringCursor creates a cursor for a string field.
+func (e *Engine) buildStringCursor(measurement, seriesKey, field string, opt influxql.IteratorOptions) stringCursor {
+	cacheValues := e.Cache.Values(SeriesFieldKey(seriesKey, field))
+	keyCursor := e.KeyCursor(SeriesFieldKey(seriesKey, field), opt.SeekTime(), opt.Ascending)
+	return newStringCursor(opt.SeekTime(), opt.Ascending, cacheValues, keyCursor)
+}
+
+// buildBooleanCursor creates a cursor for a boolean field.
+func (e *Engine) buildBooleanCursor(measurement, seriesKey, field string, opt influxql.IteratorOptions) booleanCursor {
+	cacheValues := e.Cache.Values(SeriesFieldKey(seriesKey, field))
+	keyCursor := e.KeyCursor(SeriesFieldKey(seriesKey, field), opt.SeekTime(), opt.Ascending)
+	return newBooleanCursor(opt.SeekTime(), opt.Ascending, cacheValues, keyCursor)
 }
 
 // SeriesFieldKey combine a series key and field name for a unique string to be hashed to a numeric ID
@@ -847,9 +1039,9 @@ func tsmFieldTypeToInfluxQLDataType(typ byte) (influxql.DataType, error) {
 	switch typ {
 	case BlockFloat64:
 		return influxql.Float, nil
-	case BlockInt64:
+	case BlockInteger:
 		return influxql.Integer, nil
-	case BlockBool:
+	case BlockBoolean:
 		return influxql.Boolean, nil
 	case BlockString:
 		return influxql.String, nil
