@@ -1,8 +1,11 @@
 package tsm1
 
 import (
+	"expvar"
 	"fmt"
+	"io"
 	"log"
+	"math"
 	"os"
 	"path/filepath"
 	"sort"
@@ -10,6 +13,9 @@ import (
 	"strings"
 	"sync"
 	"time"
+
+	"github.com/influxdata/influxdb"
+	"github.com/influxdata/influxdb/tsdb"
 )
 
 type TSMFile interface {
@@ -18,39 +24,53 @@ type TSMFile interface {
 	Path() string
 
 	// Read returns all the values in the block where time t resides
-	Read(key string, t time.Time) ([]Value, error)
+	Read(key string, t int64) ([]Value, error)
 
-	// Read returns all the values in the block identified by entry.
+	// ReadAt returns all the values in the block identified by entry.
 	ReadAt(entry *IndexEntry, values []Value) ([]Value, error)
+	ReadFloatBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *FloatDecoder, values *[]FloatValue) ([]FloatValue, error)
+	ReadIntegerBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *IntegerDecoder, values *[]IntegerValue) ([]IntegerValue, error)
+	ReadStringBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *StringDecoder, values *[]StringValue) ([]StringValue, error)
+	ReadBooleanBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *BooleanDecoder, values *[]BooleanValue) ([]BooleanValue, error)
 
 	// Entries returns the index entries for all blocks for the given key.
-	Entries(key string) []*IndexEntry
+	Entries(key string) []IndexEntry
+	ReadEntries(key string, entries *[]IndexEntry)
 
 	// Returns true if the TSMFile may contain a value with the specified
 	// key and time
-	ContainsValue(key string, t time.Time) bool
+	ContainsValue(key string, t int64) bool
 
 	// Contains returns true if the file contains any values for the given
 	// key.
 	Contains(key string) bool
 
 	// TimeRange returns the min and max time across all keys in the file.
-	TimeRange() (time.Time, time.Time)
+	TimeRange() (int64, int64)
+
+	// TombstoneRange returns ranges of time that are deleted for the given key.
+	TombstoneRange(key string) []TimeRange
 
 	// KeyRange returns the min and max keys in the file.
 	KeyRange() (string, string)
 
-	// Keys returns all keys contained in the file.
-	Keys() []string
+	// KeyCount returns the number of distict keys in the file.
+	KeyCount() int
+
+	// KeyAt returns the key located at index position idx
+	KeyAt(idx int) (string, byte)
 
 	// Type returns the block type of the values stored for the key.  Returns one of
-	// BlockFloat64, BlockInt64, BlockBool, BlockString.  If key does not exist,
+	// BlockFloat64, BlockInt64, BlockBoolean, BlockString.  If key does not exist,
 	// an error is returned.
 	Type(key string) (byte, error)
 
 	// Delete removes the keys from the set of keys available in this file.
 	Delete(keys []string) error
 
+	// DeleteRange removes the values for keys between min and max.
+	DeleteRange(keys []string, min, max int64) error
+
 	// HasTombstones returns true if file contains values that have been deleted.
 	HasTombstones() bool
 
@@ -75,6 +95,11 @@ type TSMFile interface {
 	BlockIterator() *BlockIterator
 }
 
+// Statistics gathered by the FileStore.
+const (
+	statFileStoreBytes = "diskBytes"
+)
+
 type FileStore struct {
 	mu           sync.RWMutex
 	lastModified time.Time
@@ -86,20 +111,21 @@ type FileStore struct {
 
 	Logger       *log.Logger
 	traceLogging bool
+
+	statMap *expvar.Map
 }
 
 type FileStat struct {
 	Path             string
 	HasTombstone     bool
 	Size             uint32
-	LastModified     time.Time
-	MinTime, MaxTime time.Time
+	LastModified     int64
+	MinTime, MaxTime int64
 	MinKey, MaxKey   string
 }
 
-func (f FileStat) OverlapsTimeRange(min, max time.Time) bool {
-	return (f.MinTime.Equal(max) || f.MinTime.Before(max)) &&
-		(f.MaxTime.Equal(min) || f.MaxTime.After(min))
+func (f FileStat) OverlapsTimeRange(min, max int64) bool {
+	return f.MinTime <= max && f.MaxTime >= min
 }
 
 func (f FileStat) OverlapsKeyRange(min, max string) bool {
@@ -111,13 +137,25 @@ func (f FileStat) ContainsKey(key string) bool {
 }
 
 func NewFileStore(dir string) *FileStore {
+	db, rp := tsdb.DecodeStorePath(dir)
 	return &FileStore{
 		dir:          dir,
 		lastModified: time.Now(),
-		Logger:       log.New(os.Stderr, "[filestore]", log.LstdFlags),
+		Logger:       log.New(os.Stderr, "[filestore] ", log.LstdFlags),
+		statMap: influxdb.NewStatistics(
+			"tsm1_filestore:"+dir,
+			"tsm1_filestore",
+			map[string]string{"path": dir, "database": db, "retentionPolicy": rp},
+		),
 	}
 }
 
+// SetLogOutput sets the logger used for all messages. It must not be called
+// after the Open method has been called.
+func (f *FileStore) SetLogOutput(w io.Writer) {
+	f.Logger = log.New(w, "[filestore] ", log.LstdFlags)
+}
+
 // Returns the number of TSM files currently loaded
 func (f *FileStore) Count() int {
 	f.mu.RLock()
@@ -125,6 +163,13 @@ func (f *FileStore) Count() int {
 	return len(f.files)
 }
 
+// Files returns TSM files currently loaded.
+func (f *FileStore) Files() []TSMFile {
+	f.mu.RLock()
+	defer f.mu.RUnlock()
+	return f.files
+}
+
 // CurrentGeneration returns the current generation of the TSM files
 func (f *FileStore) CurrentGeneration() int {
 	f.mu.RLock()
@@ -143,6 +188,9 @@ func (f *FileStore) NextGeneration() int {
 func (f *FileStore) Add(files ...TSMFile) {
 	f.mu.Lock()
 	defer f.mu.Unlock()
+	for _, file := range files {
+		f.statMap.Add(statFileStoreBytes, int64(file.Size()))
+	}
 	f.files = append(f.files, files...)
 	sort.Sort(tsmReaders(f.files))
 }
@@ -164,29 +212,46 @@ func (f *FileStore) Remove(paths ...string) {
 
 		if keep {
 			active = append(active, file)
+		} else {
+			// Removing the file, remove the file size from the total file store bytes
+			f.statMap.Add(statFileStoreBytes, -int64(file.Size()))
 		}
 	}
 	f.files = active
 	sort.Sort(tsmReaders(f.files))
 }
 
-func (f *FileStore) Keys() []string {
+// WalkKeys calls fn for every key in every TSM file known to the FileStore.  If the key
+// exists in multiple files, it will be invoked for each file.
+func (f *FileStore) WalkKeys(fn func(key string, typ byte) error) error {
 	f.mu.RLock()
 	defer f.mu.RUnlock()
 
-	uniqueKeys := map[string]struct{}{}
 	for _, f := range f.files {
-		for _, key := range f.Keys() {
-			uniqueKeys[key] = struct{}{}
+		for i := 0; i < f.KeyCount(); i++ {
+			key, typ := f.KeyAt(i)
+			if err := fn(key, typ); err != nil {
+				return err
+			}
+		}
+	}
+	return nil
+}
+
+// Keys returns all keys and types for all files
+func (f *FileStore) Keys() map[string]byte {
+	f.mu.RLock()
+	defer f.mu.RUnlock()
+
+	uniqueKeys := map[string]byte{}
+	for _, f := range f.files {
+		for i := 0; i < f.KeyCount(); i++ {
+			key, typ := f.KeyAt(i)
+			uniqueKeys[key] = typ
 		}
 	}
 
-	var keys []string
-	for key := range uniqueKeys {
-		keys = append(keys, key)
-	}
-	sort.Strings(keys)
-	return keys
+	return uniqueKeys
 }
 
 func (f *FileStore) Type(key string) (byte, error) {
@@ -202,13 +267,18 @@ func (f *FileStore) Type(key string) (byte, error) {
 }
 
 func (f *FileStore) Delete(keys []string) error {
+	return f.DeleteRange(keys, math.MinInt64, math.MaxInt64)
+}
+
+// DeleteRange removes the values for keys between min and max.
+func (f *FileStore) DeleteRange(keys []string, min, max int64) error {
 	f.mu.Lock()
 	defer f.mu.Unlock()
 
 	f.lastModified = time.Now()
 
 	for _, file := range f.files {
-		if err := file.Delete(keys); err != nil {
+		if err := file.DeleteRange(keys, min, max); err != nil {
 			return err
 		}
 	}
@@ -252,11 +322,14 @@ func (f *FileStore) Open() error {
 			return fmt.Errorf("error opening file %s: %v", fn, err)
 		}
 
+		// Accumulate file store size stat
+		if fi, err := file.Stat(); err == nil {
+			f.statMap.Add(statFileStoreBytes, fi.Size())
+		}
+
 		go func(idx int, file *os.File) {
 			start := time.Now()
-			df, err := NewTSMReaderWithOptions(TSMReaderOptions{
-				MMAPFile: file,
-			})
+			df, err := NewTSMReader(file)
 			if f.traceLogging {
 				f.Logger.Printf("%s (#%d) opened in %v", file.Name(), idx, time.Now().Sub(start))
 			}
@@ -295,7 +368,7 @@ func (f *FileStore) Close() error {
 	return nil
 }
 
-func (f *FileStore) Read(key string, t time.Time) ([]Value, error) {
+func (f *FileStore) Read(key string, t int64) ([]Value, error) {
 	f.mu.RLock()
 	defer f.mu.RUnlock()
 
@@ -318,10 +391,10 @@ func (f *FileStore) Read(key string, t time.Time) ([]Value, error) {
 	return nil, nil
 }
 
-func (f *FileStore) KeyCursor(key string) *KeyCursor {
+func (f *FileStore) KeyCursor(key string, t int64, ascending bool) *KeyCursor {
 	f.mu.RLock()
 	defer f.mu.RUnlock()
-	return &KeyCursor{key: key, fs: f}
+	return newKeyCursor(f, key, t, ascending)
 }
 
 func (f *FileStore) Stats() []FileStat {
@@ -366,9 +439,7 @@ func (f *FileStore) Replace(oldFiles, newFiles []string) error {
 			return err
 		}
 
-		tsm, err := NewTSMReaderWithOptions(TSMReaderOptions{
-			MMAPFile: fd,
-		})
+		tsm, err := NewTSMReader(fd)
 		if err != nil {
 			return err
 		}
@@ -398,9 +469,22 @@ func (f *FileStore) Replace(oldFiles, newFiles []string) error {
 		}
 	}
 
+	if err := syncDir(f.dir); err != nil {
+		return err
+	}
+
 	f.files = active
 	sort.Sort(tsmReaders(f.files))
 
+	// Recalculate the disk size stat
+	var totalSize int64
+	for _, file := range f.files {
+		totalSize += int64(file.Size())
+	}
+	sizeStat := new(expvar.Int)
+	sizeStat.Set(totalSize)
+	f.statMap.Set(statFileStoreBytes, sizeStat)
+
 	return nil
 }
 
@@ -433,7 +517,7 @@ func (f *FileStore) BlockCount(path string, idx int) int {
 					return 0
 				}
 			}
-			_, _, _, block, _ := iter.Read()
+			_, _, _, _, block, _ := iter.Read()
 			return BlockCount(block)
 		}
 	}
@@ -442,44 +526,59 @@ func (f *FileStore) BlockCount(path string, idx int) int {
 
 // locations returns the files and index blocks for a key and time.  ascending indicates
 // whether the key will be scan in ascending time order or descenging time order.
-func (f *FileStore) locations(key string, t time.Time, ascending bool) []*location {
-	var locations []*location
+// This function assumes the read-lock has been taken.
+func (f *FileStore) locations(key string, t int64, ascending bool) []location {
+	var locations []location
 
-	f.mu.RLock()
 	filesSnapshot := make([]TSMFile, len(f.files))
 	for i := range f.files {
 		filesSnapshot[i] = f.files[i]
 	}
-	f.mu.RUnlock()
 
+	var entries []IndexEntry
 	for _, fd := range filesSnapshot {
 		minTime, maxTime := fd.TimeRange()
 
+		tombstones := fd.TombstoneRange(key)
 		// If we ascending and the max time of the file is before where we want to start
 		// skip it.
-		if ascending && maxTime.Before(t) {
+		if ascending && maxTime < t {
 			continue
 			// If we are descending and the min time fo the file is after where we want to start,
 			// then skip it.
-		} else if !ascending && minTime.After(t) {
+		} else if !ascending && minTime > t {
 			continue
 		}
 
 		// This file could potential contain points we are looking for so find the blocks for
 		// the given key.
-		for _, ie := range fd.Entries(key) {
+		fd.ReadEntries(key, &entries)
+		for _, ie := range entries {
+
+			// Skip any blocks only contain values that are tombstoned.
+			var skip bool
+			for _, t := range tombstones {
+				if t.Min <= ie.MinTime && t.Max >= ie.MaxTime {
+					skip = true
+					break
+				}
+			}
+
+			if skip {
+				continue
+			}
 			// If we ascending and the max time of a block is before where we are looking, skip
 			// it since the data is out of our range
-			if ascending && ie.MaxTime.Before(t) {
+			if ascending && ie.MaxTime < t {
 				continue
 				// If we descending and the min time of a block is after where we are looking, skip
 				// it since the data is out of our range
-			} else if !ascending && minTime.After(t) {
+			} else if !ascending && ie.MinTime > t {
 				continue
 			}
 
 			// Otherwise, add this file and block location
-			locations = append(locations, &location{
+			locations = append(locations, location{
 				r:     fd,
 				entry: ie,
 			})
@@ -514,12 +613,12 @@ type KeyCursor struct {
 	fs  *FileStore
 
 	// seeks is all the file locations that we need to return during iteration.
-	seeks []*location
+	seeks []location
 
 	// current is the set of blocks possibly containing the next set of points.
 	// Normally this is just one entry, but there may be multiple if points have
 	// been overwritten.
-	current []*location
+	current []location
 	buf     []Value
 
 	// pos is the index within seeks.  Based on ascending, it will increment or
@@ -527,9 +626,6 @@ type KeyCursor struct {
 	pos       int
 	ascending bool
 
-	// ready indicates that we know the files and blocks to seek to for the key.
-	ready bool
-
 	// duplicates is a hint that there are overlapping blocks for this key in
 	// multiple files (e.g. points have been overwritten but not fully compacted)
 	// If this is true, we need to scan the duplicate blocks and dedup the points
@@ -539,38 +635,27 @@ type KeyCursor struct {
 
 type location struct {
 	r     TSMFile
-	entry *IndexEntry
+	entry IndexEntry
 
 	// Has this location been read before
 	read bool
 }
 
-func (c *KeyCursor) init(t time.Time, ascending bool) {
-	if c.ready {
-		// Re-set the read status of each blocks in case a cursor is Seeked to
-		// multiple times
-		for _, c := range c.seeks {
-			c.read = false
-		}
-		return
+// newKeyCursor returns a new instance of KeyCursor.
+// This function assumes the read-lock has been taken.
+func newKeyCursor(fs *FileStore, key string, t int64, ascending bool) *KeyCursor {
+	c := &KeyCursor{
+		key:       key,
+		fs:        fs,
+		seeks:     fs.locations(key, t, ascending),
+		ascending: ascending,
 	}
-	c.ascending = ascending
-	c.seeks = c.fs.locations(c.key, t, ascending)
-
-	if len(c.seeks) > 0 {
-		for i := 1; i < len(c.seeks); i++ {
-			prev := c.seeks[i-1]
-			cur := c.seeks[i]
-
-			if prev.entry.MaxTime.Equal(cur.entry.MinTime) || prev.entry.MaxTime.After(cur.entry.MinTime) {
-				c.duplicates = true
-				break
-			}
-		}
-	}
-	c.ready = true
+	c.duplicates = c.hasOverlappingBlocks()
+	c.seek(t)
+	return c
 }
 
+// Close removes all references on the cursor.
 func (c *KeyCursor) Close() {
 	c.buf = nil
 	c.seeks = nil
@@ -578,160 +663,388 @@ func (c *KeyCursor) Close() {
 	c.current = nil
 }
 
-func (c *KeyCursor) SeekTo(t time.Time, ascending bool) ([]Value, error) {
-	c.init(t, ascending)
+// hasOverlappingBlocks returns true if blocks have overlapping time ranges.
+// This result is computed once and stored as the "duplicates" field.
+func (c *KeyCursor) hasOverlappingBlocks() bool {
 	if len(c.seeks) == 0 {
-		return nil, nil
+		return false
+	}
+
+	for i := 1; i < len(c.seeks); i++ {
+		prev := c.seeks[i-1]
+		cur := c.seeks[i]
+		if prev.entry.MaxTime >= cur.entry.MinTime {
+			return true
+		}
+	}
+	return false
+}
+
+// seek positions the cursor at the given time.
+func (c *KeyCursor) seek(t int64) {
+	if len(c.seeks) == 0 {
+		return
 	}
 	c.current = nil
 
-	if ascending {
-		for i, e := range c.seeks {
-			if t.Before(e.entry.MinTime) || e.entry.Contains(t) {
-				// Record the position of the first block matching our seek time
-				if len(c.current) == 0 {
-					c.pos = i
-				}
-
-				c.current = append(c.current, e)
-
-				// If we don't have duplicates, break.  Otherwise, keep looking for additional blocks containing
-				// this point.
-				if !c.duplicates {
-					break
-				}
-			}
-		}
+	if c.ascending {
+		c.seekAscending(t)
 	} else {
-		for i := len(c.seeks) - 1; i >= 0; i-- {
-			e := c.seeks[i]
-			if t.After(e.entry.MaxTime) || e.entry.Contains(t) {
-				// Record the position of the first block matching our seek time
-				if len(c.current) == 0 {
-					c.pos = i
-				}
+		c.seekDescending(t)
+	}
+}
 
-				c.current = append(c.current, e)
+func (c *KeyCursor) seekAscending(t int64) {
+	for i, e := range c.seeks {
+		if t < e.entry.MinTime || e.entry.Contains(t) {
+			// Record the position of the first block matching our seek time
+			if len(c.current) == 0 {
+				c.pos = i
+			}
 
-				// If we don't have duplicates, break.  Otherwise, keep looking for additional blocks containing
-				// this point.
-				if !c.duplicates {
-					break
-				}
+			c.current = append(c.current, e)
+
+			// Exit if we don't have duplicates.
+			// Otherwise, keep looking for additional blocks containing this point.
+			if !c.duplicates {
+				return
 			}
 		}
 	}
-
-	return c.readAt()
 }
 
-func (c *KeyCursor) readAt() ([]Value, error) {
+func (c *KeyCursor) seekDescending(t int64) {
+	for i := len(c.seeks) - 1; i >= 0; i-- {
+		e := c.seeks[i]
+		if t > e.entry.MaxTime || e.entry.Contains(t) {
+			// Record the position of the first block matching our seek time
+			if len(c.current) == 0 {
+				c.pos = i
+			}
+			c.current = append(c.current, e)
+
+			// Exit if we don't have duplicates.
+			// Otherwise, keep looking for additional blocks containing this point.
+			if !c.duplicates {
+				return
+			}
+		}
+	}
+}
+
+// Next moves the cursor to the next position.
+// Data should be read by the ReadBlock functions.
+func (c *KeyCursor) Next() {
+	c.current = c.current[:0]
+	if c.ascending {
+		c.nextAscending()
+	} else {
+		c.nextDescending()
+	}
+}
+
+func (c *KeyCursor) nextAscending() {
+	for {
+		c.pos++
+		if c.pos >= len(c.seeks) {
+			return
+		} else if !c.seeks[c.pos].read {
+			break
+		}
+	}
+
+	// Append the first matching block
+	if len(c.current) == 0 {
+		c.current = append(c.current, location{})
+	} else {
+		c.current = c.current[:1]
+	}
+	c.current[0] = c.seeks[c.pos]
+
+	// We're done if there are no overlapping blocks.
+	if !c.duplicates {
+		return
+	}
+
+	// If we have ovelapping blocks, append all their values so we can dedup
+	first := c.seeks[c.pos]
+	for i := c.pos + 1; i < len(c.seeks); i++ {
+		if c.seeks[i].read {
+			continue
+		}
+
+		if c.seeks[i].entry.MinTime <= first.entry.MaxTime {
+			c.current = append(c.current, c.seeks[i])
+		}
+	}
+}
+
+func (c *KeyCursor) nextDescending() {
+	for {
+		c.pos--
+		if c.pos < 0 {
+			return
+		} else if !c.seeks[c.pos].read {
+			break
+		}
+	}
+
+	// Append the first matching block
+	if len(c.current) == 0 {
+		c.current = make([]location, 1)
+	} else {
+		c.current = c.current[:1]
+	}
+	c.current[0] = c.seeks[c.pos]
+
+	// We're done if there are no overlapping blocks.
+	if !c.duplicates {
+		return
+	}
+
+	// If we have ovelapping blocks, append all their values so we can dedup
+	first := c.seeks[c.pos]
+	for i := c.pos; i >= 0; i-- {
+		if c.seeks[i].read {
+			continue
+		}
+		if c.seeks[i].entry.MaxTime >= first.entry.MinTime {
+			c.current = append(c.current, c.seeks[i])
+		}
+	}
+}
+
+// ReadFloatBlock reads the next block as a set of float values.
+func (c *KeyCursor) ReadFloatBlock(tdec *TimeDecoder, fdec *FloatDecoder, buf *[]FloatValue) ([]FloatValue, error) {
 	// No matching blocks to decode
 	if len(c.current) == 0 {
 		return nil, nil
 	}
 
 	// First block is the oldest block containing the points we're search for.
-	first := c.current[0]
-	values, err := first.r.ReadAt(first.entry, c.buf[:0])
+	first := &c.current[0]
+	*buf = (*buf)[:0]
+	values, err := first.r.ReadFloatBlockAt(&first.entry, tdec, fdec, buf)
 	first.read = true
 
+	tombstones := first.r.TombstoneRange(c.key)
+
 	// Only one block with this key and time range so return it
 	if len(c.current) == 1 {
-		return values, err
+		return c.filterFloatValues(tombstones, values), err
 	}
 
 	// Otherwise, search the remaining blocks that overlap and append their values so we can
 	// dedup them.
 	for i := 1; i < len(c.current); i++ {
 		cur := c.current[i]
-		if c.ascending && cur.entry.OverlapsTimeRange(first.entry.MinTime, first.entry.MaxTime) && !cur.read {
+		tombstones := cur.r.TombstoneRange(c.key)
+
+		if c.ascending && !cur.read {
 			cur.read = true
 			c.pos++
-			v, err := cur.r.ReadAt(cur.entry, nil)
+
+			var a []FloatValue
+			v, err := cur.r.ReadFloatBlockAt(&cur.entry, tdec, fdec, &a)
 			if err != nil {
 				return nil, err
 			}
-			values = append(values, v...)
-		} else if !c.ascending && cur.entry.OverlapsTimeRange(first.entry.MinTime, first.entry.MaxTime) && !cur.read {
+			values = append(values, c.filterFloatValues(tombstones, v)...)
+		} else if !c.ascending && !cur.read {
 			cur.read = true
 			c.pos--
 
-			v, err := cur.r.ReadAt(cur.entry, nil)
+			var a []FloatValue
+			v, err := cur.r.ReadFloatBlockAt(&cur.entry, tdec, fdec, &a)
 			if err != nil {
 				return nil, err
 			}
-			values = append(v, values...)
+			values = append(c.filterFloatValues(tombstones, v), values...)
 		}
 	}
 
-	return Values(values).Deduplicate(), err
+	return FloatValues(values).Deduplicate(), err
 }
 
-func (c *KeyCursor) Next(ascending bool) ([]Value, error) {
-	c.current = c.current[:0]
-
-	if ascending {
-		for {
-			c.pos++
-			if c.pos >= len(c.seeks) {
-				return nil, nil
-			}
-
-			if !c.seeks[c.pos].read {
-				break
-			}
-		}
-
-		// Append the first matching block
-		c.current = []*location{c.seeks[c.pos]}
-
-		// If we have ovelapping blocks, append all their values so we can dedup
-		if c.duplicates {
-			first := c.seeks[c.pos]
-			for i := c.pos + 1; i < len(c.seeks); i++ {
-				if c.seeks[i].read {
-					continue
-				}
-
-				if c.seeks[i].entry.MinTime.Before(first.entry.MaxTime) || c.seeks[i].entry.MinTime.Equal(first.entry.MaxTime) {
-					c.current = append(c.current, c.seeks[i])
-				}
-			}
-		}
-
-		return c.readAt()
-
-	} else {
-		for {
-			c.pos--
-			if c.pos < 0 {
-				return nil, nil
-			}
-
-			if !c.seeks[c.pos].read {
-				break
-			}
-		}
-
-		// Append the first matching block
-		c.current = []*location{c.seeks[c.pos]}
-
-		// If we have ovelapping blocks, append all their values so we can dedup
-		if c.duplicates {
-			first := c.seeks[c.pos]
-			for i := c.pos; i >= 0; i-- {
-				if c.seeks[i].read {
-					continue
-				}
-				if c.seeks[i].entry.MaxTime.After(first.entry.MinTime) || c.seeks[i].entry.MaxTime.Equal(first.entry.MinTime) {
-					c.current = append(c.current, c.seeks[i])
-				}
-			}
-		}
-
-		return c.readAt()
+// ReadIntegerBlock reads the next block as a set of integer values.
+func (c *KeyCursor) ReadIntegerBlock(tdec *TimeDecoder, vdec *IntegerDecoder, buf *[]IntegerValue) ([]IntegerValue, error) {
+	// No matching blocks to decode
+	if len(c.current) == 0 {
+		return nil, nil
 	}
+
+	// First block is the oldest block containing the points we're search for.
+	first := &c.current[0]
+	*buf = (*buf)[:0]
+	values, err := first.r.ReadIntegerBlockAt(&first.entry, tdec, vdec, buf)
+	first.read = true
+
+	tombstones := first.r.TombstoneRange(c.key)
+
+	// Only one block with this key and time range so return it
+	if len(c.current) == 1 {
+		return c.filterIntegerValues(tombstones, values), err
+	}
+
+	// Otherwise, search the remaining blocks that overlap and append their values so we can
+	// dedup them.
+	for i := 1; i < len(c.current); i++ {
+		cur := c.current[i]
+		tombstones = cur.r.TombstoneRange(c.key)
+
+		if c.ascending && !cur.read {
+			cur.read = true
+			c.pos++
+
+			var a []IntegerValue
+			v, err := cur.r.ReadIntegerBlockAt(&cur.entry, tdec, vdec, &a)
+			if err != nil {
+				return nil, err
+			}
+			values = append(values, c.filterIntegerValues(tombstones, v)...)
+		} else if !c.ascending && !cur.read {
+			cur.read = true
+			c.pos--
+
+			var a []IntegerValue
+			v, err := cur.r.ReadIntegerBlockAt(&cur.entry, tdec, vdec, &a)
+			if err != nil {
+				return nil, err
+			}
+			values = append(c.filterIntegerValues(tombstones, v), values...)
+		}
+	}
+
+	return IntegerValues(values).Deduplicate(), err
+}
+
+// ReadStringBlock reads the next block as a set of string values.
+func (c *KeyCursor) ReadStringBlock(tdec *TimeDecoder, vdec *StringDecoder, buf *[]StringValue) ([]StringValue, error) {
+	// No matching blocks to decode
+	if len(c.current) == 0 {
+		return nil, nil
+	}
+
+	// First block is the oldest block containing the points we're search for.
+	first := &c.current[0]
+	*buf = (*buf)[:0]
+	values, err := first.r.ReadStringBlockAt(&first.entry, tdec, vdec, buf)
+	first.read = true
+
+	tombstones := first.r.TombstoneRange(c.key)
+
+	// Only one block with this key and time range so return it
+	if len(c.current) == 1 {
+		return c.filterStringValues(tombstones, values), err
+	}
+
+	// Otherwise, search the remaining blocks that overlap and append their values so we can
+	// dedup them.
+	for i := 1; i < len(c.current); i++ {
+		cur := c.current[i]
+		tombstones = cur.r.TombstoneRange(c.key)
+		if c.ascending && !cur.read {
+			cur.read = true
+			c.pos++
+			var a []StringValue
+			v, err := cur.r.ReadStringBlockAt(&cur.entry, tdec, vdec, &a)
+			if err != nil {
+				return nil, err
+			}
+			values = append(values, c.filterStringValues(tombstones, v)...)
+		} else if !c.ascending && !cur.read {
+			cur.read = true
+			c.pos--
+
+			var a []StringValue
+			v, err := cur.r.ReadStringBlockAt(&cur.entry, tdec, vdec, &a)
+			if err != nil {
+				return nil, err
+			}
+			values = append(c.filterStringValues(tombstones, v), values...)
+		}
+	}
+
+	return StringValues(values).Deduplicate(), err
+}
+
+// ReadBooleanBlock reads the next block as a set of boolean values.
+func (c *KeyCursor) ReadBooleanBlock(tdec *TimeDecoder, vdec *BooleanDecoder, buf *[]BooleanValue) ([]BooleanValue, error) {
+	// No matching blocks to decode
+	if len(c.current) == 0 {
+		return nil, nil
+	}
+
+	// First block is the oldest block containing the points we're search for.
+	first := &c.current[0]
+	*buf = (*buf)[:0]
+	values, err := first.r.ReadBooleanBlockAt(&first.entry, tdec, vdec, buf)
+	first.read = true
+
+	tombstones := first.r.TombstoneRange(c.key)
+
+	// Only one block with this key and time range so return it
+	if len(c.current) == 1 {
+		return c.filterBooleanValues(tombstones, values), err
+	}
+
+	// Otherwise, search the remaining blocks that overlap and append their values so we can
+	// dedup them.
+	for i := 1; i < len(c.current); i++ {
+		cur := c.current[i]
+		tombstones = cur.r.TombstoneRange(c.key)
+		if c.ascending && !cur.read {
+			cur.read = true
+			c.pos++
+
+			var a []BooleanValue
+			v, err := cur.r.ReadBooleanBlockAt(&cur.entry, tdec, vdec, &a)
+			if err != nil {
+				return nil, err
+			}
+			values = append(values, c.filterBooleanValues(tombstones, v)...)
+		} else if !c.ascending && !cur.read {
+			cur.read = true
+			c.pos--
+
+			var a []BooleanValue
+			v, err := cur.r.ReadBooleanBlockAt(&cur.entry, tdec, vdec, &a)
+			if err != nil {
+				return nil, err
+			}
+			values = append(c.filterBooleanValues(tombstones, v), values...)
+		}
+	}
+
+	return BooleanValues(values).Deduplicate(), err
+}
+
+func (c *KeyCursor) filterFloatValues(tombstones []TimeRange, values FloatValues) FloatValues {
+	for _, t := range tombstones {
+		values = values.Filter(t.Min, t.Max)
+	}
+	return values
+}
+
+func (c *KeyCursor) filterIntegerValues(tombstones []TimeRange, values IntegerValues) IntegerValues {
+	for _, t := range tombstones {
+		values = values.Filter(t.Min, t.Max)
+	}
+	return values
+}
+
+func (c *KeyCursor) filterStringValues(tombstones []TimeRange, values StringValues) StringValues {
+	for _, t := range tombstones {
+		values = values.Filter(t.Min, t.Max)
+	}
+	return values
+}
+
+func (c *KeyCursor) filterBooleanValues(tombstones []TimeRange, values BooleanValues) BooleanValues {
+	for _, t := range tombstones {
+		values = values.Filter(t.Min, t.Max)
+	}
+	return values
 }
 
 type tsmReaders []TSMFile
