@@ -2,13 +2,13 @@ package tsm1
 
 import (
 	"bytes"
+	"encoding/binary"
 	"fmt"
 	"io"
 	"math"
 	"os"
 	"sort"
 	"sync"
-	"time"
 )
 
 type TSMReader struct {
@@ -27,7 +27,66 @@ type TSMReader struct {
 	size int64
 
 	// lastModified is the last time this file was modified on disk
-	lastModified time.Time
+	lastModified int64
+}
+
+// TSMIndex represent the index section of a TSM file.  The index records all
+// blocks, their locations, sizes, min and max times.
+type TSMIndex interface {
+
+	// Delete removes the given keys from the index.
+	Delete(keys []string)
+
+	// DeleteRange removes the given keys with data between minTime and maxTime from the index.
+	DeleteRange(keys []string, minTime, maxTime int64)
+
+	// Contains return true if the given key exists in the index.
+	Contains(key string) bool
+
+	// ContainsValue returns true if key and time might exists in this file.  This function could
+	// return true even though the actual point does not exists.  For example, the key may
+	// exists in this file, but not have point exactly at time t.
+	ContainsValue(key string, timestamp int64) bool
+
+	// Entries returns all index entries for a key.
+	Entries(key string) []IndexEntry
+
+	// ReadEntries reads the index entries for key into entries.
+	ReadEntries(key string, entries *[]IndexEntry)
+
+	// Entry returns the index entry for the specified key and timestamp.  If no entry
+	// matches the key and timestamp, nil is returned.
+	Entry(key string, timestamp int64) *IndexEntry
+
+	// Key returns the key in the index at the given postion.
+	Key(index int) (string, []IndexEntry)
+
+	// KeyAt returns the key in the index at the given postion.
+	KeyAt(index int) (string, byte)
+
+	// KeyCount returns the count of unique keys in the index.
+	KeyCount() int
+
+	// Size returns the size of a the current index in bytes
+	Size() uint32
+
+	// TimeRange returns the min and max time across all keys in the file.
+	TimeRange() (int64, int64)
+
+	// TombstoneRange returns ranges of time that are deleted for the given key.
+	TombstoneRange(key string) []TimeRange
+
+	// KeyRange returns the min and max keys in the file.
+	KeyRange() (string, string)
+
+	// Type returns the block type of the values stored for the key.  Returns one of
+	// BlockFloat64, BlockInt64, BlockBool, BlockString.  If key does not exist,
+	// an error is returned.
+	Type(key string) (byte, error)
+
+	// UnmarshalBinary populates an index from an encoded byte slice
+	// representation of an index.
+	UnmarshalBinary(b []byte) error
 }
 
 // BlockIterator allows iterating over each block in a TSM file in order.  It provides
@@ -42,7 +101,7 @@ type BlockIterator struct {
 	n int
 
 	key     string
-	entries []*IndexEntry
+	entries []IndexEntry
 	err     error
 }
 
@@ -50,7 +109,7 @@ func (b *BlockIterator) PeekNext() string {
 	if len(b.entries) > 1 {
 		return b.key
 	} else if b.n-b.i > 1 {
-		key := b.r.KeyAt(b.i + 1)
+		key, _ := b.r.KeyAt(b.i + 1)
 		return key
 	}
 	return ""
@@ -71,84 +130,53 @@ func (b *BlockIterator) Next() bool {
 	if b.n-b.i > 0 {
 		b.key, b.entries = b.r.Key(b.i)
 		b.i++
-		return true
+
+		if len(b.entries) > 0 {
+			return true
+		}
 	}
 
 	return false
 }
 
-func (b *BlockIterator) Read() (string, time.Time, time.Time, []byte, error) {
+func (b *BlockIterator) Read() (string, int64, int64, uint32, []byte, error) {
 	if b.err != nil {
-		return "", time.Unix(0, 0), time.Unix(0, 0), nil, b.err
+		return "", 0, 0, 0, nil, b.err
 	}
-
-	buf, err := b.r.readBytes(b.entries[0], nil)
+	checksum, buf, err := b.r.readBytes(&b.entries[0], nil)
 	if err != nil {
-		return "", time.Unix(0, 0), time.Unix(0, 0), nil, err
+		return "", 0, 0, 0, nil, err
 	}
-	return b.key, b.entries[0].MinTime, b.entries[0].MaxTime, buf, err
+	return b.key, b.entries[0].MinTime, b.entries[0].MaxTime, checksum, buf, err
 }
 
 // blockAccessor abstracts a method of accessing blocks from a
 // TSM file.
 type blockAccessor interface {
-	init() (TSMIndex, error)
-	read(key string, timestamp time.Time) ([]Value, error)
+	init() (*indirectIndex, error)
+	read(key string, timestamp int64) ([]Value, error)
 	readAll(key string) ([]Value, error)
 	readBlock(entry *IndexEntry, values []Value) ([]Value, error)
-	readBytes(entry *IndexEntry, buf []byte) ([]byte, error)
+	readFloatBlock(entry *IndexEntry, tdec *TimeDecoder, fdec *FloatDecoder, values *[]FloatValue) ([]FloatValue, error)
+	readIntegerBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *IntegerDecoder, values *[]IntegerValue) ([]IntegerValue, error)
+	readStringBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *StringDecoder, values *[]StringValue) ([]StringValue, error)
+	readBooleanBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *BooleanDecoder, values *[]BooleanValue) ([]BooleanValue, error)
+	readBytes(entry *IndexEntry, buf []byte) (uint32, []byte, error)
 	path() string
 	close() error
 }
 
-type TSMReaderOptions struct {
-	// Reader is used to create file IO based reader.
-	Reader io.ReadSeeker
-
-	// MMAPFile is used to create an MMAP based reader.
-	MMAPFile *os.File
-}
-
-func NewTSMReader(r io.ReadSeeker) (*TSMReader, error) {
-	return NewTSMReaderWithOptions(
-		TSMReaderOptions{
-			Reader: r,
-		})
-}
-
-func NewTSMReaderWithOptions(opt TSMReaderOptions) (*TSMReader, error) {
+func NewTSMReader(f *os.File) (*TSMReader, error) {
 	t := &TSMReader{}
-	if opt.Reader != nil {
-		// Seek to the end of the file to determine the size
-		size, err := opt.Reader.Seek(2, 0)
-		if err != nil {
-			return nil, err
-		}
-		t.size = size
-		if f, ok := opt.Reader.(*os.File); ok {
-			stat, err := f.Stat()
-			if err != nil {
-				return nil, err
-			}
 
-			t.lastModified = stat.ModTime()
-		}
-		t.accessor = &fileAccessor{
-			r: opt.Reader,
-		}
-
-	} else if opt.MMAPFile != nil {
-		stat, err := opt.MMAPFile.Stat()
-		if err != nil {
-			return nil, err
-		}
-		t.size = stat.Size()
-		t.lastModified = stat.ModTime()
-		t.accessor = &mmapAccessor{
-			f: opt.MMAPFile,
-		}
-	} else {
-		panic("invalid options: need Reader or MMAPFile")
+	stat, err := f.Stat()
+	if err != nil {
+		return nil, err
+	}
+	t.size = stat.Size()
+	t.lastModified = stat.ModTime().UnixNano()
+	t.accessor = &mmapAccessor{
+		f: f,
 	}
 
 	index, err := t.accessor.init()
@@ -173,8 +201,26 @@ func (t *TSMReader) applyTombstones() error {
 		return fmt.Errorf("init: read tombstones: %v", err)
 	}
 
-	// Update our index
-	t.index.Delete(tombstones)
+	if len(tombstones) == 0 {
+		return nil
+	}
+
+	var cur, prev Tombstone
+	cur = tombstones[0]
+	batch := []string{cur.Key}
+	for i := 1; i < len(tombstones); i++ {
+		cur = tombstones[i]
+		prev = tombstones[i-1]
+		if prev.Min != cur.Min || prev.Max != cur.Max {
+			t.index.DeleteRange(batch, prev.Min, prev.Max)
+			batch = batch[:0]
+		}
+		batch = append(batch, cur.Key)
+	}
+
+	if len(batch) > 0 {
+		t.index.DeleteRange(batch, cur.Min, cur.Max)
+	}
 	return nil
 }
 
@@ -185,15 +231,12 @@ func (t *TSMReader) Path() string {
 	return t.accessor.path()
 }
 
-func (t *TSMReader) Keys() []string {
-	return t.index.Keys()
-}
-
-func (t *TSMReader) Key(index int) (string, []*IndexEntry) {
+func (t *TSMReader) Key(index int) (string, []IndexEntry) {
 	return t.index.Key(index)
 }
 
-func (t *TSMReader) KeyAt(idx int) string {
+// KeyAt returns the key and key type at position idx in the index.
+func (t *TSMReader) KeyAt(idx int) (string, byte) {
 	return t.index.KeyAt(idx)
 }
 
@@ -204,7 +247,31 @@ func (t *TSMReader) ReadAt(entry *IndexEntry, vals []Value) ([]Value, error) {
 	return t.accessor.readBlock(entry, vals)
 }
 
-func (t *TSMReader) Read(key string, timestamp time.Time) ([]Value, error) {
+func (t *TSMReader) ReadFloatBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *FloatDecoder, vals *[]FloatValue) ([]FloatValue, error) {
+	t.mu.RLock()
+	defer t.mu.RUnlock()
+	return t.accessor.readFloatBlock(entry, tdec, vdec, vals)
+}
+
+func (t *TSMReader) ReadIntegerBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *IntegerDecoder, vals *[]IntegerValue) ([]IntegerValue, error) {
+	t.mu.RLock()
+	defer t.mu.RUnlock()
+	return t.accessor.readIntegerBlock(entry, tdec, vdec, vals)
+}
+
+func (t *TSMReader) ReadStringBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *StringDecoder, vals *[]StringValue) ([]StringValue, error) {
+	t.mu.RLock()
+	defer t.mu.RUnlock()
+	return t.accessor.readStringBlock(entry, tdec, vdec, vals)
+}
+
+func (t *TSMReader) ReadBooleanBlockAt(entry *IndexEntry, tdec *TimeDecoder, vdec *BooleanDecoder, vals *[]BooleanValue) ([]BooleanValue, error) {
+	t.mu.RLock()
+	defer t.mu.RUnlock()
+	return t.accessor.readBooleanBlock(entry, tdec, vdec, vals)
+}
+
+func (t *TSMReader) Read(key string, timestamp int64) ([]Value, error) {
 	t.mu.RLock()
 	defer t.mu.RUnlock()
 
@@ -219,7 +286,7 @@ func (t *TSMReader) ReadAll(key string) ([]Value, error) {
 	return t.accessor.readAll(key)
 }
 
-func (t *TSMReader) readBytes(e *IndexEntry, b []byte) ([]byte, error) {
+func (t *TSMReader) readBytes(e *IndexEntry, b []byte) (uint32, []byte, error) {
 	t.mu.RLock()
 	defer t.mu.RUnlock()
 	return t.accessor.readBytes(e, b)
@@ -259,10 +326,20 @@ func (t *TSMReader) Contains(key string) bool {
 // ContainsValue returns true if key and time might exists in this file.  This function could
 // return true even though the actual point does not exists.  For example, the key may
 // exists in this file, but not have point exactly at time t.
-func (t *TSMReader) ContainsValue(key string, ts time.Time) bool {
+func (t *TSMReader) ContainsValue(key string, ts int64) bool {
 	return t.index.ContainsValue(key, ts)
 }
 
+// DeleteRange removes the given points for keys between minTime and maxTime
+func (t *TSMReader) DeleteRange(keys []string, minTime, maxTime int64) error {
+	if err := t.tombstoner.AddRange(keys, minTime, maxTime); err != nil {
+		return err
+	}
+
+	t.index.DeleteRange(keys, minTime, maxTime)
+	return nil
+}
+
 func (t *TSMReader) Delete(keys []string) error {
 	if err := t.tombstoner.Add(keys); err != nil {
 		return err
@@ -273,7 +350,7 @@ func (t *TSMReader) Delete(keys []string) error {
 }
 
 // TimeRange returns the min and max time across all keys in the file.
-func (t *TSMReader) TimeRange() (time.Time, time.Time) {
+func (t *TSMReader) TimeRange() (int64, int64) {
 	return t.index.TimeRange()
 }
 
@@ -286,10 +363,14 @@ func (t *TSMReader) KeyCount() int {
 	return t.index.KeyCount()
 }
 
-func (t *TSMReader) Entries(key string) []*IndexEntry {
+func (t *TSMReader) Entries(key string) []IndexEntry {
 	return t.index.Entries(key)
 }
 
+func (t *TSMReader) ReadEntries(key string, entries *[]IndexEntry) {
+	t.index.ReadEntries(key, entries)
+}
+
 func (t *TSMReader) IndexSize() uint32 {
 	return t.index.Size()
 }
@@ -300,7 +381,7 @@ func (t *TSMReader) Size() uint32 {
 	return uint32(t.size)
 }
 
-func (t *TSMReader) LastModified() time.Time {
+func (t *TSMReader) LastModified() int64 {
 	t.mu.RLock()
 	defer t.mu.RUnlock()
 	return t.lastModified
@@ -320,6 +401,13 @@ func (t *TSMReader) TombstoneFiles() []FileStat {
 	return t.tombstoner.TombstoneFiles()
 }
 
+// TombstoneRange returns ranges of time that are deleted for the given key.
+func (t *TSMReader) TombstoneRange(key string) []TimeRange {
+	t.mu.RLock()
+	defer t.mu.RUnlock()
+	return t.index.TombstoneRange(key)
+}
+
 func (t *TSMReader) Stats() FileStat {
 	minTime, maxTime := t.index.TimeRange()
 	minKey, maxKey := t.index.KeyRange()
@@ -393,20 +481,22 @@ type indirectIndex struct {
 
 	// minTime, maxTime are the minimum and maximum times contained in the file across all
 	// series.
-	minTime, maxTime time.Time
+	minTime, maxTime int64
+
+	// tombstones contains only the tombstoned keys with subset of time values deleted.  An
+	// entry would exist here if a subset of the points for a key were deleted and the file
+	// had not be re-compacted to remove the points on disk.
+	tombstones map[string][]TimeRange
 }
 
-func NewIndirectIndex() TSMIndex {
-	return &indirectIndex{}
+type TimeRange struct {
+	Min, Max int64
 }
 
-// Add records a new block entry for a key in the index.
-func (d *indirectIndex) Add(key string, blockType byte, minTime, maxTime time.Time, offset int64, size uint32) {
-	panic("unsupported operation")
-}
-
-func (d *indirectIndex) Write(w io.Writer) error {
-	panic("unsupported operation")
+func NewIndirectIndex() *indirectIndex {
+	return &indirectIndex{
+		tombstones: make(map[string][]TimeRange),
+	}
 }
 
 // search returns the index of i in offsets for where key is located.  If key is not
@@ -419,7 +509,7 @@ func (d *indirectIndex) search(key []byte) int {
 		offset := d.offsets[i]
 
 		// It's pointing to the start of the key which is a 2 byte length
-		keyLen := int32(btou16(d.b[offset : offset+2]))
+		keyLen := int32(binary.BigEndian.Uint16(d.b[offset : offset+2]))
 
 		// See if it matches
 		return bytes.Compare(d.b[offset+2:offset+2+keyLen], key) >= 0
@@ -449,7 +539,7 @@ func (d *indirectIndex) search(key []byte) int {
 }
 
 // Entries returns all index entries for a key.
-func (d *indirectIndex) Entries(key string) []*IndexEntry {
+func (d *indirectIndex) Entries(key string) []IndexEntry {
 	d.mu.RLock()
 	defer d.mu.RUnlock()
 
@@ -471,10 +561,9 @@ func (d *indirectIndex) Entries(key string) []*IndexEntry {
 
 		// Read and return all the entries
 		ofs += n
-		_, entries, err := readEntries(d.b[ofs:])
-		if err != nil {
+		var entries indexEntries
+		if _, err := readEntries(d.b[ofs:], &entries); err != nil {
 			panic(fmt.Sprintf("error reading entries: %v", err))
-
 		}
 		return entries.entries
 	}
@@ -483,31 +572,24 @@ func (d *indirectIndex) Entries(key string) []*IndexEntry {
 	return nil
 }
 
+// ReadEntries returns all index entries for a key.
+func (d *indirectIndex) ReadEntries(key string, entries *[]IndexEntry) {
+	*entries = d.Entries(key)
+}
+
 // Entry returns the index entry for the specified key and timestamp.  If no entry
 // matches the key an timestamp, nil is returned.
-func (d *indirectIndex) Entry(key string, timestamp time.Time) *IndexEntry {
+func (d *indirectIndex) Entry(key string, timestamp int64) *IndexEntry {
 	entries := d.Entries(key)
 	for _, entry := range entries {
 		if entry.Contains(timestamp) {
-			return entry
+			return &entry
 		}
 	}
 	return nil
 }
 
-func (d *indirectIndex) Keys() []string {
-	d.mu.RLock()
-	defer d.mu.RUnlock()
-
-	var keys []string
-	for _, offset := range d.offsets {
-		_, key, _ := readKey(d.b[offset:])
-		keys = append(keys, string(key))
-	}
-	return keys
-}
-
-func (d *indirectIndex) Key(idx int) (string, []*IndexEntry) {
+func (d *indirectIndex) Key(idx int) (string, []IndexEntry) {
 	d.mu.RLock()
 	defer d.mu.RUnlock()
 
@@ -515,19 +597,21 @@ func (d *indirectIndex) Key(idx int) (string, []*IndexEntry) {
 		return "", nil
 	}
 	n, key, _ := readKey(d.b[d.offsets[idx]:])
-	_, entries, _ := readEntries(d.b[int(d.offsets[idx])+n:])
+
+	var entries indexEntries
+	readEntries(d.b[int(d.offsets[idx])+n:], &entries)
 	return string(key), entries.entries
 }
 
-func (d *indirectIndex) KeyAt(idx int) string {
+func (d *indirectIndex) KeyAt(idx int) (string, byte) {
 	d.mu.RLock()
 	defer d.mu.RUnlock()
 
 	if idx < 0 || idx >= len(d.offsets) {
-		return ""
+		return "", 0
 	}
-	_, key, _ := readKey(d.b[d.offsets[idx]:])
-	return string(key)
+	n, key, _ := readKey(d.b[d.offsets[idx]:])
+	return string(key), d.b[d.offsets[idx]+int32(n)]
 }
 
 func (d *indirectIndex) KeyCount() int {
@@ -562,12 +646,89 @@ func (d *indirectIndex) Delete(keys []string) {
 	d.offsets = offsets
 }
 
+func (d *indirectIndex) DeleteRange(keys []string, minTime, maxTime int64) {
+	// No keys, nothing to do
+	if len(keys) == 0 {
+		return
+	}
+
+	// If we're deleting the max time range, just use tombstoning to remove the
+	// key from the offsets slice
+	if minTime == math.MinInt64 && maxTime == math.MaxInt64 {
+		d.Delete(keys)
+		return
+	}
+
+	// Is the range passed in outside of the time range for the file?
+	min, max := d.TimeRange()
+	if minTime > max || maxTime < min {
+		return
+	}
+
+	tombstones := map[string][]TimeRange{}
+	for _, k := range keys {
+
+		// Is the range passed in outside the time range for this key?
+		entries := d.Entries(k)
+
+		// If multiple tombstones are saved for the same key
+		if len(entries) == 0 {
+			continue
+		}
+
+		min, max := entries[0].MinTime, entries[len(entries)-1].MaxTime
+		if minTime > max || maxTime < min {
+			continue
+		}
+
+		// Is the range passed in cover every value for the key?
+		if minTime <= min && maxTime >= max {
+			d.Delete(keys)
+			continue
+		}
+
+		tombstones[k] = append(tombstones[k], TimeRange{minTime, maxTime})
+	}
+
+	if len(tombstones) == 0 {
+		return
+	}
+
+	d.mu.Lock()
+	for k, v := range tombstones {
+		d.tombstones[k] = append(d.tombstones[k], v...)
+	}
+	d.mu.Unlock()
+
+}
+
+func (d *indirectIndex) TombstoneRange(key string) []TimeRange {
+	d.mu.RLock()
+	r := d.tombstones[key]
+	d.mu.RUnlock()
+	return r
+}
+
 func (d *indirectIndex) Contains(key string) bool {
 	return len(d.Entries(key)) > 0
 }
 
-func (d *indirectIndex) ContainsValue(key string, timestamp time.Time) bool {
-	return d.Entry(key, timestamp) != nil
+func (d *indirectIndex) ContainsValue(key string, timestamp int64) bool {
+	entry := d.Entry(key, timestamp)
+	if entry == nil {
+		return false
+	}
+
+	d.mu.RLock()
+	tombstones := d.tombstones[key]
+	d.mu.RUnlock()
+
+	for _, t := range tombstones {
+		if t.Min <= timestamp && t.Max >= timestamp {
+			return false
+		}
+	}
+	return true
 }
 
 func (d *indirectIndex) Type(key string) (byte, error) {
@@ -592,7 +753,7 @@ func (d *indirectIndex) KeyRange() (string, string) {
 	return d.minKey, d.maxKey
 }
 
-func (d *indirectIndex) TimeRange() (time.Time, time.Time) {
+func (d *indirectIndex) TimeRange() (int64, int64) {
 	return d.minTime, d.maxTime
 }
 
@@ -625,30 +786,24 @@ func (d *indirectIndex) UnmarshalBinary(b []byte) error {
 	for i < int32(len(b)) {
 		d.offsets = append(d.offsets, i)
 
-		keyLen := btou16(b[i : i+2])
-		// Skip to the start of the key
-		i += 2
+		// Skip to the start of the values
+		// key length value (2) + type (1) + length of key
+		i += 3 + int32(binary.BigEndian.Uint16(b[i:i+2]))
 
-		// Skip over the key
-		i += int32(keyLen)
-
-		// Skip over the type
-		i++
-
-		// 2 byte count of index entries
-		count := int32(btou16(b[i : i+indexCountSize]))
+		// count of index entries
+		count := int32(binary.BigEndian.Uint16(b[i : i+indexCountSize]))
 		i += indexCountSize
 
 		// Find the min time for the block
-		minT := int64(btou64(b[int(i) : int(i)+8]))
+		minT := int64(binary.BigEndian.Uint64(b[i : i+8]))
 		if minT < minTime {
 			minTime = minT
 		}
 
-		i += int32((count - 1) * indexEntrySize)
+		i += (count - 1) * indexEntrySize
 
 		// Find the max time for the block
-		maxT := int64(btou64(b[int(i)+8 : int(i)+16]))
+		maxT := int64(binary.BigEndian.Uint64(b[i+8 : i+16]))
 		if maxT > maxTime {
 			maxTime = maxT
 		}
@@ -670,8 +825,8 @@ func (d *indirectIndex) UnmarshalBinary(b []byte) error {
 	}
 	d.maxKey = string(key)
 
-	d.minTime = time.Unix(0, minTime)
-	d.maxTime = time.Unix(0, maxTime)
+	d.minTime = minTime
+	d.maxTime = maxTime
 
 	return nil
 }
@@ -683,169 +838,6 @@ func (d *indirectIndex) Size() uint32 {
 	return uint32(len(d.b))
 }
 
-// fileAccessor is file IO based block accessor.  It provides access to blocks
-// using a file IO based approach (seek, read, etc.)
-type fileAccessor struct {
-	mu    sync.Mutex
-	r     io.ReadSeeker
-	index TSMIndex
-}
-
-func (f *fileAccessor) init() (TSMIndex, error) {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-
-	// Verify it's a TSM file of the right version
-	if err := verifyVersion(f.r); err != nil {
-		return nil, err
-	}
-
-	// Current the readers size
-	size, err := f.r.Seek(0, os.SEEK_END)
-	if err != nil {
-		return nil, fmt.Errorf("init: failed to seek: %v", err)
-	}
-
-	indexEnd := size - 8
-
-	// Seek to index location pointer
-	_, err = f.r.Seek(-8, os.SEEK_END)
-	if err != nil {
-		return nil, fmt.Errorf("init: failed to seek to index ptr: %v", err)
-	}
-
-	// Read the absolute position of the start of the index
-	b := make([]byte, 8)
-	_, err = f.r.Read(b)
-	if err != nil {
-		return nil, fmt.Errorf("init: failed to read index ptr: %v", err)
-
-	}
-
-	indexStart := int64(btou64(b))
-
-	_, err = f.r.Seek(indexStart, os.SEEK_SET)
-	if err != nil {
-		return nil, fmt.Errorf("init: failed to seek to index: %v", err)
-	}
-
-	b = make([]byte, indexEnd-indexStart)
-	f.index = &directIndex{
-		blocks: map[string]*indexEntries{},
-	}
-	_, err = f.r.Read(b)
-	if err != nil {
-		return nil, fmt.Errorf("init: read index: %v", err)
-	}
-
-	if err := f.index.UnmarshalBinary(b); err != nil {
-		return nil, fmt.Errorf("init: unmarshal error: %v", err)
-	}
-
-	return f.index, nil
-}
-
-func (f *fileAccessor) read(key string, timestamp time.Time) ([]Value, error) {
-	entry := f.index.Entry(key, timestamp)
-
-	if entry == nil {
-		return nil, nil
-	}
-
-	return f.readBlock(entry, nil)
-}
-
-func (f *fileAccessor) readBlock(entry *IndexEntry, values []Value) ([]Value, error) {
-	b, err := f.readBytes(entry, nil)
-	if err != nil {
-		return nil, err
-	}
-
-	//TODO: Validate checksum
-	values, err = DecodeBlock(b, values)
-	if err != nil {
-		return nil, err
-	}
-
-	return values, nil
-}
-
-func (f *fileAccessor) readBytes(entry *IndexEntry, b []byte) ([]byte, error) {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-
-	// TODO: remove this allocation
-	if b == nil {
-		b = make([]byte, entry.Size)
-	}
-	_, err := f.r.Seek(entry.Offset, os.SEEK_SET)
-	if err != nil {
-		return nil, err
-	}
-
-	if int(entry.Size) > len(b) {
-		b = make([]byte, entry.Size)
-	}
-
-	n, err := f.r.Read(b)
-	if err != nil {
-		return nil, err
-	}
-
-	return b[4:n], nil
-}
-
-// ReadAll returns all values for a key in all blocks.
-func (f *fileAccessor) readAll(key string) ([]Value, error) {
-	var values []Value
-	blocks := f.index.Entries(key)
-	if len(blocks) == 0 {
-		return values, nil
-	}
-
-	var temp []Value
-	// TODO: we can determine the max block size when loading the file create/re-use
-	// a reader level buf then.
-	b := make([]byte, 16*1024)
-	for _, block := range blocks {
-
-		b, err := f.readBytes(block, b)
-		if err != nil {
-			return nil, err
-		}
-
-		//TODO: Validate checksum
-		temp = temp[:0]
-		temp, err = DecodeBlock(b, temp)
-		if err != nil {
-			return nil, err
-		}
-		values = append(values, temp...)
-	}
-
-	return values, nil
-}
-
-func (f *fileAccessor) path() string {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-
-	if fd, ok := f.r.(*os.File); ok {
-		return fd.Name()
-	}
-	return ""
-}
-
-func (f *fileAccessor) close() error {
-	f.mu.Lock()
-	defer f.mu.Unlock()
-
-	if c, ok := f.r.(io.Closer); ok {
-		return c.Close()
-	}
-	return nil
-}
-
 // mmapAccess is mmap based block accessor.  It access blocks through an
 // MMAP file interface.
 type mmapAccessor struct {
@@ -853,10 +845,10 @@ type mmapAccessor struct {
 
 	f     *os.File
 	b     []byte
-	index TSMIndex
+	index *indirectIndex
 }
 
-func (m *mmapAccessor) init() (TSMIndex, error) {
+func (m *mmapAccessor) init() (*indirectIndex, error) {
 	m.mu.Lock()
 	defer m.mu.Unlock()
 
@@ -881,7 +873,7 @@ func (m *mmapAccessor) init() (TSMIndex, error) {
 	}
 
 	indexOfsPos := len(m.b) - 8
-	indexStart := btou64(m.b[indexOfsPos : indexOfsPos+8])
+	indexStart := binary.BigEndian.Uint64(m.b[indexOfsPos : indexOfsPos+8])
 
 	m.index = NewIndirectIndex()
 	if err := m.index.UnmarshalBinary(m.b[indexStart:indexOfsPos]); err != nil {
@@ -891,7 +883,7 @@ func (m *mmapAccessor) init() (TSMIndex, error) {
 	return m.index, nil
 }
 
-func (m *mmapAccessor) read(key string, timestamp time.Time) ([]Value, error) {
+func (m *mmapAccessor) read(key string, timestamp int64) ([]Value, error) {
 	entry := m.index.Entry(key, timestamp)
 	if entry == nil {
 		return nil, nil
@@ -917,7 +909,7 @@ func (m *mmapAccessor) readBlock(entry *IndexEntry, values []Value) ([]Value, er
 	return values, nil
 }
 
-func (m *mmapAccessor) readBytes(entry *IndexEntry, b []byte) ([]byte, error) {
+func (m *mmapAccessor) readFloatBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *FloatDecoder, values *[]FloatValue) ([]FloatValue, error) {
 	m.mu.RLock()
 	defer m.mu.RUnlock()
 
@@ -925,8 +917,73 @@ func (m *mmapAccessor) readBytes(entry *IndexEntry, b []byte) ([]byte, error) {
 		return nil, ErrTSMClosed
 	}
 
+	//TODO: Validate checksum
+	a, err := DecodeFloatBlock(m.b[entry.Offset+4:entry.Offset+int64(entry.Size)], tdec, vdec, values)
+	if err != nil {
+		return nil, err
+	}
+
+	return a, nil
+}
+
+func (m *mmapAccessor) readIntegerBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *IntegerDecoder, values *[]IntegerValue) ([]IntegerValue, error) {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	if int64(len(m.b)) < entry.Offset+int64(entry.Size) {
+		return nil, ErrTSMClosed
+	}
+	//TODO: Validate checksum
+	a, err := DecodeIntegerBlock(m.b[entry.Offset+4:entry.Offset+int64(entry.Size)], tdec, vdec, values)
+	if err != nil {
+		return nil, err
+	}
+
+	return a, nil
+}
+
+func (m *mmapAccessor) readStringBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *StringDecoder, values *[]StringValue) ([]StringValue, error) {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	if int64(len(m.b)) < entry.Offset+int64(entry.Size) {
+		return nil, ErrTSMClosed
+	}
+	//TODO: Validate checksum
+	a, err := DecodeStringBlock(m.b[entry.Offset+4:entry.Offset+int64(entry.Size)], tdec, vdec, values)
+	if err != nil {
+		return nil, err
+	}
+
+	return a, nil
+}
+
+func (m *mmapAccessor) readBooleanBlock(entry *IndexEntry, tdec *TimeDecoder, vdec *BooleanDecoder, values *[]BooleanValue) ([]BooleanValue, error) {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	if int64(len(m.b)) < entry.Offset+int64(entry.Size) {
+		return nil, ErrTSMClosed
+	}
+	//TODO: Validate checksum
+	a, err := DecodeBooleanBlock(m.b[entry.Offset+4:entry.Offset+int64(entry.Size)], tdec, vdec, values)
+	if err != nil {
+		return nil, err
+	}
+
+	return a, nil
+}
+
+func (m *mmapAccessor) readBytes(entry *IndexEntry, b []byte) (uint32, []byte, error) {
+	m.mu.RLock()
+	defer m.mu.RUnlock()
+
+	if int64(len(m.b)) < entry.Offset+int64(entry.Size) {
+		return 0, nil, ErrTSMClosed
+	}
+
 	// return the bytes after the 4 byte checksum
-	return m.b[entry.Offset+4 : entry.Offset+int64(entry.Size)], nil
+	return binary.BigEndian.Uint32(m.b[entry.Offset : entry.Offset+4]), m.b[entry.Offset+4 : entry.Offset+int64(entry.Size)], nil
 }
 
 // ReadAll returns all values for a key in all blocks.
@@ -936,6 +993,8 @@ func (m *mmapAccessor) readAll(key string) ([]Value, error) {
 		return nil, nil
 	}
 
+	tombstones := m.index.TombstoneRange(key)
+
 	m.mu.RLock()
 	defer m.mu.RUnlock()
 
@@ -943,6 +1002,18 @@ func (m *mmapAccessor) readAll(key string) ([]Value, error) {
 	var err error
 	var values []Value
 	for _, block := range blocks {
+		var skip bool
+		for _, t := range tombstones {
+			// Should we skip this block because it contains points that have been deleted
+			if t.Min <= block.MinTime && t.Max >= block.MaxTime {
+				skip = true
+				break
+			}
+		}
+
+		if skip {
+			continue
+		}
 		//TODO: Validate checksum
 		temp = temp[:0]
 		// The +4 is the 4 byte checksum length
@@ -950,6 +1021,12 @@ func (m *mmapAccessor) readAll(key string) ([]Value, error) {
 		if err != nil {
 			return nil, err
 		}
+
+		// Filter out any values that were deleted
+		for _, t := range tombstones {
+			temp = Values(temp).Filter(t.Min, t.Max)
+		}
+
 		values = append(values, temp...)
 	}
 
@@ -982,56 +1059,44 @@ func (m *mmapAccessor) close() error {
 
 type indexEntries struct {
 	Type    byte
-	entries []*IndexEntry
+	entries []IndexEntry
 }
 
 func (a *indexEntries) Len() int      { return len(a.entries) }
 func (a *indexEntries) Swap(i, j int) { a.entries[i], a.entries[j] = a.entries[j], a.entries[i] }
 func (a *indexEntries) Less(i, j int) bool {
-	return a.entries[i].MinTime.UnixNano() < a.entries[j].MinTime.UnixNano()
+	return a.entries[i].MinTime < a.entries[j].MinTime
 }
 
-func (a *indexEntries) Append(entry ...*IndexEntry) {
-	a.entries = append(a.entries, entry...)
-}
+func (a *indexEntries) MarshalBinary() ([]byte, error) {
+	buf := make([]byte, len(a.entries)*indexEntrySize)
 
-func (a *indexEntries) MarshalBinary() (b []byte, err error) {
-	for _, entry := range a.entries {
-		b = append(b, u64tob(uint64(entry.MinTime.UnixNano()))...)
-		b = append(b, u64tob(uint64(entry.MaxTime.UnixNano()))...)
-		b = append(b, u64tob(uint64(entry.Offset))...)
-		b = append(b, u32tob(entry.Size)...)
+	for i, entry := range a.entries {
+		entry.AppendTo(buf[indexEntrySize*i:])
 	}
-	return b, nil
+
+	return buf, nil
 }
 
-func (a *indexEntries) Write(w io.Writer) error {
+func (a *indexEntries) WriteTo(w io.Writer) (total int64, err error) {
+	var buf [indexEntrySize]byte
+	var n int
+
 	for _, entry := range a.entries {
-		_, err := w.Write(u64tob(uint64(entry.MinTime.UnixNano())))
+		entry.AppendTo(buf[:])
+		n, err = w.Write(buf[:])
+		total += int64(n)
 		if err != nil {
-			return err
-		}
-		_, err = w.Write(u64tob(uint64(entry.MaxTime.UnixNano())))
-		if err != nil {
-			return err
-		}
-
-		_, err = w.Write(u64tob(uint64(entry.Offset)))
-		if err != nil {
-			return err
-		}
-
-		_, err = w.Write(u32tob(entry.Size))
-		if err != nil {
-			return err
+			return total, err
 		}
 	}
-	return nil
+
+	return total, nil
 }
 
 func readKey(b []byte) (n int, key []byte, err error) {
 	// 2 byte size of key
-	n, size := 2, int(btou16(b[:2]))
+	n, size := 2, int(binary.BigEndian.Uint16(b[:2]))
 
 	// N byte key
 	key = b[n : n+size]
@@ -1040,25 +1105,22 @@ func readKey(b []byte) (n int, key []byte, err error) {
 	return
 }
 
-func readEntries(b []byte) (n int, entries *indexEntries, err error) {
+func readEntries(b []byte, entries *indexEntries) (n int, err error) {
 	// 1 byte block type
-	blockType := b[n]
-	entries = &indexEntries{
-		Type:    blockType,
-		entries: []*IndexEntry{},
-	}
+	entries.Type = b[n]
 	n++
 
 	// 2 byte count of index entries
-	count := int(btou16(b[n : n+indexCountSize]))
+	count := int(binary.BigEndian.Uint16(b[n : n+indexCountSize]))
 	n += indexCountSize
 
+	entries.entries = make([]IndexEntry, count)
 	for i := 0; i < count; i++ {
-		ie := &IndexEntry{}
+		var ie IndexEntry
 		if err := ie.UnmarshalBinary(b[i*indexEntrySize+indexCountSize+indexTypeSize : i*indexEntrySize+indexCountSize+indexEntrySize+indexTypeSize]); err != nil {
-			return 0, nil, fmt.Errorf("readEntries: unmarshal error: %v", err)
+			return 0, fmt.Errorf("readEntries: unmarshal error: %v", err)
 		}
-		entries.Append(ie)
+		entries.entries[i] = ie
 		n += indexEntrySize
 	}
 	return
