@@ -1,23 +1,25 @@
 package tsm1_test
 
 import (
-	"bytes"
 	"fmt"
+	"math"
 	"os"
 	"testing"
-	"time"
 
-	"github.com/influxdb/influxdb/tsdb/engine/tsm1"
+	"github.com/influxdata/influxdb/tsdb/engine/tsm1"
 )
 
 func TestTSMReader_Type(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), int64(1))}
+	values := []tsm1.Value{tsm1.NewValue(0, int64(1))}
 	if err := w.Write("cpu", values); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 
@@ -30,7 +32,11 @@ func TestTSMReader_Type(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error opening: %v", err)
+	}
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -40,7 +46,7 @@ func TestTSMReader_Type(t *testing.T) {
 		fatal(t, "reading type", err)
 	}
 
-	if got, exp := typ, tsm1.BlockInt64; got != exp {
+	if got, exp := typ, tsm1.BlockInteger; got != exp {
 		t.Fatalf("type mismatch: got %v, exp %v", got, exp)
 	}
 }
@@ -61,16 +67,16 @@ func TestTSMReader_MMAP_ReadAll(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"float", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), 1.0)},
+			tsm1.NewValue(1, 1.0)},
 		},
 		{"int", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), int64(1))},
+			tsm1.NewValue(1, int64(1))},
 		},
 		{"bool", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), true)},
+			tsm1.NewValue(1, true)},
 		},
 		{"string", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), "foo")},
+			tsm1.NewValue(1, "foo")},
 		},
 	}
 
@@ -93,10 +99,7 @@ func TestTSMReader_MMAP_ReadAll(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -142,16 +145,16 @@ func TestTSMReader_MMAP_Read(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"float", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), 1.0)},
+			tsm1.NewValue(1, 1.0)},
 		},
 		{"int", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), int64(1))},
+			tsm1.NewValue(1, int64(1))},
 		},
 		{"bool", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), true)},
+			tsm1.NewValue(1, true)},
 		},
 		{"string", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), "foo")},
+			tsm1.NewValue(1, "foo")},
 		},
 	}
 	for _, d := range data {
@@ -173,10 +176,7 @@ func TestTSMReader_MMAP_Read(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -184,7 +184,7 @@ func TestTSMReader_MMAP_Read(t *testing.T) {
 
 	var count int
 	for _, d := range data {
-		readValues, err := r.Read(d.key, d.values[0].Time())
+		readValues, err := r.Read(d.key, d.values[0].UnixNano())
 		if err != nil {
 			t.Fatalf("unexpected error readin: %v", err)
 		}
@@ -222,16 +222,16 @@ func TestTSMReader_MMAP_Keys(t *testing.T) {
 		values []tsm1.Value
 	}{
 		{"float", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), 1.0)},
+			tsm1.NewValue(1, 1.0)},
 		},
 		{"int", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), int64(1))},
+			tsm1.NewValue(1, int64(1))},
 		},
 		{"bool", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), true)},
+			tsm1.NewValue(1, true)},
 		},
 		{"string", []tsm1.Value{
-			tsm1.NewValue(time.Unix(1, 0), "foo")},
+			tsm1.NewValue(1, "foo")},
 		},
 	}
 
@@ -254,10 +254,7 @@ func TestTSMReader_MMAP_Keys(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -265,7 +262,7 @@ func TestTSMReader_MMAP_Keys(t *testing.T) {
 
 	var count int
 	for _, d := range data {
-		readValues, err := r.Read(d.key, d.values[0].Time())
+		readValues, err := r.Read(d.key, d.values[0].UnixNano())
 		if err != nil {
 			t.Fatalf("unexpected error readin: %v", err)
 		}
@@ -298,7 +295,7 @@ func TestTSMReader_MMAP_Tombstone(t *testing.T) {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), 1.0)}
+	values := []tsm1.Value{tsm1.NewValue(0, 1.0)}
 	if err := w.Write("cpu", values); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 	}
@@ -320,10 +317,7 @@ func TestTSMReader_MMAP_Tombstone(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -332,20 +326,278 @@ func TestTSMReader_MMAP_Tombstone(t *testing.T) {
 		t.Fatalf("unexpected error deleting: %v", err)
 	}
 
-	r, err = tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err = tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
 	defer r.Close()
 
-	if got, exp := len(r.Keys()), 1; got != exp {
+	if got, exp := r.KeyCount(), 1; got != exp {
 		t.Fatalf("key length mismatch: got %v, exp %v", got, exp)
 	}
 }
 
+func TestTSMReader_MMAP_TombstoneRange(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	expValues := []tsm1.Value{
+		tsm1.NewValue(1, 1.0),
+		tsm1.NewValue(2, 2.0),
+		tsm1.NewValue(3, 3.0),
+	}
+	if err := w.Write("cpu", expValues); err != nil {
+		t.Fatalf("unexpected error writing: %v", err)
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(f)
+	if err != nil {
+		t.Fatalf("unexpected error created reader: %v", err)
+	}
+
+	if err := r.DeleteRange([]string{"cpu"}, 2, math.MaxInt64); err != nil {
+		t.Fatalf("unexpected error deleting: %v", err)
+	}
+	defer r.Close()
+
+	if got, exp := r.ContainsValue("cpu", 1), true; got != exp {
+		t.Fatalf("ContainsValue mismatch: got %v, exp %v", got, exp)
+	}
+
+	if got, exp := r.ContainsValue("cpu", 3), false; got != exp {
+		t.Fatalf("ContainsValue mismatch: got %v, exp %v", got, exp)
+	}
+
+	values, err := r.ReadAll("cpu")
+	if err != nil {
+		t.Fatalf("unexpected error reading all: %v", err)
+	}
+
+	if got, exp := len(values), 1; got != exp {
+		t.Fatalf("values length mismatch: got %v, exp %v", got, exp)
+	}
+
+	if got, exp := values[0].String(), expValues[0].String(); got != exp {
+		t.Fatalf("value mismatch: got %v, exp %v", got, exp)
+	}
+}
+
+func TestTSMReader_MMAP_TombstoneFullRange(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	expValues := []tsm1.Value{
+		tsm1.NewValue(1, 1.0),
+		tsm1.NewValue(2, 2.0),
+		tsm1.NewValue(3, 3.0),
+	}
+	if err := w.Write("cpu", expValues); err != nil {
+		t.Fatalf("unexpected error writing: %v", err)
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(f)
+	if err != nil {
+		t.Fatalf("unexpected error created reader: %v", err)
+	}
+
+	if err := r.DeleteRange([]string{"cpu"}, math.MinInt64, math.MaxInt64); err != nil {
+		t.Fatalf("unexpected error deleting: %v", err)
+	}
+	defer r.Close()
+
+	values, err := r.ReadAll("cpu")
+	if err != nil {
+		t.Fatalf("unexpected error reading all: %v", err)
+	}
+
+	if got, exp := len(values), 0; got != exp {
+		t.Fatalf("values length mismatch: got %v, exp %v", got, exp)
+	}
+}
+
+func TestTSMReader_MMAP_TombstoneMultipleRanges(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	expValues := []tsm1.Value{
+		tsm1.NewValue(1, 1.0),
+		tsm1.NewValue(2, 2.0),
+		tsm1.NewValue(3, 3.0),
+		tsm1.NewValue(4, 4.0),
+		tsm1.NewValue(5, 5.0),
+	}
+	if err := w.Write("cpu", expValues); err != nil {
+		t.Fatalf("unexpected error writing: %v", err)
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(f)
+	if err != nil {
+		t.Fatalf("unexpected error created reader: %v", err)
+	}
+	defer r.Close()
+
+	if err := r.DeleteRange([]string{"cpu"}, 2, 2); err != nil {
+		t.Fatalf("unexpected error deleting: %v", err)
+	}
+
+	if err := r.DeleteRange([]string{"cpu"}, 4, 4); err != nil {
+		t.Fatalf("unexpected error deleting: %v", err)
+	}
+
+	values, err := r.ReadAll("cpu")
+	if err != nil {
+		t.Fatalf("unexpected error reading all: %v", err)
+	}
+
+	if got, exp := len(values), 3; got != exp {
+		t.Fatalf("values length mismatch: got %v, exp %v", got, exp)
+	}
+}
+
+func TestTSMReader_MMAP_TombstoneOutsideRange(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	cpuValues := []tsm1.Value{
+		tsm1.NewValue(1, 1.0),
+		tsm1.NewValue(2, 2.0),
+		tsm1.NewValue(3, 3.0),
+	}
+	if err := w.Write("cpu", cpuValues); err != nil {
+		t.Fatalf("unexpected error writing: %v", err)
+	}
+
+	memValues := []tsm1.Value{
+		tsm1.NewValue(1, 1.0),
+		tsm1.NewValue(2, 2.0),
+		tsm1.NewValue(30, 3.0),
+	}
+	if err := w.Write("mem", memValues); err != nil {
+		t.Fatalf("unexpected error writing: %v", err)
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(f)
+	if err != nil {
+		t.Fatalf("unexpected error created reader: %v", err)
+	}
+
+	if err := r.DeleteRange([]string{"cpu", "mem"}, 5, math.MaxInt64); err != nil {
+		t.Fatalf("unexpected error deleting: %v", err)
+	}
+	defer r.Close()
+
+	if got, exp := r.KeyCount(), 2; got != exp {
+		t.Fatalf("key count mismatch: got %v, exp %v", got, exp)
+	}
+
+	if got, exp := len(r.TombstoneRange("cpu")), 0; got != exp {
+		t.Fatalf("tombstone range mismatch: got %v, exp %v", got, exp)
+	}
+
+	values, err := r.ReadAll("cpu")
+	if err != nil {
+		t.Fatalf("unexpected error reading all: %v", err)
+	}
+
+	if got, exp := len(values), len(cpuValues); got != exp {
+		t.Fatalf("values length mismatch: got %v, exp %v", got, exp)
+	}
+
+	if got, exp := len(r.TombstoneRange("mem")), 1; got != exp {
+		t.Fatalf("tombstone range mismatch: got %v, exp %v", got, exp)
+	}
+
+	values, err = r.ReadAll("mem")
+	if err != nil {
+		t.Fatalf("unexpected error reading all: %v", err)
+	}
+
+	if got, exp := len(values), len(memValues[:2]); got != exp {
+		t.Fatalf("values length mismatch: got %v, exp %v", got, exp)
+	}
+
+}
+
 func TestTSMReader_MMAP_Stats(t *testing.T) {
 	dir := MustTempDir()
 	defer os.RemoveAll(dir)
@@ -357,12 +609,12 @@ func TestTSMReader_MMAP_Stats(t *testing.T) {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values1 := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), 1.0)}
+	values1 := []tsm1.Value{tsm1.NewValue(0, 1.0)}
 	if err := w.Write("cpu", values1); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 	}
 
-	values2 := []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), 1.0)}
+	values2 := []tsm1.Value{tsm1.NewValue(1, 1.0)}
 	if err := w.Write("mem", values2); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 	}
@@ -380,10 +632,7 @@ func TestTSMReader_MMAP_Stats(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -398,15 +647,15 @@ func TestTSMReader_MMAP_Stats(t *testing.T) {
 		t.Fatalf("max key mismatch: got %v, exp %v", got, exp)
 	}
 
-	if got, exp := stats.MinTime, values1[0].Time(); got != exp {
+	if got, exp := stats.MinTime, values1[0].UnixNano(); got != exp {
 		t.Fatalf("min time mismatch: got %v, exp %v", got, exp)
 	}
 
-	if got, exp := stats.MaxTime, values2[0].Time(); got != exp {
+	if got, exp := stats.MaxTime, values2[0].UnixNano(); got != exp {
 		t.Fatalf("max time mismatch: got %v, exp %v", got, exp)
 	}
 
-	if got, exp := len(r.Keys()), 2; got != exp {
+	if got, exp := r.KeyCount(), 2; got != exp {
 		t.Fatalf("key length mismatch: got %v, exp %v", got, exp)
 	}
 }
@@ -421,20 +670,17 @@ func TestTSMReader_VerifiesFileType(t *testing.T) {
 	// write some garbage
 	f.Write([]byte{0x23, 0xac, 0x99, 0x22, 0x77, 0x23, 0xac, 0x99, 0x22, 0x77, 0x23, 0xac, 0x99, 0x22, 0x77, 0x23, 0xac, 0x99, 0x22, 0x77})
 
-	_, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	_, err := tsm1.NewTSMReader(f)
 	if err == nil {
 		t.Fatal("expected error trying to open non-tsm file")
 	}
 }
 
 func TestIndirectIndex_Entries(t *testing.T) {
-	index := tsm1.NewDirectIndex()
-	index.Add("cpu", tsm1.BlockFloat64, time.Unix(0, 0), time.Unix(1, 0), 10, 100)
-	index.Add("cpu", tsm1.BlockFloat64, time.Unix(2, 0), time.Unix(3, 0), 20, 200)
-	index.Add("mem", tsm1.BlockFloat64, time.Unix(0, 0), time.Unix(1, 0), 10, 100)
+	index := tsm1.NewIndexWriter()
+	index.Add("cpu", tsm1.BlockFloat64, 0, 1, 10, 100)
+	index.Add("cpu", tsm1.BlockFloat64, 2, 3, 20, 200)
+	index.Add("mem", tsm1.BlockFloat64, 0, 1, 10, 100)
 
 	b, err := index.MarshalBinary()
 	if err != nil {
@@ -473,9 +719,9 @@ func TestIndirectIndex_Entries(t *testing.T) {
 }
 
 func TestIndirectIndex_Entries_NonExistent(t *testing.T) {
-	index := tsm1.NewDirectIndex()
-	index.Add("cpu", tsm1.BlockFloat64, time.Unix(0, 0), time.Unix(1, 0), 10, 100)
-	index.Add("cpu", tsm1.BlockFloat64, time.Unix(2, 0), time.Unix(3, 0), 20, 200)
+	index := tsm1.NewIndexWriter()
+	index.Add("cpu", tsm1.BlockFloat64, 0, 1, 10, 100)
+	index.Add("cpu", tsm1.BlockFloat64, 2, 3, 20, 200)
 
 	b, err := index.MarshalBinary()
 	if err != nil {
@@ -498,9 +744,9 @@ func TestIndirectIndex_Entries_NonExistent(t *testing.T) {
 }
 
 func TestIndirectIndex_MaxBlocks(t *testing.T) {
-	index := tsm1.NewDirectIndex()
+	index := tsm1.NewIndexWriter()
 	for i := 0; i < 1<<16; i++ {
-		index.Add("cpu", tsm1.BlockFloat64, time.Unix(0, 0), time.Unix(1, 0), 10, 20)
+		index.Add("cpu", tsm1.BlockFloat64, 0, 1, 10, 20)
 	}
 
 	if _, err := index.MarshalBinary(); err == nil {
@@ -511,8 +757,8 @@ func TestIndirectIndex_MaxBlocks(t *testing.T) {
 }
 
 func TestIndirectIndex_Type(t *testing.T) {
-	index := tsm1.NewDirectIndex()
-	index.Add("cpu", tsm1.BlockInt64, time.Unix(0, 0), time.Unix(1, 0), 10, 20)
+	index := tsm1.NewIndexWriter()
+	index.Add("cpu", tsm1.BlockInteger, 0, 1, 10, 20)
 
 	b, err := index.MarshalBinary()
 
@@ -526,16 +772,16 @@ func TestIndirectIndex_Type(t *testing.T) {
 		fatal(t, "reading type", err)
 	}
 
-	if got, exp := typ, tsm1.BlockInt64; got != exp {
+	if got, exp := typ, tsm1.BlockInteger; got != exp {
 		t.Fatalf("type mismatch: got %v, exp %v", got, exp)
 	}
 }
 
 func TestIndirectIndex_Keys(t *testing.T) {
-	index := tsm1.NewDirectIndex()
-	index.Add("cpu", tsm1.BlockFloat64, time.Unix(0, 0), time.Unix(1, 0), 10, 20)
-	index.Add("mem", tsm1.BlockFloat64, time.Unix(0, 0), time.Unix(1, 0), 10, 20)
-	index.Add("cpu", tsm1.BlockFloat64, time.Unix(1, 0), time.Unix(2, 0), 20, 30)
+	index := tsm1.NewIndexWriter()
+	index.Add("cpu", tsm1.BlockFloat64, 0, 1, 10, 20)
+	index.Add("mem", tsm1.BlockFloat64, 0, 1, 10, 20)
+	index.Add("cpu", tsm1.BlockFloat64, 1, 2, 20, 30)
 
 	keys := index.Keys()
 
@@ -555,13 +801,16 @@ func TestIndirectIndex_Keys(t *testing.T) {
 }
 
 func TestBlockIterator_Single(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), int64(1))}
+	values := []tsm1.Value{tsm1.NewValue(0, int64(1))}
 	if err := w.Write("cpu", values); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 
@@ -574,7 +823,12 @@ func TestBlockIterator_Single(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error opening: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -582,7 +836,7 @@ func TestBlockIterator_Single(t *testing.T) {
 	var count int
 	iter := r.BlockIterator()
 	for iter.Next() {
-		key, minTime, maxTime, buf, err := iter.Read()
+		key, minTime, maxTime, _, buf, err := iter.Read()
 
 		if err != nil {
 			t.Fatalf("unexpected error creating iterator: %v", err)
@@ -592,11 +846,11 @@ func TestBlockIterator_Single(t *testing.T) {
 			t.Fatalf("key mismatch: got %v, exp %v", got, exp)
 		}
 
-		if got, exp := minTime, time.Unix(0, 0); got != exp {
+		if got, exp := minTime, int64(0); got != exp {
 			t.Fatalf("min time mismatch: got %v, exp %v", got, exp)
 		}
 
-		if got, exp := maxTime, time.Unix(0, 0); got != exp {
+		if got, exp := maxTime, int64(0); got != exp {
 			t.Fatalf("max time mismatch: got %v, exp %v", got, exp)
 		}
 
@@ -613,18 +867,21 @@ func TestBlockIterator_Single(t *testing.T) {
 }
 
 func TestBlockIterator_MultipleBlocks(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values1 := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), int64(1))}
+	values1 := []tsm1.Value{tsm1.NewValue(0, int64(1))}
 	if err := w.Write("cpu", values1); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 	}
 
-	values2 := []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), int64(2))}
+	values2 := []tsm1.Value{tsm1.NewValue(1, int64(2))}
 	if err := w.Write("cpu", values2); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 	}
@@ -637,7 +894,12 @@ func TestBlockIterator_MultipleBlocks(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error opening: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -647,7 +909,7 @@ func TestBlockIterator_MultipleBlocks(t *testing.T) {
 	iter := r.BlockIterator()
 	var i int
 	for iter.Next() {
-		key, minTime, maxTime, buf, err := iter.Read()
+		key, minTime, maxTime, _, buf, err := iter.Read()
 
 		if err != nil {
 			t.Fatalf("unexpected error creating iterator: %v", err)
@@ -657,11 +919,11 @@ func TestBlockIterator_MultipleBlocks(t *testing.T) {
 			t.Fatalf("key mismatch: got %v, exp %v", got, exp)
 		}
 
-		if got, exp := minTime, expData[i][0].Time(); got != exp {
+		if got, exp := minTime, expData[i][0].UnixNano(); got != exp {
 			t.Fatalf("min time mismatch: got %v, exp %v", got, exp)
 		}
 
-		if got, exp := maxTime, expData[i][0].Time(); got != exp {
+		if got, exp := maxTime, expData[i][0].UnixNano(); got != exp {
 			t.Fatalf("max time mismatch: got %v, exp %v", got, exp)
 		}
 
@@ -677,18 +939,22 @@ func TestBlockIterator_MultipleBlocks(t *testing.T) {
 		t.Fatalf("value count mismatch: got %v, exp %v", got, exp)
 	}
 }
+
 func TestBlockIterator_Sorted(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
 	values := map[string][]tsm1.Value{
-		"mem":  []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), int64(1))},
-		"cpu":  []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), float64(2))},
-		"disk": []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), true)},
-		"load": []tsm1.Value{tsm1.NewValue(time.Unix(1, 0), "string")},
+		"mem":  []tsm1.Value{tsm1.NewValue(0, int64(1))},
+		"cpu":  []tsm1.Value{tsm1.NewValue(1, float64(2))},
+		"disk": []tsm1.Value{tsm1.NewValue(1, true)},
+		"load": []tsm1.Value{tsm1.NewValue(1, "string")},
 	}
 
 	for k, v := range values {
@@ -706,7 +972,12 @@ func TestBlockIterator_Sorted(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error opening: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -715,7 +986,7 @@ func TestBlockIterator_Sorted(t *testing.T) {
 	iter := r.BlockIterator()
 	var lastKey string
 	for iter.Next() {
-		key, _, _, buf, err := iter.Read()
+		key, _, _, _, buf, err := iter.Read()
 
 		if key < lastKey {
 			t.Fatalf("keys not sorted: got %v, last %v", key, lastKey)
@@ -751,7 +1022,7 @@ func TestIndirectIndex_UnmarshalBinary_BlockCountOverflow(t *testing.T) {
 	}
 
 	for i := 0; i < 3280; i++ {
-		w.Write("cpu", []tsm1.Value{tsm1.NewValue(time.Unix(int64(i), 0), float64(i))})
+		w.Write("cpu", []tsm1.Value{tsm1.NewValue(int64(i), float64(i))})
 	}
 
 	if err := w.WriteIndex(); err != nil {
@@ -767,10 +1038,7 @@ func TestIndirectIndex_UnmarshalBinary_BlockCountOverflow(t *testing.T) {
 		t.Fatalf("unexpected error open file: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReaderWithOptions(
-		tsm1.TSMReaderOptions{
-			MMAPFile: f,
-		})
+	r, err := tsm1.NewTSMReader(f)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -778,13 +1046,16 @@ func TestIndirectIndex_UnmarshalBinary_BlockCountOverflow(t *testing.T) {
 }
 
 func TestCompacted_NotFull(t *testing.T) {
-	var b bytes.Buffer
-	w, err := tsm1.NewTSMWriter(&b)
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+
+	w, err := tsm1.NewTSMWriter(f)
 	if err != nil {
 		t.Fatalf("unexpected error creating writer: %v", err)
 	}
 
-	values := []tsm1.Value{tsm1.NewValue(time.Unix(0, 0), 1.0)}
+	values := []tsm1.Value{tsm1.NewValue(0, 1.0)}
 	if err := w.Write("cpu", values); err != nil {
 		t.Fatalf("unexpected error writing: %v", err)
 
@@ -797,7 +1068,12 @@ func TestCompacted_NotFull(t *testing.T) {
 		t.Fatalf("unexpected error closing: %v", err)
 	}
 
-	r, err := tsm1.NewTSMReader(bytes.NewReader(b.Bytes()))
+	fd, err := os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(fd)
 	if err != nil {
 		t.Fatalf("unexpected error created reader: %v", err)
 	}
@@ -807,7 +1083,7 @@ func TestCompacted_NotFull(t *testing.T) {
 		t.Fatalf("expected next, got false")
 	}
 
-	_, _, _, block, err := iter.Read()
+	_, _, _, _, block, err := iter.Read()
 	if err != nil {
 		t.Fatalf("unexpected error reading block: %v", err)
 	}
@@ -817,10 +1093,165 @@ func TestCompacted_NotFull(t *testing.T) {
 	}
 }
 
+func TestTSMReader_File_ReadAll(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	var data = []struct {
+		key    string
+		values []tsm1.Value
+	}{
+		{"float", []tsm1.Value{
+			tsm1.NewValue(1, 1.0)},
+		},
+		{"int", []tsm1.Value{
+			tsm1.NewValue(1, int64(1))},
+		},
+		{"bool", []tsm1.Value{
+			tsm1.NewValue(1, true)},
+		},
+		{"string", []tsm1.Value{
+			tsm1.NewValue(1, "foo")},
+		},
+	}
+
+	for _, d := range data {
+		if err := w.Write(d.key, d.values); err != nil {
+			t.Fatalf("unexpected error writing: %v", err)
+		}
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(f)
+	if err != nil {
+		t.Fatalf("unexpected error created reader: %v", err)
+	}
+	defer r.Close()
+
+	var count int
+	for _, d := range data {
+		readValues, err := r.ReadAll(d.key)
+		if err != nil {
+			t.Fatalf("unexpected error reading: %v", err)
+		}
+
+		if exp := len(d.values); exp != len(readValues) {
+			t.Fatalf("read values length mismatch: exp %v, got %v", exp, len(readValues))
+		}
+
+		for i, v := range d.values {
+			if exp, got := v.Value(), readValues[i].Value(); exp != got {
+				t.Fatalf("read value mismatch(%d): exp %v, got %d", i, v.Value(), readValues[i].Value())
+			}
+		}
+		count++
+	}
+
+	if exp, got := len(data), count; exp != got {
+		t.Fatalf("read values count mismatch: exp %v, got %v", exp, got)
+	}
+}
+
+func TestTSMReader_File_Read(t *testing.T) {
+	dir := MustTempDir()
+	defer os.RemoveAll(dir)
+	f := MustTempFile(dir)
+	defer f.Close()
+
+	w, err := tsm1.NewTSMWriter(f)
+	if err != nil {
+		t.Fatalf("unexpected error creating writer: %v", err)
+	}
+
+	var data = []struct {
+		key    string
+		values []tsm1.Value
+	}{
+		{"float", []tsm1.Value{
+			tsm1.NewValue(1, 1.0)},
+		},
+		{"int", []tsm1.Value{
+			tsm1.NewValue(1, int64(1))},
+		},
+		{"bool", []tsm1.Value{
+			tsm1.NewValue(1, true)},
+		},
+		{"string", []tsm1.Value{
+			tsm1.NewValue(1, "foo")},
+		},
+	}
+	for _, d := range data {
+		if err := w.Write(d.key, d.values); err != nil {
+			t.Fatalf("unexpected error writing: %v", err)
+		}
+	}
+
+	if err := w.WriteIndex(); err != nil {
+		t.Fatalf("unexpected error writing index: %v", err)
+	}
+
+	if err := w.Close(); err != nil {
+		t.Fatalf("unexpected error closing: %v", err)
+	}
+
+	f, err = os.Open(f.Name())
+	if err != nil {
+		t.Fatalf("unexpected error open file: %v", err)
+	}
+
+	r, err := tsm1.NewTSMReader(f)
+	if err != nil {
+		t.Fatalf("unexpected error created reader: %v", err)
+	}
+	defer r.Close()
+
+	var count int
+	for _, d := range data {
+		readValues, err := r.Read(d.key, d.values[0].UnixNano())
+		if err != nil {
+			t.Fatalf("unexpected error readin: %v", err)
+		}
+
+		if exp, got := len(d.values), len(readValues); exp != got {
+			t.Fatalf("read values length mismatch: exp %v, got %v", exp, len(readValues))
+		}
+
+		for i, v := range d.values {
+			if v.Value() != readValues[i].Value() {
+				t.Fatalf("read value mismatch(%d): exp %v, got %d", i, v.Value(), readValues[i].Value())
+			}
+		}
+		count++
+	}
+
+	if exp, got := count, len(data); exp != got {
+		t.Fatalf("read values count mismatch: exp %v, got %v", exp, got)
+	}
+}
+
 func BenchmarkIndirectIndex_UnmarshalBinary(b *testing.B) {
-	index := tsm1.NewDirectIndex()
+	index := tsm1.NewIndexWriter()
 	for i := 0; i < 100000; i++ {
-		index.Add(fmt.Sprintf("cpu-%d", i), tsm1.BlockFloat64, time.Unix(int64(i*2), 0), time.Unix(int64(i*2+1), 0), 10, 100)
+		index.Add(fmt.Sprintf("cpu-%d", i), tsm1.BlockFloat64, int64(i*2), int64(i*2+1), 10, 100)
 	}
 
 	bytes, err := index.MarshalBinary()
